/* packet-dcerpc-TKN4Int.c
 *
 * Routines for dcerpc Token Server Calls
 * Copyright 2002, Jaime Fournier <jafour1@yahoo.com>
 * This information is based off the released idl files from opengroup.
 * ftp://opengroup.org/
 *     
 * $Id: packet-dcerpc-TKN4Int.c,v 1.2 2002/01/21 07:36:33 guy Exp $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif


#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <string.h>

#include <glib.h>
#include <epan/packet.h>
#include "packet-dcerpc.h"


static int proto_TKN4Int = -1;

static gint ett_TKN4Int = -1;


static e_uuid_t uuid_TKN4Int = { 0x4d37f2dd, 0xed96, 0x0000, { 0x02, 0xc0, 0x37, 0xcf, 0x1e, 0x00, 0x00, 0x00 } };
static guint16  ver_TKN4Int = 4;
static int hf_tkn4int_tokenrevoke_rqst_colbchoice_low = -1;
static int hf_tkn4int_tokenrevoke_rqst_colbchoice_high = -1;
static int hf_tkn4int_tokenrevoke_rqst_columnb_low = -1;
static int hf_tkn4int_tokenrevoke_rqst_columnb_high = -1;
static int hf_tkn4int_tokenrevoke_rqst_colachoice_low = -1;
static int hf_tkn4int_tokenrevoke_rqst_colachoice_high = -1;
static int hf_tkn4int_tokenrevoke_rqst_endrangeext = -1;
static int hf_tkn4int_tokenrevoke_rqst_beginrangeext = -1;
static int hf_tkn4int_tokenrevoke_rqst_endrange = -1;
static int hf_tkn4int_tokenrevoke_rqst_beginrange = -1;
static int hf_tkn4int_tokenrevoke_rqst_type_low2 = -1;
static int hf_tkn4int_tokenrevoke_rqst_type_hi2 = -1;
static int hf_tkn4int_tokenrevoke_rqst_expirationtime = -1;
static int hf_tkn4int_tokenrevoke_rqst_tokenid_low2 = -1;
static int hf_tkn4int_tokenrevoke_rqst_tokenid_hi2 = -1;
static int hf_tkn4int_tokenrevoke_rqst_errorids_low = -1;
static int hf_tkn4int_tokenrevoke_rqst_errorids_high = -1;
static int hf_tkn4int_tokenrevoke_rqst_outflags = -1;
static int hf_tkn4int_tokenrevoke_rqst_flags = -1;
static int hf_tkn4int_tokenrevoke_rqst_type_low = -1;
static int hf_tkn4int_tokenrevoke_rqst_type_high = -1;
static int hf_tkn4int_tokenrevoke_rqst_tokenid_low = -1;
static int hf_tkn4int_tokenrevoke_rqst_tokenid_high = -1;
static int hf_tkn4int_tokenrevoke_rqst_unique = -1;
static int hf_tkn4int_tokenrevoke_rqst_vnode = -1;
static int hf_tkn4int_tokenrevoke_rqst_volume_low = -1;
static int hf_tkn4int_tokenrevoke_rqst_volume_high = -1;
static int hf_tkn4int_tokenrevoke_rqst_cell_high = -1;
static int hf_tkn4int_tokenrevoke_rqst_cell_low = -1;
static int hf_tkn4int_tokenrevoke_rqst_l_type = -1;
static int hf_tkn4int_tokenrevoke_rqst_l_whence = -1;
static int hf_tkn4int_tokenrevoke_rqst_l_start_pos = -1;
static int hf_tkn4int_tokenrevoke_rqst_l_end_pos = -1;
static int hf_tkn4int_tokenrevoke_rqst_l_pid = -1;
static int hf_tkn4int_tokenrevoke_rqst_l_sysid = -1;
static int hf_tkn4int_tokenrevoke_rqst_l_fstype = -1;
static int hf_tkn4int_tokenrevoke_rqst_l_start_pos_ext = -1;
static int hf_tkn4int_tokenrevoke_rqst_l_end_pos_ext = -1;

static int
tkn4int_dissect_tokenrevoke_rqst (tvbuff_t *tvb, int offset,
                                 packet_info *pinfo, proto_tree *tree,
                                 char *drep)
{
	 guint64 volume, tokenid, type, errorid;
	 guint32 volume_high, volume_low, vnode;
	 guint32 tokenid_high, tokenid_low, type_high, type_low, flags, outflags, errorids_high, errorids_low;
	 guint32 tokenid_hi2, tokenid_low2, expirationtime, type_hi2, type_low2, beginrange, endrange, beginrangeext, endrangeext;
	 guint32 colachoice_high, colachoice_low;
	 guint32 column_high, column_low, colbchoice_high, colbchoice_low, columnb_high, columnb_low;
	 guint32 l_type, l_whence, l_start_pos, l_end_pos, l_pid, l_sysid, l_fstype, l_start_pos_ext, l_end_pos_ext, inode;


	 /*
	  * AFS_BULKMAX = 32
         afsFid fid;             # useful hint 
	 */

         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_cell_high, NULL);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_cell_low, NULL);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_volume_high, &volume_high);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_volume_low, &volume_low);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_vnode, &vnode);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_unique, NULL);

	/*
         afsHyper tokenID;
         afsHyper type;          # mask 
         unsigned32 flags;       # input flags to RPC
         unsigned32 outFlags;    # output flags from RPC 
         afsHyper errorIDs;      # (mask[dude] == 1) <==> dude not revoked 
	*/

         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_tokenid_high, &tokenid_high);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_tokenid_low, &tokenid_low);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_type_high, &type_high);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_type_low, &type_low);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_flags, &flags);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_outflags, &outflags);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_errorids_high, &errorids_high);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_errorids_low, &errorids_low);

	 tokenid  = tokenid_high | tokenid_low;
	 type  = type_high | type_low;
	 errorid = errorids_high | errorids_low;

	/*
                                 # Info about the tokens to be offered for swapping
	 *                       # The first one
	 *                         
	 afsToken columnA;       # the offer 
	 */

        /*       unsigned32 tokenID_hi;
        	 unsigned32 tokenID_lo;
        	 unsigned32 expirationTime;
                 unsigned32 type_hi;
                 unsigned32 type_lo;
                 unsigned32 beginRange;
                 unsigned32 endRange;
                 unsigned32 beginRangeExt;
                 unsigned32 endRangeExt;
        */	
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_tokenid_hi2, &tokenid_hi2);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_tokenid_low2, &tokenid_low2);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_expirationtime, &expirationtime);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_type_hi2, &type_hi2);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_type_low2, &type_low2);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_beginrange, &beginrange);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_endrange, &endrange);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_beginrangeext, &beginrangeext);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_endrangeext, &endrangeext);

	/*
	 afsHyper colAChoice;    # the accepted token types 
	*/

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_colachoice_high, &colachoice_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_colachoice_low, &colachoice_low);

	/*
	   the other token to be offered
	         
         afsToken columnB;       # the offer 
	 */

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_columnb_high, &columnb_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_columnb_low, &columnb_low);

	/*
         afsHyper colBChoice;    # the accepted token types 
	 */
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_colbchoice_high, &colbchoice_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_colbchoice_low, &colbchoice_low);
        

	 
	/* afsRevokeDesc Flags*/
#define AFS_REVOKE_COL_A_VALID  0x1
#define AFS_REVOKE_COL_B_VALID  0x2
#define AFS_REVOKE_FORCED_REVOCATION  0x4
#define AFS_REVOKE_DUE_TO_GC  0x8
#define AFS_REVOKE_NO_QUEUED_ASYNC  0x10


	/*
         afsRecordLock recordLock; # the blocker's info 
	 */
        offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_l_type, &l_type);
        offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_l_whence, &l_whence);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_l_start_pos, &l_start_pos);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_l_end_pos, &l_end_pos);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_l_pid, &l_pid);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_l_sysid, &l_sysid);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_l_fstype, &l_fstype);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_l_start_pos_ext, &l_start_pos_ext);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_tkn4int_tokenrevoke_rqst_l_end_pos_ext, &l_end_pos_ext);


	volume = volume_high | volume_low;
        inode = ((volume << 16) + vnode) & 0x7fffffff;

        if (check_col(pinfo->cinfo, COL_INFO)) col_clear(pinfo->cinfo, COL_INFO);
        if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_TokenRevoke Request FSID: %llu inode: %u type: %llu tokenid: %llu errorid: %llu", volume, inode, type, tokenid, errorid);
	
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, " Flags=");

if ((flags & AFS_REVOKE_COL_A_VALID) == AFS_REVOKE_COL_A_VALID) {
	     if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":COL_A_VALID");
}
if ((flags & AFS_REVOKE_COL_B_VALID) == AFS_REVOKE_COL_B_VALID) {
	     if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":COL_B_VALID");
}
if ((flags & AFS_REVOKE_FORCED_REVOCATION) == AFS_REVOKE_FORCED_REVOCATION) {
	     if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":FORCED_REVOCATION");
}
if ((flags & AFS_REVOKE_DUE_TO_GC) == AFS_REVOKE_DUE_TO_GC) {
	     if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":DUE_TO_GC");
}
if ((flags & AFS_REVOKE_DUE_TO_GC) == AFS_REVOKE_DUE_TO_GC) {
	     if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":NO_QUEUED_ASYNC");
}

        return offset;
}


static dcerpc_sub_dissector TKN4Int_dissectors[] = {
    { 0, "TKN_Probe", NULL, NULL},
    { 1, "TKN_InitTokenState", NULL, NULL},
    { 2, "TKN_TokenRevoke", tkn4int_dissect_tokenrevoke_rqst, NULL},
    { 3, "TKN_GetCellName", NULL, NULL},
    { 4, "TKN_GetLock", NULL, NULL},
    { 5, "TKN_GetCE", NULL, NULL},
    { 6, "TKN_GetServerInterfaces", NULL, NULL},
    { 7, "TKN_SetParams", NULL, NULL},
    { 8, "TKN_AsyncGrant", NULL, NULL},
    { 0, NULL, NULL, NULL },

};

void
proto_register_TKN4Int (void)
{
	static hf_register_info hf[] = {
{ &hf_tkn4int_tokenrevoke_rqst_colbchoice_low,
        {"hf_tkn4int_tokenrevoke_rqst_colbchoice_low", "hf_tkn4int_tokenrevoke_rqst_colbchoice_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_colbchoice_high,
        {"hf_tkn4int_tokenrevoke_rqst_colbchoice_high", "hf_tkn4int_tokenrevoke_rqst_colbchoice_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_columnb_low,
        {"hf_tkn4int_tokenrevoke_rqst_columnb_low", "hf_tkn4int_tokenrevoke_rqst_columnb_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_columnb_high,
        {"hf_tkn4int_tokenrevoke_rqst_columnb_high", "hf_tkn4int_tokenrevoke_rqst_columnb_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_colachoice_low,
        {"hf_tkn4int_tokenrevoke_rqst_colachoice_low", "hf_tkn4int_tokenrevoke_rqst_colachoice_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_colachoice_high,
        {"hf_tkn4int_tokenrevoke_rqst_colachoice_high", "hf_tkn4int_tokenrevoke_rqst_colachoice_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_endrangeext,
        {"hf_tkn4int_tokenrevoke_rqst_endrangeext", "hf_tkn4int_tokenrevoke_rqst_endrangeext", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_beginrangeext,
        {"hf_tkn4int_tokenrevoke_rqst_beginrangeext", "hf_tkn4int_tokenrevoke_rqst_beginrangeext", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_endrange,
        {"hf_tkn4int_tokenrevoke_rqst_endrange", "hf_tkn4int_tokenrevoke_rqst_endrange", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_beginrange,
        {"hf_tkn4int_tokenrevoke_rqst_beginrange", "hf_tkn4int_tokenrevoke_rqst_beginrange", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_type_low2,
        {"hf_tkn4int_tokenrevoke_rqst_type_low2", "hf_tkn4int_tokenrevoke_rqst_type_low2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_type_hi2,
        {"hf_tkn4int_tokenrevoke_rqst_type_hi2", "hf_tkn4int_tokenrevoke_rqst_type_hi2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_expirationtime,
        {"hf_tkn4int_tokenrevoke_rqst_expirationtime", "hf_tkn4int_tokenrevoke_rqst_expirationtime", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_tokenid_low2,
        {"hf_tkn4int_tokenrevoke_rqst_tokenid_low2", "hf_tkn4int_tokenrevoke_rqst_tokenid_low2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_tokenid_hi2,
        {"hf_tkn4int_tokenrevoke_rqst_tokenid_hi2", "hf_tkn4int_tokenrevoke_rqst_tokenid_hi2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_errorids_low,
        {"hf_tkn4int_tokenrevoke_rqst_errorids_low", "hf_tkn4int_tokenrevoke_rqst_errorids_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_errorids_high,
        {"hf_tkn4int_tokenrevoke_rqst_errorids_high", "hf_tkn4int_tokenrevoke_rqst_errorids_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_outflags,
        {"hf_tkn4int_tokenrevoke_rqst_outflags", "hf_tkn4int_tokenrevoke_rqst_outflags", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_flags,
        {"hf_tkn4int_tokenrevoke_rqst_flags", "hf_tkn4int_tokenrevoke_rqst_flags", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_type_low,
        {"hf_tkn4int_tokenrevoke_rqst_type_low", "hf_tkn4int_tokenrevoke_rqst_type_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_type_high,
        {"hf_tkn4int_tokenrevoke_rqst_type_high", "hf_tkn4int_tokenrevoke_rqst_type_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_tokenid_low,
        {"hf_tkn4int_tokenrevoke_rqst_tokenid_low", "hf_tkn4int_tokenrevoke_rqst_tokenid_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_tokenid_high,
        {"hf_tkn4int_tokenrevoke_rqst_tokenid_high", "hf_tkn4int_tokenrevoke_rqst_tokenid_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_unique,
        {"hf_tkn4int_tokenrevoke_rqst_unique", "hf_tkn4int_tokenrevoke_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_vnode,
        {"hf_tkn4int_tokenrevoke_rqst_vnode", "hf_tkn4int_tokenrevoke_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_volume_low,
        {"hf_tkn4int_tokenrevoke_rqst_volume_low", "hf_tkn4int_tokenrevoke_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_volume_high,
        {"hf_tkn4int_tokenrevoke_rqst_volume_high", "hf_tkn4int_tokenrevoke_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_cell_high,
        {"hf_tkn4int_tokenrevoke_rqst_cell_high", "hf_tkn4int_tokenrevoke_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_cell_low,
        {"hf_tkn4int_tokenrevoke_rqst_cell_low", "hf_tkn4int_tokenrevoke_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_l_type,
        {"hf_tkn4int_tokenrevoke_rqst_l_type", "hf_tkn4int_tokenrevoke_rqst_l_type", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_l_whence,
        {"hf_tkn4int_tokenrevoke_rqst_l_whence", "hf_tkn4int_tokenrevoke_rqst_l_whence", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_l_start_pos,
        {"hf_tkn4int_tokenrevoke_rqst_l_start_pos", "hf_tkn4int_tokenrevoke_rqst_l_start_pos", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_l_end_pos,
        {"hf_tkn4int_tokenrevoke_rqst_l_end_pos", "hf_tkn4int_tokenrevoke_rqst_l_end_pos", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_l_pid,
        {"hf_tkn4int_tokenrevoke_rqst_l_pid", "hf_tkn4int_tokenrevoke_rqst_l_pid", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_l_sysid,
        {"hf_tkn4int_tokenrevoke_rqst_l_sysid", "hf_tkn4int_tokenrevoke_rqst_l_sysid", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_l_fstype,
        {"hf_tkn4int_tokenrevoke_rqst_l_fstype", "hf_tkn4int_tokenrevoke_rqst_l_fstype", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_l_start_pos_ext,
        {"hf_tkn4int_tokenrevoke_rqst_l_start_pos_ext", "hf_tkn4int_tokenrevoke_rqst_l_start_pos_ext", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_tkn4int_tokenrevoke_rqst_l_end_pos_ext,
        {"hf_tkn4int_tokenrevoke_rqst_l_end_pos_ext", "hf_tkn4int_tokenrevoke_rqst_l_end_pos_ext", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
 
			      
	};

	static gint *ett[] = {
		&ett_TKN4Int,
	};
	proto_TKN4Int = proto_register_protocol ("DCE/RPC TokenServer Calls", "TKN4Int", "TKN4Int");
	proto_register_field_array (proto_TKN4Int, hf, array_length (hf));
	proto_register_subtree_array (ett, array_length (ett));
}

void
proto_reg_handoff_TKN4Int (void)
{
	/* Register the protocol as dcerpc */
	dcerpc_init_uuid (proto_TKN4Int, ett_TKN4Int, &uuid_TKN4Int, ver_TKN4Int, TKN4Int_dissectors, -1);
}
