/* packet-dcerpc-cds_clerkserver.c
 *
 * Routines for cds_clerkserver  dissection
 * Routines for dcerpc Afs4Int dissection
 * Copyright 2002, Jaime Fournier <jafour1@yahoo.com>
 * This information is based off the released idl files from opengroup.
 * ftp://opengroup.org/
 *      
 * $Id: packet-dcerpc-cds_clerkserver.c,v 1.2 2002/01/21 07:36:33 guy Exp $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif


#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <string.h>

#include <glib.h>
#include <epan/packet.h>
#include "packet-dcerpc.h"


static int proto_cds_clerkserver = -1;

static gint ett_cds_clerkserver = -1;
static int hf_cds_clerk_readattribute_rqst_var1 = -1;
static int hf_cds_clerk_readattribute_rqst_var2 = -1;
static int hf_cds_clerk_readattribute_rqst_var3 = -1;
static int hf_cds_clerk_readattribute_rqst_var4 = -1;
static int hf_cds_clerk_readattribute_rqst_var5 = -1;
static int hf_cds_clerk_readattribute_rqst_var6 = -1;
static int hf_cds_clerk_readattribute_rqst_var7 = -1;
static int hf_cds_clerk_readattribute_rqst_var8 = -1;
static int hf_cds_clerk_readattribute_rqst_key_size = -1;
static int hf_cds_clerk_readattribute_rqst_key_t = -1;


static e_uuid_t uuid_cds_clerkserver = { 0x257df1c9, 0xc6d3, 0x11ca, { 0x85, 0x54, 0x08, 0x00, 0x2b, 0x1c, 0x8f, 0x1f } };
static guint16  ver_cds_clerkserver = 1;

static int
cds_clerk_dissect_readattribute_rqst (tvbuff_t *tvb, int offset,
                                packet_info *pinfo, proto_tree *tree,
                                char *drep)
{
       /*
	*      [in,out]cds_Progress_t *Progress_p,
	*               unsigned small int  pr_flags;
	*
	*               [ptr]cds_Timeout_t  *pr_timeout;
	*                          byte    to_expire[16];
	*                          byte    to_extend[16];
	*
	*               cds_FullName_t      pr_unresolved;
	*                       uuid_t          fn_root;
	*                       long int        fn_length;
	*                       [length_is(fn_length)] char fn_name[1023];
	*                       
	*               cds_FullName_t      pr_resolved;
	*                       uuid_t          fn_root;
	*                       long int        fn_length;
	*                       [length_is(fn_length)] char fn_name[1023];
	*
	*               [ptr]cds_Set_t      *pr_replicas_p;
	*                        unsigned small int set_type;
	*                        unsigned short int set_length;
	*                        [size_is(set_length)] cds_SetMember_t set_members[];
	*                       
	*      [in] unsigned small type,
	*
	*      [in] cds_Name_t  *att_p,
	*               unsigned short int          nm_length;
	*               [length_is(nm_length)] byte nm_name[257];
	*
	*      [in] cds_Timestamp_t  *context_p,
	*               byte               ts_node[6];
	*               unsigned hyper int ts_time;
	*
	*      [in] unsigned32 max_size,
	*      		
	*      [in] unsigned32 maybemore,
	*                              
	*/

       guint32 var1, var2, var3, var4, var5, var6, var7, var8;
       guint32 key_size;
       char key_t[100];

       offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_cds_clerk_readattribute_rqst_var1, &var1);
       offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_cds_clerk_readattribute_rqst_var2, &var2);
       offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_cds_clerk_readattribute_rqst_var3, &var3);
       offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_cds_clerk_readattribute_rqst_var4, &var4);
       offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_cds_clerk_readattribute_rqst_var5, &var5);
       offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_cds_clerk_readattribute_rqst_var6, &var6);
       offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_cds_clerk_readattribute_rqst_var7, &var7);
       offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_cds_clerk_readattribute_rqst_var8, &var8);
       offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_cds_clerk_readattribute_rqst_key_size, &key_size);

       proto_tree_add_string (tree, hf_cds_clerk_readattribute_rqst_key_t, tvb, offset, hf_cds_clerk_readattribute_rqst_key_size, tvb_get_ptr (tvb, offset, key_size));

        tvb_memcpy(tvb, key_t, offset, key_size);
       	key_t[key_size] = '\0';
	offset += key_size;

       proto_tree_add_string (tree, hf_cds_clerk_readattribute_rqst_key_t, tvb, offset, hf_cds_clerk_readattribute_rqst_key_size, tvb_get_ptr (tvb, offset, key_size));

       if (check_col(pinfo->cinfo, COL_INFO)) col_clear(pinfo->cinfo, COL_INFO); 
       if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO,
		  "cds_ReadAttribute Request for: %s",  key_t);
				    

     return offset;
}

static dcerpc_sub_dissector cds_clerkserver_dissectors[] = {
    { 0, "cds_AddReplica", NULL, NULL},
    { 1, "cds_AllowClearinghouses", NULL, NULL},
    { 2, "cds_Combine", NULL, NULL},
    { 3, "cds_CreateChild", NULL, NULL},
    { 4, "cds_CreateDirectory", NULL, NULL},
    { 5, "cds_CreateSoftLink", NULL, NULL},
    { 6, "cds_CreateObject", NULL, NULL},
    { 7, "cds_DeleteChild", NULL, NULL},
    { 8, "cds_DeleteObject", NULL, NULL},
    { 9, "cds_DeleteSoftLink", NULL, NULL},
    { 10, "cds_DeleteDirectory", NULL, NULL},
    { 11, "cds_DisallowClearinghouses", NULL, NULL},
    { 12, "cds_DoUpdate", NULL, NULL},
    { 13, "cds_EnumerateAttributes", NULL, NULL},
    { 14, "cds_EnumerateChildren", NULL, NULL},
    { 15, "cds_EnumerateObjects", NULL, NULL},
    { 16, "cds_EnumerateSoftLinks", NULL, NULL},
    { 17, "cds_LinkReplica", NULL, NULL},
    { 18, "cds_ModifyAttribute", NULL, NULL},
    { 19, "cds_ModifyReplica", NULL, NULL},
    { 20, "cds_NewEpoch", NULL, NULL},
    { 21, "cds_ReadAttribute", cds_clerk_dissect_readattribute_rqst, NULL},
    { 22, "cds_RemoveReplica", NULL, NULL},
    { 23, "cds_ResolveName", NULL, NULL},
    { 24, "cds_Skulk", NULL, NULL},
    { 25, "cds_TestAttribute", NULL, NULL},
    { 26, "cds_TestGroup", NULL, NULL},
    { 0, NULL, NULL, NULL },
};


void
proto_register_cds_clerkserver (void)
{
	static hf_register_info hf[] = {
       { &hf_cds_clerk_readattribute_rqst_var1,
      { "Var1", "cds_clerk.readattribute_rqst_var1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
       { &hf_cds_clerk_readattribute_rqst_var2,
      { "Var2", "cds_clerk.readattribute_rqst_var2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
       { &hf_cds_clerk_readattribute_rqst_var3,
      { "Var3", "cds_clerk.readattribute_rqst_var3", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
       { &hf_cds_clerk_readattribute_rqst_var4,
      { "Var4", "cds_clerk.readattribute_rqst_var4", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
       { &hf_cds_clerk_readattribute_rqst_var5,
      { "Var5", "cds_clerk.readattribute_rqst_var5", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
       { &hf_cds_clerk_readattribute_rqst_var6,
      { "Var6", "cds_clerk.readattribute_rqst_var6", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
       { &hf_cds_clerk_readattribute_rqst_var7,
      { "Var7", "cds_clerk.readattribute_rqst_var7", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
       { &hf_cds_clerk_readattribute_rqst_var8,
      { "Var8", "cds_clerk.readattribute_rqst_var8", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
       { &hf_cds_clerk_readattribute_rqst_key_size,
      { "Key Size", "cds_clerk.readattribute_rqst_key_size", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
       { &hf_cds_clerk_readattribute_rqst_key_t,
      { "Key", "cds_clerk.readattribute_key_t", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
							   
	};

	static gint *ett[] = {
		&ett_cds_clerkserver,
	};
	proto_cds_clerkserver = proto_register_protocol ("CDS Clerk Server Calls", "CDS_CLERK", "cds_clerkserver");
	proto_register_field_array (proto_cds_clerkserver, hf, array_length (hf));
	proto_register_subtree_array (ett, array_length (ett));
}

void
proto_reg_handoff_cds_clerkserver (void)
{
	/* Register the protocol as dcerpc */
	dcerpc_init_uuid (proto_cds_clerkserver, ett_cds_clerkserver, &uuid_cds_clerkserver, ver_cds_clerkserver, cds_clerkserver_dissectors, -1);
}
