/* packet-dcerpc-krb5rpc.c
 *
 * Routines for dcerpc DCE/KRB5 interface
 * Copyright 2002, Jaime Fournier <jafour1@yahoo.com>
 * This information is based off the released idl files from opengroup.
 * ftp://opengroup.org/
 *
 *
 * $Id: packet-dcerpc-krb5rpc.c,v 1.2 2002/01/21 07:36:33 guy Exp $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif


#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <string.h>

#include <glib.h>
#include <epan/packet.h>
#include "packet-dcerpc.h"




static int proto_krb5rpc = -1;

static gint ett_krb5rpc = -1;


static e_uuid_t uuid_krb5rpc = { 0x8f73de50, 0x768c, 0x11ca, { 0xbf, 0xfc, 0x08, 0x00, 0x1e, 0x03, 0x94, 0x31 } };
static guint16  ver_krb5rpc = 1;
extern guint32 master_seqnum;
static int hf_krb5rpc_sendto_kdc_rqst_key_size = -1;
static int hf_krb5rpc_sendto_kdc_rqst_key_t = -1;
static int hf_krb5rpc_sendto_kdc_rqst_out_buf_len = -1;
static int hf_krb5rpc_sendto_kdc_resp_key_size = -1;
static int hf_krb5rpc_sendto_kdc_resp_key_t = -1;
static int hf_krb5rpc_sendto_kdc_resp_error_status_t = -1;

static int
krb5rpc_dissect_sendto_kdc_rqst (tvbuff_t *tvb, int offset,
		                 packet_info *pinfo, proto_tree *tree,
		                 char *drep)
{
	        guint32 key_size, out_buf_len;
		char *key_t = NULL;

	/*
	 *        [in]        handle_t        h,
	 *        [in]        unsigned32      len,
	 *        [in, size_is(len)]
	 *        byte            message[],
	 *        [in]        unsigned32      out_buf_len,
	 */                                                                                                 

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_krb5rpc_sendto_kdc_rqst_key_size, &key_size);
	/*
        proto_tree_add_string (tree, hf_krb5rpc_sendto_kdc_rqst_key_t, tvb, offset, hf_krb5rpc_sendto_kdc_rqst_key_size, tvb_get_ptr (tvb, offset, key_size));
        key_t = tvb_get_ptr(tvb,offset,key_size);
        offset += key_size;
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_krb5rpc_sendto_kdc_rqst_out_buf_len, &out_buf_len);
	*/
				 
                if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "KRB5RPC_sendto_kdc Request KerbKeySize: %u", key_size);
		if (check_col(pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
		return offset;
} 




static int
krb5rpc_dissect_sendto_kdc_resp (tvbuff_t *tvb, int offset,
		                 packet_info *pinfo, proto_tree *tree,
		                 char *drep)
{
		guint32 key_size, error_status_t;
		char *key_t = NULL;

	/*
	 *
	 *        [out]       unsigned32      *resp_len,
	 *        [out, length_is(*resp_len), size_is(out_buf_len)]
	 *        byte            out_buf[],
	 *        [out]       error_status_t  *st unsigned long
	 *
	 */
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_krb5rpc_sendto_kdc_resp_key_size, &key_size);
	/*
        proto_tree_add_string (tree, hf_krb5rpc_sendto_kdc_resp_key_t, tvb, offset, hf_krb5rpc_sendto_kdc_resp_key_size, tvb_get_ptr (tvb, offset, key_size));
        key_t = tvb_get_ptr(tvb,offset,key_size);
        offset += key_size;
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_krb5rpc_sendto_kdc_resp_error_status_t, &error_status_t);
	*/
				 
                if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "KRB5RPC_sendto_kdc Reply KerbKeySize: %u", key_size);
		if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
		return offset;
}


static dcerpc_sub_dissector krb5rpc_dissectors[] = {
	    { 0, "rsec_krb5rpc_sendto_kdc", krb5rpc_dissect_sendto_kdc_rqst, krb5rpc_dissect_sendto_kdc_resp},
	         { 0, NULL, NULL, NULL },
};


void
proto_register_krb5rpc (void)
{
	static hf_register_info hf[] = {
{ &hf_krb5rpc_sendto_kdc_resp_key_size,
{"hf_krb5rpc_sendto_kdc_resp_key_size", "hf_krb5rpc_sendto_kdc_resp_key_size", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_krb5rpc_sendto_kdc_resp_key_t,
{"hf_krb5rpc_sendto_kdc_resp_key_t", "hf_krb5rpc_sendto_kdc_resp_key_t", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_krb5rpc_sendto_kdc_resp_error_status_t,
{"hf_krb5rpc_sendto_kdc_resp_error_status_t", "hf_krb5rpc_sendto_kdc_resp_error_status_t", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_krb5rpc_sendto_kdc_rqst_key_size,
{"hf_krb5rpc_sendto_kdc_rqst_key_size", "hf_krb5rpc_sendto_kdc_rqst_key_size", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_krb5rpc_sendto_kdc_rqst_key_t,
{"hf_krb5rpc_sendto_kdc_rqst_key_t", "hf_krb5rpc_sendto_kdc_rqst_key_t", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_krb5rpc_sendto_kdc_rqst_out_buf_len,
{"hf_krb5rpc_sendto_kdc_rqst_out_buf_len", "hf_krb5rpc_sendto_kdc_rqst_out_buf_len", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
	};

	static gint *ett[] = {
		&ett_krb5rpc,
	};
	proto_krb5rpc = proto_register_protocol ("DCE/RPC Kerberos V", "krb5rpc", "krb5rpc");
	proto_register_field_array (proto_krb5rpc, hf, array_length (hf));
	proto_register_subtree_array (ett, array_length (ett));
}

void
proto_reg_handoff_krb5rpc (void)
{
	/* Register the protocol as dcerpc */
	dcerpc_init_uuid (proto_krb5rpc, ett_krb5rpc, &uuid_krb5rpc, ver_krb5rpc, krb5rpc_dissectors, -1);
}
