/* packet-dcerpc-rep_proc.c
 *
 * Routines for dcerpc Replica Server Call dissection
 * Copyright 2002, Jaime Fournier <jafour1@yahoo.com>
 * This information is based off the released idl files from opengroup.
 * ftp://opengroup.org/
 *
 * $Id: packet-dcerpc-rep_proc.c,v 1.2 2002/01/21 07:36:33 guy Exp $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif


#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <string.h>

#include <glib.h>
#include <epan/packet.h>
#include "packet-dcerpc.h"


static int proto_rep_proc = -1;

static gint ett_rep_proc = -1;


static e_uuid_t uuid_rep_proc = { 0x4d37f2dd, 0xed43, 0x0005, { 0x02, 0xc0, 0x37, 0xcf, 0x1e, 0x00, 0x00, 0x00 } };
static guint16  ver_rep_proc = 4;
static int hf_rep_proc_checkreplicationconfig_rqst_cell = -1;
static int hf_rep_proc_checkreplicationconfig_rqst_cell_low = -1;
static int hf_rep_proc_checkreplicationconfig_rqst_cell_high = -1;
static int hf_rep_proc_checkreplicationconfig_rqst_vol_high = -1;
static int hf_rep_proc_checkreplicationconfig_rqst_vol_low = -1;
static int hf_rep_proc_checkreplicationconfig_rqst_vol = -1;
static int hf_rep_proc_checkreplicationconfig_rqst_flags = -1;
static int hf_rep_proc_checkreplicationconfig_rqst_spare1 = -1;
static int hf_rep_proc_checkreplicationconfig_rqst_spare2 = -1;
static int hf_rep_proc_checkreplicationconfig_rqst_spare3 = -1;
static int hf_rep_proc_checkreplicationconfig_resp_spare4 = -1;
static int hf_rep_proc_checkreplicationconfig_resp_spare5 = -1;
static int hf_rep_proc_allcheckreplicationconfig_rqst_cell_high = -1;
static int hf_rep_proc_allcheckreplicationconfig_rqst_cell_low = -1;
static int hf_rep_proc_allcheckreplicationconfig_rqst_vol_low = -1;
static int hf_rep_proc_allcheckreplicationconfig_rqst_vol_high = -1;
static int hf_rep_proc_allcheckreplicationconfig_rqst_howmanyguys = -1;
static int hf_rep_proc_allcheckreplicationconfig_rqst_theseguystoo = -1;
static int hf_rep_proc_allcheckreplicationconfig_rqst_flags = -1;
static int hf_rep_proc_allcheckreplicationconfig_rqst_spare1 = -1;
static int hf_rep_proc_allcheckreplicationconfig_rqst_spare2 = -1;
static int hf_rep_proc_allcheckreplicationconfig_rqst_spare3 = -1;
static int hf_rep_proc_getonerepstatus_rqst_cellidp_high = -1;
static int hf_rep_proc_getonerepstatus_rqst_cellidp_low = -1;
static int hf_rep_proc_getonerepstatus_rqst_volidp_low = -1;
static int hf_rep_proc_getonerepstatus_rqst_volidp_high = -1;
static int hf_rep_proc_getonerepstatus_rqst_spare1 = -1;
static int hf_rep_proc_getonerepstatus_rqst_spare2 = -1;
static int hf_rep_proc_getonerepstatus_rqst_spare3 = -1;
static int hf_rep_getvolchangedfiles_rqst_cellidp_high = -1;
static int hf_rep_getvolchangedfiles_rqst_cellidp_low = -1;
static int hf_rep_getvolchangedfiles_rqst_volidp_high = -1;
static int hf_rep_getvolchangedfiles_rqst_volidp_low = -1;
static int hf_rep_getvolchangedfiles_rqst_initialvv_high = -1;
static int hf_rep_getvolchangedfiles_rqst_initialvv_low = -1;
static int hf_rep_getvolchangedfiles_rqst_finalvv_high = -1;
static int hf_rep_getvolchangedfiles_rqst_finalvv_low = -1;
static int hf_rep_getvolchangedfiles_rqst_cookie = -1;
static int hf_rep_getvolchangedfiles_rqst_spare1 = -1;
static int hf_rep_getvolchangedfiles_rqst_spare2 = -1;
static int hf_rep_getvolchangedfiles_rqst_spare3 = -1;
static int hf_rep_getvolchangedfiles_resp_rfinalvv_high = -1;
static int hf_rep_getvolchangedfiles_resp_rfinalvv_low = -1;
static int hf_rep_getvolchangedfiles_resp_ncookie = -1;

static int
rep_proc_dissect_checkreplicationconfig_resp (tvbuff_t *tvb, int offset,
                                              packet_info *pinfo, proto_tree *tree,
                           		      char *drep)
{
	  guint32 spare4, spare5;
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_checkreplicationconfig_resp_spare4, &spare4);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_checkreplicationconfig_resp_spare5, &spare5);
	 /*
	  *        [out] unsigned32 *spare4,
	  *        [out] unsigned32 *spare5
	  */
	  return offset;

}

static int
rep_proc_dissect_allcheckreplicationconfig_rqst (tvbuff_t *tvb, int offset,
                                              packet_info *pinfo, proto_tree *tree,
                           		      char *drep)
{
	  guint32 cell_high, cell_low, vol_high, vol_low, howmanyguys, theseguystoo, flags, spare1, spare2, spare3;

	/*   
	 *        [in] afsHyper *cellIdP,
	 *        [in] afsHyper *volIdP,
	 *        [in] unsigned32 howManyGuys,
	 *        [in] afsNetAddrs *TheseGuysToo,
	 *        [in] unsigned32 flags,
	 *        [in] unsigned32 spare1,
	 *        [in] unsigned32 spare2,
	 *        [in] unsigned32 spare3,
	 */                                                                        

	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_allcheckreplicationconfig_rqst_cell_high, &cell_high);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_allcheckreplicationconfig_rqst_cell_low, &cell_low);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_allcheckreplicationconfig_rqst_vol_high, &vol_high);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_allcheckreplicationconfig_rqst_vol_low, &vol_low);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_allcheckreplicationconfig_rqst_howmanyguys, &howmanyguys);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_allcheckreplicationconfig_rqst_theseguystoo, &theseguystoo);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_allcheckreplicationconfig_rqst_flags, &flags);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_allcheckreplicationconfig_rqst_spare1, &spare1);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_allcheckreplicationconfig_rqst_spare2, &spare2);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_allcheckreplicationconfig_rqst_spare3, &spare3);
	 if (check_col(pinfo->cinfo, COL_INFO)) col_add_str(pinfo->cinfo, COL_INFO, "REP_AllCheckReplicationConfig Request");
	  
	return offset;
}
static int
rep_proc_dissect_checkreplicationconfig_rqst (tvbuff_t *tvb, int offset,
                                              packet_info *pinfo, proto_tree *tree,
                           		      char *drep)
{
		 guint32 cell_high, cell_low, vol_high, vol_low, flags, spare1, spare2,  spare3;
		 guint64 cell, vol;

	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_checkreplicationconfig_rqst_cell_high, &cell_high);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_checkreplicationconfig_rqst_cell_low, &cell_low);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_checkreplicationconfig_rqst_vol_high, &vol_high);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_checkreplicationconfig_rqst_vol_low, &vol_low);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_checkreplicationconfig_rqst_flags, &flags);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_checkreplicationconfig_rqst_spare1, &spare1);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_checkreplicationconfig_rqst_spare2, &spare2);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_checkreplicationconfig_rqst_spare3, &spare3);
	  cell = cell_high | cell_low;
	  vol = vol_high | vol_low;

	 /*
	  *       [in] afsHyper *cellIdP,
	  *       [in] afsHyper *volIdP,
	  *       [in] unsigned32 flags,
	  *       [in] unsigned32 spare1,
	  *       [in] unsigned32 spare2,
	  *       [in] unsigned32 spare3,
	  */
	 if (check_col(pinfo->cinfo, COL_INFO)) col_add_str(pinfo->cinfo, COL_INFO, "REP_CheckReplicationConfig Request");

	  return offset;
							   
			          
}
static int
rep_proc_dissect_getonerepstatus_rqst (tvbuff_t *tvb, int offset,
                                              packet_info *pinfo, proto_tree *tree,
                           		      char *drep)
{
	guint64 cellidp, volidp;
	guint32 cellidp_high, cellidp_low, volidp_high, volidp_low, spare1, spare2, spare3;

	/*        [in] handle_t h,
	 *        [in] afsHyper *cellIdP,
	 *        [in] afsHyper *volIdP,
	 *        [in] unsigned32 spare1,
	 *        [in] unsigned32 spare2,
	 *        [in] unsigned32 spare3,
	 */
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_getonerepstatus_rqst_cellidp_high, &cellidp_high);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_getonerepstatus_rqst_cellidp_low, &cellidp_low);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_getonerepstatus_rqst_volidp_high, &volidp_high);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_getonerepstatus_rqst_volidp_low, &volidp_low);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_getonerepstatus_rqst_spare1, &spare1);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_getonerepstatus_rqst_spare2, &spare2);
	  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_proc_getonerepstatus_rqst_spare3, &spare3);
	  cellidp = cellidp_high | cellidp_low;
	  volidp = volidp_high | volidp_low;

	 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "REP_GetOneRepStatus Request for FSID: %llu", volidp);

	return offset;
}
static int
rep_proc_dissect_getvolchangedfiles_rqst (tvbuff_t *tvb, int offset,
		                 packet_info *pinfo, proto_tree *tree,
		                 char *drep)
{
	/*        [in] afsHyper *cellIdP,
	 *        [in] afsHyper *volIdP,
	 *        [in] afsHyper *initialVV,
	 *        [in] afsHyper *finalVV,
	 *        [in] unsigned32 cookie,
	 *        [in] unsigned32 spare1,
	 *        [in] unsigned32 spare2,
	 *        [in] unsigned32 spare3,
	 */                                                                 

	guint32 cellidp_high, cellidp_low, volidp_high, volidp_low, initialvv_high, initialvv_low, finalvv_high, finalvv_low, cookie, spare1, spare2, spare3;
	guint64 volume, initialvv, finalvv;

         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_rqst_cellidp_high, NULL);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_rqst_cellidp_low, NULL);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_rqst_volidp_high, &volidp_high);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_rqst_volidp_low, &volidp_low);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_rqst_initialvv_high, &initialvv_high);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_rqst_initialvv_low, &initialvv_low);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_rqst_finalvv_high, &finalvv_high);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_rqst_finalvv_low, &finalvv_low);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_rqst_cookie, &cookie);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_rqst_spare1, &spare1);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_rqst_spare2, &spare2);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_rqst_spare3, &spare3);

	 volume = volidp_high | volidp_low;
	 initialvv = initialvv_high | initialvv_low;
	 finalvv = finalvv_high | finalvv_low;

 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "REP_GetVolChangedFiles Request for FSID: %llu Ivv: %llu Fvv: %llu ", volume, initialvv, finalvv);

}
static int
rep_proc_dissect_getvolchangedfiles_resp (tvbuff_t *tvb, int offset,
	                                  packet_info *pinfo, proto_tree *tree,
	                                  char *drep)
{
	guint32 rfinalvv_high, rfinalvv_low, ncookie;
	guint64 rfinal;
		/*        [out] afsHyper *reallyFinalVV,
		 *        [out] unsigned32 *nextCookieP,
		 */       

         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_resp_rfinalvv_high, &rfinalvv_high);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_resp_rfinalvv_low, &rfinalvv_low);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_getvolchangedfiles_resp_ncookie, &ncookie);

	 rfinal = rfinalvv_high | rfinalvv_low;

	        /*
		 *        [out] fidsInVol *fidPerVolP,
		 *        [out] unsigned32 *numFidsInVolP,
		 *        [out] unsigned32 *flagsP,
		 *        [out] unsigned32 *spare4,
		 *        [out] unsigned32 *spare5
		 */                                       

 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "REP_GetVolChangedFiles Response for Fvv: %llu Cookie: %u ", rfinal, ncookie);
	return offset;
}
	 

/*
static int
rep_dissect_keepfilesalive_rqst (tvbuff_t *tvb, int offset,
		                 packet_info *pinfo, proto_tree *tree,
		                 char *drep)
{
	        guint64 cellidp, volidp;
		*
		 *         [in] afsFids *files,
		 *                 unsigned32 afsFids_len;
		 *                 [length_is(afsFids_len)] afsFid afsFids_val[AFS_BULKMAX];
		 *         [in] unsigned32 howManyFiles,
		 *         [in] unsigned32 flags,
		 *         [in] unsigned32 spare1,
		 *         [in] unsigned32 spare2,
		 *         [in] unsigned32 spare3,
		 *                                                  
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_keepfilesalive_rqst_afsfids_len, &afsfids_len);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_keepfilesalive_rqst_afsfids_len, &afsfids_len);
	 

         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_keepfilesalive_rqst_howmanyfiles, &howmanyfiles);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_keepfilesalive_rqst_flags, &flags);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_keepfilesalive_rqst_spare1, &spare1);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_keepfilesalive_rqst_spare2, &spare2);
         offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rep_keepfilesalive_rqst_spare3, &spare3);
			   
		 
}

*/


static dcerpc_sub_dissector rep_proc_dissectors[] = {
    { 0, "REP_CheckReplicationConfig", rep_proc_dissect_checkreplicationconfig_rqst, rep_proc_dissect_checkreplicationconfig_resp},
    { 1, "REP_AllCheckReplicationConfig", rep_proc_dissect_allcheckreplicationconfig_rqst, NULL},
    { 2, "REP_KeepFilesAlive", NULL , NULL},
    { 3, "REP_GetVolChangedFiles", rep_proc_dissect_getvolchangedfiles_rqst, rep_proc_dissect_getvolchangedfiles_resp,},
    { 4, "REP_GetRepStatus", NULL, NULL},
    { 5, "REP_GetRepServerStatus", NULL, NULL},
    { 6, "REP_UpdateSelf", NULL, NULL},
    { 7, "REP_Probe", NULL, NULL},
    { 8, "REP_GetOneRepStatus", rep_proc_dissect_getonerepstatus_rqst, NULL},
    { 9, "REP_GetServerInterfaces", NULL, NULL},
    { 0, NULL, NULL, NULL },
};


void
proto_register_rep_proc (void)
{
	static hf_register_info hf[] = {
{ &hf_rep_proc_checkreplicationconfig_rqst_cell,
   {"Cell:", "hf_rep_proc_checkreplicationconfig_rqst_cell,", FT_UINT64, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_checkreplicationconfig_rqst_cell_high,
   {"Cell_high:", "hf_rep_proc_checkreplicationconfig_rqst_cell_high,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_checkreplicationconfig_rqst_cell_low,
   {"Cell_low:", "hf_rep_proc_checkreplicationconfig_rqst_cell_low,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_checkreplicationconfig_rqst_vol_low,
   {"Vol_low:", "hf_rep_proc_checkreplicationconfig_rqst_vol_low,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_checkreplicationconfig_rqst_vol_high,
   {"Vol_high:", "hf_rep_proc_checkreplicationconfig_rqst_vol_high,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_checkreplicationconfig_rqst_vol,
   {"Vol:", "hf_rep_proc_checkreplicationconfig_rqst_vol,", FT_UINT64, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_checkreplicationconfig_rqst_flags,
   {"Flags:", "hf_rep_proc_checkreplicationconfig_rqst_flags,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_checkreplicationconfig_rqst_spare1,
   {"Spare1:", "hf_rep_proc_checkreplicationconfig_rqst_spare1,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_checkreplicationconfig_rqst_spare2,
   {"Spare2:", "hf_rep_proc_checkreplicationconfig_rqst_spare2,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_checkreplicationconfig_rqst_spare3,
   {"Spare2:", "hf_rep_proc_checkreplicationconfig_rqst_spare3,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_checkreplicationconfig_resp_spare4,
   {"Spare4:", "hf_rep_proc_checkreplicationconfig_resp_spare4,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_checkreplicationconfig_resp_spare5,
   {"Spare4:", "hf_rep_proc_checkreplicationconfig_resp_spare5,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_allcheckreplicationconfig_rqst_cell_high,
   {"Cell_high:", "hf_rep_proc_allcheckreplicationconfig_rqst_cell_high,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_allcheckreplicationconfig_rqst_cell_low,
   {"Cell_low:", "hf_rep_proc_allcheckreplicationconfig_rqst_cell_low,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_allcheckreplicationconfig_rqst_vol_low,
   {"Vol_low:", "hf_rep_proc_allcheckreplicationconfig_rqst_vol_low,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_allcheckreplicationconfig_rqst_vol_high,
   {"Vol_high", "hf_rep_proc_allcheckreplicationconfig_rqst_vol_high,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_allcheckreplicationconfig_rqst_howmanyguys,
   {"HowManyGuys", "hf_rep_proc_allcheckreplicationconfig_rqst_howmanyguys,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_allcheckreplicationconfig_rqst_theseguystoo,
   {"TheseGuysToo", "hf_rep_proc_allcheckreplicationconfig_rqst_thesegyystoo,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_allcheckreplicationconfig_rqst_flags,
   {"Flags", "hf_rep_proc_allcheckreplicationconfig_rqst_flags,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_allcheckreplicationconfig_rqst_spare1,
   {"Spare1", "hf_rep_proc_allcheckreplicationconfig_rqst_spare1,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_allcheckreplicationconfig_rqst_spare2,
   {"Spare2", "hf_rep_proc_allcheckreplicationconfig_rqst_spare2,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_allcheckreplicationconfig_rqst_spare3,
   {"Spare3", "hf_rep_proc_allcheckreplicationconfig_rqst_spare3,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_getonerepstatus_rqst_cellidp_high,
   {"Cellidp_high", "hf_rep_proc_getonerepstatus_rqst_cellidp_high,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_getonerepstatus_rqst_cellidp_low,
   {"Cellidp_low", "hf_rep_proc_getonerepstatus_rqst_cellidp_low,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_getonerepstatus_rqst_volidp_low,
   {"Volidp_low", "hf_rep_proc_getonerepstatus_rqst_volidp_low,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_getonerepstatus_rqst_volidp_high,
   {"Volidp_high", "hf_rep_proc_getonerepstatus_rqst_volidp_high,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_getonerepstatus_rqst_spare1,
   {"Spare1", "hf_rep_proc_getonerepstatus_rqst_spare1,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_getonerepstatus_rqst_spare2,
   {"Spare2", "hf_rep_proc_getonerepstatus_rqst_spare2,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_proc_getonerepstatus_rqst_spare3,
   {"Spare3", "hf_rep_proc_getonerepstatus_rqst_spare3,", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_rqst_cellidp_high,
       {"hf_rep_getvolchangedfiles_rqst_cellidp_high", "hf_rep_getvolchangedfiles_rqst_cellidp_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_rqst_cellidp_low,
       {"hf_rep_getvolchangedfiles_rqst_cellidp_low", "hf_rep_getvolchangedfiles_rqst_cellidp_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_rqst_volidp_high,
       {"hf_rep_getvolchangedfiles_rqst_volidp_high", "hf_rep_getvolchangedfiles_rqst_volidp_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_rqst_volidp_low,
       {"hf_rep_getvolchangedfiles_rqst_volidp_low", "hf_rep_getvolchangedfiles_rqst_volidp_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_rqst_initialvv_high,
       {"hf_rep_getvolchangedfiles_rqst_initialvv_high", "hf_rep_getvolchangedfiles_rqst_initialvv_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_rqst_initialvv_low,
       {"hf_rep_getvolchangedfiles_rqst_initialvv_low", "hf_rep_getvolchangedfiles_rqst_initialvv_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_rqst_finalvv_low,
       {"hf_rep_getvolchangedfiles_rqst_finalvv_low", "hf_rep_getvolchangedfiles_rqst_finalvv_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_rqst_finalvv_high,
       {"hf_rep_getvolchangedfiles_rqst_finalvv_high", "hf_rep_getvolchangedfiles_rqst_finalvv_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_rqst_cookie,
       {"hf_rep_getvolchangedfiles_rqst_cookie", "hf_rep_getvolchangedfiles_rqst_cookie", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_rqst_spare1,
       {"hf_rep_getvolchangedfiles_rqst_spare1", "hf_rep_getvolchangedfiles_rqst_spare1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_rqst_spare2,
       {"hf_rep_getvolchangedfiles_rqst_spare2", "hf_rep_getvolchangedfiles_rqst_spare2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_rqst_spare3,
       {"hf_rep_getvolchangedfiles_rqst_spare3", "hf_rep_getvolchangedfiles_rqst_spare3", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_resp_ncookie,
       {"hf_rep_getvolchangedfiles_resp_ncookie", "hf_rep_getvolchangedfiles_resp_ncookie", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_resp_rfinalvv_high,
       {"hf_rep_getvolchangedfiles_resp_rfinalvv_high", "hf_rep_getvolchangedfiles_resp_rfinalvv_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rep_getvolchangedfiles_resp_rfinalvv_low,
       {"hf_rep_getvolchangedfiles_resp_rfinalvv_low", "hf_rep_getvolchangedfiles_resp_rfinalvv_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
	};

	static gint *ett[] = {
		&ett_rep_proc,
	};
	proto_rep_proc = proto_register_protocol ("AFS (4.0) Replication Server call declarations", "rep_proc", "rep_proc");
	proto_register_field_array (proto_rep_proc, hf, array_length (hf));
	proto_register_subtree_array (ett, array_length (ett));
}

void
proto_reg_handoff_rep_proc (void)
{
	/* Register the protocol as dcerpc */
	dcerpc_init_uuid (proto_rep_proc, ett_rep_proc, &uuid_rep_proc, ver_rep_proc, rep_proc_dissectors, -1);
}
