/* packet-dcerpc-rs_repadm.c
 *
 * Routines for dcerpc Registry server administration operations.
 * Copyright 2002, Jaime Fournier <jafour1@yahoo.com>
 * This information is based off the released idl files from opengroup.
 * ftp://opengroup.org/
 *
 * $Id: packet-dcerpc-rs_repadm.c,v 1.2 2002/01/21 07:36:33 guy Exp $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif


#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <string.h>

#include <glib.h>
#include <epan/packet.h>
#include "packet-dcerpc.h"


static int proto_rs_repadm = -1;

static gint ett_rs_repadm = -1;


static e_uuid_t uuid_rs_repadm = { 0x5b8c2fa8, 0xb60b, 0x11c9, { 0xbe, 0x0f, 0x08, 0x00, 0x1e, 0x01, 0x8f, 0xa0 } };
static guint16  ver_rs_repadm = 1.1;

static int hf_rs_repadmin_get_sw_vers_info_rqst_handle = -1;
static int hf_rs_repadmin_get_sw_vers_info_rqst_max_num_sw_vers = -1;
static int hf_rs_repadmin_get_sw_vers_info_resp_num_sw_vers = -1;
static int hf_rs_repadmin_info_resp_repstate = -1;
static int hf_rs_repadmin_info_resp_cell_sec_id = -1;
static int hf_rs_repadmin_info_resp_rep_id = -1;
static int hf_rs_repadmin_info_resp_init_id = -1;
static int hf_rs_repadmin_info_resp_seqno_hi = -1;
static int hf_rs_repadmin_info_resp_seqno_low = -1;
static int hf_rs_repadmin_info_resp_last_upd_ts_sec = -1;
static int hf_rs_repadmin_info_resp_last_upd_ts_msec = -1;
static int hf_rs_repadmin_info_resp_sw_rev_hi = -1;
static int hf_rs_repadmin_info_resp_sw_rev_low = -1;
static int hf_rs_repadmin_info_resp_compat_sw_rev = -1;
static int hf_rs_repadmin_info_resp_base_propq_seqno_hi = -1;
static int hf_rs_repadmin_info_resp_base_propq_seqno_low = -1;
static int hf_rs_repadmin_info_resp_master = -1;
static int hf_rs_repadmin_info_resp_master_known = -1;
static int hf_rs_repadmin_info_resp_master_id = -1;
static int hf_rs_repadmin_info_resp_master_seqno_hi = -1;
static int hf_rs_repadmin_info_resp_master_seqno_low = -1;
static int hf_rs_repadmin_info_resp_error_status = -1;

static int
rs_repadmin_dissect_get_sw_vers_info_rqst (tvbuff_t *tvb, int offset,
	                         packet_info *pinfo, proto_tree *tree,
	                         char *drep)
{

	/*        [in]    handle_t                    h,
	 *        [in]    unsigned32                  max_num_sw_vers,
	 */                
	extern guint32 master_seqnum;
	guint32 handle, max_num_sw_vers;

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_get_sw_vers_info_rqst_handle, &handle);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_get_sw_vers_info_rqst_max_num_sw_vers, &max_num_sw_vers);

	        if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "rs_rep_admin_get_sw_vers_info Request Handle: %u Max: %u", handle, max_num_sw_vers);
		if (check_col (pinfo->cinfo, COL_INFO)) {
		         col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum);
											                                                                     }
return offset;

}

static int
rs_repadmin_dissect_get_sw_vers_info_resp (tvbuff_t *tvb, int offset,
	                         packet_info *pinfo, proto_tree *tree,
	                         char *drep)
{
	extern guint32 master_seqnum;
	guint32 num_sw_vers;
	/*      [out]   unsigned32                 *num_sw_vers,
	 *      [out, size_is(max_num_sw_vers), length_is(*num_sw_vers)]
	 *          rs_replica_sw_vers_info_t  sw_vers_info[],
	 *          	        unsigned32       sw_vers;
	 *          	        rs_sw_version_t  sw_vers_printstring;
	 *          	            typedef unsigned char           rs_sw_version_t[64];
	 *
	 *      [out]   unsigned32                 *min_sw_vers,
	 *      [out]   unsigned32                 *max_sw_vers,
	 *      [out]   unsigned32                 *cur_sw_vers,
	 *      [out]   error_status_t             *status
	 */                                                                                 

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_get_sw_vers_info_resp_num_sw_vers, &num_sw_vers);

       if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "rs_rep_admin_get_sw_vers_info Reply");
       if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


return offset;
}

static int
rs_repadmin_dissect_info_rqst (tvbuff_t *tvb, int offset,
	                         packet_info *pinfo, proto_tree *tree,
	                         char *drep)
{
	extern guint32 master_seqnum;
       if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "rs_rep_admin_info Request");

       if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
       return offset;
}

static int
rs_repadmin_dissect_info_resp (tvbuff_t *tvb, int offset,
	                         packet_info *pinfo, proto_tree *tree,
	                         char *drep)
{
       extern guint32 master_seqnum;
       guint32 base_propq_seqno_hi, base_propq_seqno_low, master, master_known, master_seqno_hi, master_seqno_low, error_status, repstate;
       e_uuid_t cell_sec_id, rep_id, init_id, master_id;
       guint32 rep_info, seqno_hi, seqno_low, last_udp_ts_sec, last_udp_ts_msec, sw_rev_hi, sw_rev_low, compat_sw_rev;

       /* 
	*         [out]   rs_replica_info_t   *rep_info,
	*         	  unsigned32              rep_state;
	*         	  uuid_t                  cell_sec_id;
	*         	  uuid_t                  rep_id;
	*         	  uuid_t                  init_id;
	*/
	offset += 4;
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_repstate, &repstate);
	offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_cell_sec_id, &cell_sec_id);
	offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_rep_id, &rep_id);
	offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_init_id, &init_id);

	/*         	  rs_update_seqno_t       last_upd_seqno;
	*                      unsigned32  high;
	*                      unsigned32  low;
	*         	  sec_timeval_t           last_upd_ts;
	*         	          signed32            sec;
	*         	          signed32            usec;
	*         	  rs_sw_version_t         sw_rev;
	*         	         unsigned32  high;
	*         	         unsigned32  low;
	*         	  unsigned32              compat_sw_rev;
	*/

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_seqno_hi, &seqno_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_seqno_low, &seqno_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_last_upd_ts_sec, &last_udp_ts_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_last_upd_ts_msec, &last_udp_ts_msec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_sw_rev_hi, &sw_rev_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_sw_rev_low, &sw_rev_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_compat_sw_rev, &compat_sw_rev);

	/*         	  # If this is info from the master, seqno of 
	*         	  # last update that has been propagated to all replicas.
	*         	  # Meaningless field if this isn't the  master
        *                 rs_update_seqno_t       base_propq_seqno;
	*                       unsigned32  high;
	*                       unsigned32  low;
	*                                     
	*                 boolean32               master;
        *	          boolean32               master_known;
        *	          uuid_t                  master_id;
        *                 # Seqno when became master
        *	          rs_update_seqno_t       master_seqno;
	*                      unsigned32  high;
	*                      unsigned32  low;
	*                 
	*         [out]   error_status_t      *status
	*                 unsigned32
	*/                  
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_base_propq_seqno_hi, &base_propq_seqno_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_base_propq_seqno_low, &base_propq_seqno_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_master, &master);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_master_known, &master_known);
	offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_master_id, &master_id);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_master_seqno_hi, &master_seqno_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_master_seqno_low, &master_seqno_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_rs_repadmin_info_resp_error_status, &error_status);

       if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "rs_rep_admin_info Reply repstate: %u", repstate);

       if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
       return offset;
}

static int
rs_repadmin_dissect_info_full_rqst (tvbuff_t *tvb, int offset,
	                         packet_info *pinfo, proto_tree *tree,
	                         char *drep)
{
       extern guint32 master_seqnum;



       if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "rs_rep_admin_info_full Request");
       if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
       return offset;
}

static int
rs_repadmin_dissect_info_full_resp (tvbuff_t *tvb, int offset,
	                         packet_info *pinfo, proto_tree *tree,
	                         char *drep)
{
       extern guint32 master_seqnum;



       if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "rs_rep_admin_info_full Reply");
       if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
       return offset;
}

	 
static dcerpc_sub_dissector rs_repadm_dissectors[] = {
    { 0, "rs_rep_admin_stop", NULL, NULL},
    { 1, "rs_rep_admin_maint", NULL, NULL},
    { 2, "rs_rep_admin_mkey", NULL, NULL},
    { 3, "rs_rep_admin_info", rs_repadmin_dissect_info_rqst, rs_repadmin_dissect_info_resp},
    { 4, "rs_rep_admin_info_full", rs_repadmin_dissect_info_full_rqst, rs_repadmin_dissect_info_full_resp},
    { 5, "rs_rep_admin_destroy", NULL, NULL},
    { 6, "rs_rep_admin_init_replica", NULL, NULL},
    { 7, "rs_rep_admin_change_master", NULL, NULL},
    { 8, "rs_rep_admin_become_master", NULL, NULL},
    { 9, "rs_rep_admin_become_slave", NULL, NULL},
    { 10, "rs_rep_admin_set_sw_rev", NULL, NULL},
    { 11, "rs_rep_admin_get_sw_vers_info", rs_repadmin_dissect_get_sw_vers_info_rqst, rs_repadmin_dissect_get_sw_vers_info_resp},
    { 0, NULL, NULL, NULL },
};

void
proto_register_rs_repadm (void)
{
	static hf_register_info hf[] = {
{ &hf_rs_repadmin_get_sw_vers_info_rqst_handle,
       {"hf_rs_repadmin_get_sw_vers_info_rqst_handle", "hf_rs_repadmin_get_sw_vers_info_rqst_handle", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_get_sw_vers_info_rqst_max_num_sw_vers,
       {"hf_rs_repadmin_get_sw_vers_info_rqst_max_num_sw_vers ", "hf_rs_repadmin_get_sw_vers_info_rqst_max_num_sw_vers", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_get_sw_vers_info_resp_num_sw_vers,
       {"hf_rs_repadmin_get_sw_vers_info_resp_num_sw_vers", "hf_rs_repadmin_get_sw_vers_info_resp_num_sw_vers", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_repstate,
{"hf_rs_repadmin_info_resp_repstate", "hf_rs_repadmin_info_resp_repstate", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_cell_sec_id,
{"hf_rs_repadmin_info_resp_cell_sec_id", "hf_rs_repadmin_info_resp_cell_sec_id", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_rs_repadmin_info_resp_rep_id,
{"hf_rs_repadmin_info_resp_rep_id", "hf_rs_repadmin_info_resp_rep_id", FT_STRING, BASE_DEC, NULL, 0x0, "UUID", HFILL }},
{ &hf_rs_repadmin_info_resp_init_id,
{"hf_rs_repadmin_info_resp_init_id", "hf_rs_repadmin_info_resp_init_id", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_rs_repadmin_info_resp_seqno_hi,
{"hf_rs_repadmin_info_resp_seqno_hi", "hf_rs_repadmin_info_resp_seqno_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_seqno_low,
{"hf_rs_repadmin_info_resp_seqno_low", "hf_rs_repadmin_info_resp_seqno_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_last_upd_ts_sec,
{"hf_rs_repadmin_info_resp_last_upd_ts_sec", "hf_rs_repadmin_info_resp_last_upd_ts_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_last_upd_ts_msec,
{"hf_rs_repadmin_info_resp_last_upd_ts_msec", "hf_rs_repadmin_info_resp_last_upd_ts_msec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_sw_rev_hi,
{"hf_rs_repadmin_info_resp_sw_rev_hi", "hf_rs_repadmin_info_resp_sw_rev_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_sw_rev_low,
{"hf_rs_repadmin_info_resp_sw_rev_low", "hf_rs_repadmin_info_resp_sw_rev_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_compat_sw_rev,
{"hf_rs_repadmin_info_resp_compat_sw_rev", "hf_rs_repadmin_info_resp_compat_sw_rev", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_base_propq_seqno_hi,
{"hf_rs_repadmin_info_resp_base_propq_seqno_hi", "hf_rs_repadmin_info_resp_base_propq_seqno_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_base_propq_seqno_low,
{"hf_rs_repadmin_info_resp_base_propq_seqno_low", "hf_rs_repadmin_info_resp_base_propq_seqno_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_master,
{"hf_rs_repadmin_info_resp_master", "hf_rs_repadmin_info_resp_master", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_master_known,
{"hf_rs_repadmin_info_resp_master_known", "hf_rs_repadmin_info_resp_master_known", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_master_id,
{"hf_rs_repadmin_info_resp_master_id", "hf_rs_repadmin_info_resp_master_id", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_rs_repadmin_info_resp_master_seqno_hi,
{"hf_rs_repadmin_info_resp_master_seqno_hi", "hf_rs_repadmin_info_resp_master_seqno_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_master_seqno_low,
{"hf_rs_repadmin_info_resp_master_seqno_low", "hf_rs_repadmin_info_resp_master_seqno_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_rs_repadmin_info_resp_error_status,
{"hf_rs_repadmin_info_resp_error_status", "hf_rs_repadmin_info_resp_error_status", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},


	};

	static gint *ett[] = {
		&ett_rs_repadm,
	};
	proto_rs_repadm = proto_register_protocol ("Registry server administration operations.", "RS_REPADM", "rs_repadm");
	proto_register_field_array (proto_rs_repadm, hf, array_length (hf));
	proto_register_subtree_array (ett, array_length (ett));
}

void
proto_reg_handoff_rs_repadm (void)
{
	/* Register the protocol as dcerpc */
	dcerpc_init_uuid (proto_rs_repadm, ett_rs_repadm, &uuid_rs_repadm, ver_rs_repadm, rs_repadm_dissectors, -1);
}
