/* packet-dcerpc-secidmap.c
 *
 * Routines for dcerpc  DCE Security ID Mapper
 * Copyright 2002, Jaime Fournier <jafour1@yahoo.com>
 * This information is based off the released idl files from opengroup.
 * ftp://opengroup.org/
 *      
 * $Id: packet-dcerpc-secidmap.c,v 1.2 2002/01/21 07:36:33 guy Exp $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif


#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <string.h>

#include <glib.h>
#include <epan/packet.h>
#include "packet-dcerpc.h"

static int proto_secidmap = -1;

static int hf_secidmap_rsec_id_parse_name_cache_rqst_selector_t = -1;
static int hf_secidmap_rsec_id_parse_name_cache_rqst_name_domain = -1;
static int hf_secidmap_rsec_id_parse_name_cache_rqst_global_name = -1;
static int hf_secidmap_rsec_id_parse_name_cache_rqst_selector = -1;
static int hf_secidmap_rsec_id_gen_name_cache_rqst_name_domain = -1;
static int hf_secidmap_rsec_id_gen_name_cache_rqst_cell_idp = -1;
static int hf_secidmap_rsec_id_gen_name_cache_rqst_princ_idp = -1;
static int hf_secidmap_rsec_id_gen_name_cache_rqst_selector = -1;
static gint ett_secidmap = -1;

static e_uuid_t uuid_secidmap = { 0x0d7c1e50, 0x113a, 0x11ca, { 0xb7, 0x1f, 0x08, 0x00, 0x1e, 0x01, 0xdc, 0x6c } };
static guint16  ver_secidmap = 1;

static int
secidmap_dissect_rsec_id_parse_name_cache_rqst (tvbuff_t *tvb, int offset,
		                 packet_info *pinfo, proto_tree *tree,
				 char *drep)
{
       guint32 name_domain, global_name, selector;
       char *selector_t = NULL;

      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_secidmap_rsec_id_parse_name_cache_rqst_name_domain, &name_domain);
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_secidmap_rsec_id_parse_name_cache_rqst_global_name, &global_name);
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_secidmap_rsec_id_parse_name_cache_rqst_selector, &selector);

      proto_tree_add_string (tree, hf_secidmap_rsec_id_parse_name_cache_rqst_selector_t, tvb, offset, selector, tvb_get_ptr (tvb, offset, selector));

      selector_t = tvb_get_ptr(tvb,offset,selector);
      offset += selector;

      if (check_col(pinfo->cinfo, COL_INFO)) col_clear(pinfo->cinfo, COL_INFO);
      if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "rsec_id_parse_name_cache Request for: %s", selector_t );

      return offset;
}


static int
secidmap_dissect_rsec_id_gen_name_cache_rqst (tvbuff_t *tvb, int offset,
		                 packet_info *pinfo, proto_tree *tree,
				 char *drep)
{
       e_uuid_t  uuid_cell, uuid_princ;
       guint32 name_domain, selector;

      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_secidmap_rsec_id_gen_name_cache_rqst_name_domain, &name_domain);
      offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_secidmap_rsec_id_gen_name_cache_rqst_cell_idp, &uuid_cell);
      offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_secidmap_rsec_id_gen_name_cache_rqst_princ_idp, &uuid_princ);

      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_secidmap_rsec_id_gen_name_cache_rqst_selector, &selector);

      if (check_col(pinfo->cinfo, COL_INFO)) col_clear(pinfo->cinfo, COL_INFO);
      if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO,
	 "rsec_id_gen_name_cache Request Cell: %08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x  Princ: %08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x Selector: %u",
		 uuid_cell.Data1, uuid_cell.Data2, uuid_cell.Data3,
		 uuid_cell.Data4[0], uuid_cell.Data4[1],
		 uuid_cell.Data4[2], uuid_cell.Data4[3],
		 uuid_cell.Data4[4], uuid_cell.Data4[5],
		 uuid_cell.Data4[6], uuid_cell.Data4[7],
		 uuid_princ.Data1, uuid_princ.Data2, uuid_princ.Data3,
		 uuid_princ.Data4[0], uuid_princ.Data4[1],
		 uuid_princ.Data4[2], uuid_princ.Data4[3],
		 uuid_princ.Data4[4], uuid_princ.Data4[5],
		 uuid_princ.Data4[6], uuid_princ.Data4[7],
	         selector);

      return offset;
}


static dcerpc_sub_dissector secidmap_dissectors[] = {
	{ 0, "rsec_id_parse_name redbox", NULL, NULL},
	{ 1, "rsec_id_gen_name bluebox", NULL, NULL},
	{ 2, "rsec_id_avoid_cn_bug", NULL, NULL},
	{ 3, "rsec_id_parse_name_cache", secidmap_dissect_rsec_id_parse_name_cache_rqst, NULL},
	{ 4, "rsec_id_gen_name_cache", secidmap_dissect_rsec_id_gen_name_cache_rqst, NULL},
        { 0, NULL, NULL, NULL },
};


void
proto_register_secidmap (void)
{
	static hf_register_info hf[] = {
{ &hf_secidmap_rsec_id_parse_name_cache_rqst_name_domain,
        {"hf_secidmap_rsec_id_parse_name_cache_rqst_name_domain", "secidmap.hf_secidmap_rsec_id_parse_name_cache_rqst_name_domain", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_secidmap_rsec_id_parse_name_cache_rqst_global_name,
        {"hf_secidmap_rsec_id_parse_name_cache_rqst_global_name", "secidmap.hf_secidmap_rsec_id_parse_name_cache_rqst_global_name", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_secidmap_rsec_id_parse_name_cache_rqst_selector,
        {"hf_secidmap_rsec_id_parse_name_cache_rqst_selector", "secidmap.hf_secidmap_rsec_id_parse_name_cache_rqst_selector", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_secidmap_rsec_id_parse_name_cache_rqst_selector_t,
        {"hf_secidmap_rsec_id_parse_name_cache_rqst_selector_t", "secidmap.hf_secidmap_rsec_id_parse_name_cache_rqst_selector_t", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_secidmap_rsec_id_gen_name_cache_rqst_name_domain,
        {"hf_secidmap_rsec_id_gen_name_cache_rqst_name_domain", "secidmap.gen_cache_name_domain", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
{ &hf_secidmap_rsec_id_gen_name_cache_rqst_cell_idp,
        {"CELL UUID", "secidmap.gen_cache_cell_idp", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_secidmap_rsec_id_gen_name_cache_rqst_princ_idp,
        {"PRINC UUID", "secidmap.gen_cache_princ_idp", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_secidmap_rsec_id_gen_name_cache_rqst_selector,
        {"Selector", "secidmap.gen_cache_selector", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
       };

	static gint *ett[] = {
		&ett_secidmap,
	};
	proto_secidmap = proto_register_protocol ("DCE Security ID Mapper", "SECIDMAP", "secidmap");
	proto_register_field_array (proto_secidmap, hf, array_length (hf));
	proto_register_subtree_array (ett, array_length (ett));
}

void
proto_reg_handoff_secidmap (void)
{
        /* Register the protocol as dcerpc */
        dcerpc_init_uuid (proto_secidmap, ett_secidmap, &uuid_secidmap, ver_secidmap, secidmap_dissectors, -1);
}
			   
