/* packet-dcerpc-AFS4Int.c
 *
 * Routines for dcerpc Afs4Int dissection
 * Copyright 2002, Jaime Fournier <jafour1@yahoo.com> 
 * This information is based off the released idl files from opengroup.
 * ftp://opengroup.org/
 *
 * $Id: packet-dcerpc-AFS4Int.c,v 1.2 2002/09/08 07:36:33 jaf Exp $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif


#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <string.h>

#include <glib.h>
#include <epan/packet.h>
#include "packet-dcerpc.h"

static int hf_afs4int_makedir_rqst_typeuuid = -1;
static int hf_afs4int_setcontext_resp_var1 = -1;
static int hf_afs4int_setcontext_resp_var2 = -1;
static int hf_afs4int_makedir_rqst_key_t = -1;
static int hf_afs4int_makedir_rqst_key_size = -1;
static int hf_afs4int_makedir_rqst_tn_tag = -1;
static int hf_afs4int_makedir_rqst_tn_length = -1;
static int hf_afs4int_makedir_rqst_cell_high = -1;
static int hf_afs4int_makedir_rqst_cell_low = -1;
static int hf_afs4int_makedir_rqst_volume_high = -1;
static int hf_afs4int_makedir_rqst_volume_low = -1;
static int hf_afs4int_makedir_rqst_vnode = -1;
static int hf_afs4int_makedir_rqst_unique = -1;
static int hf_afs4int_makedir_rqst_cell_high2 = -1;
static int hf_afs4int_makedir_rqst_cell_low2 = -1;
static int hf_afs4int_makedir_rqst_volume_high2 = -1;
static int hf_afs4int_makedir_rqst_volume_low2 = -1;
static int hf_afs4int_makedir_rqst_vnode2 = -1;
static int hf_afs4int_makedir_rqst_unique2 = -1;
static int hf_afs4int_makedir_rqst_mask = -1;
static int hf_afs4int_makedir_rqst_modtime_sec = -1;
static int hf_afs4int_makedir_rqst_modtime_usec = -1;
static int hf_afs4int_makedir_rqst_accesstime_sec = -1;
static int hf_afs4int_makedir_rqst_accesstime_usec = -1;
static int hf_afs4int_makedir_rqst_changetime_sec = -1;
static int hf_afs4int_makedir_rqst_changetime_usec = -1;
static int hf_afs4int_makedir_rqst_owner = -1;
static int hf_afs4int_makedir_rqst_group = -1;
static int hf_afs4int_makedir_rqst_mode = -1;
static int hf_afs4int_makedir_rqst_trunc_high = -1;
static int hf_afs4int_makedir_rqst_trunc_low = -1;
static int hf_afs4int_makedir_rqst_length_high = -1;
static int hf_afs4int_makedir_rqst_length_low = -1;
static int hf_afs4int_makedir_rqst_devicetype = -1;
static int hf_afs4int_makedir_rqst_devicenumber = -1;
static int hf_afs4int_makedir_rqst_cmask = -1;
static int hf_afs4int_makedir_rqst_clientspare1 = -1;
static int hf_afs4int_makedir_rqst_devicenumberhighbits = -1;
static int hf_afs4int_makedir_rqst_spare1 = -1;
static int hf_afs4int_makedir_rqst_spare2 = -1;
static int hf_afs4int_makedir_rqst_spare3 = -1;
static int hf_afs4int_makedir_rqst_spare4 = -1;
static int hf_afs4int_makedir_rqst_spare5 = -1;
static int hf_afs4int_makedir_rqst_spare6 = -1;
static int hf_afs4int_makedir_rqst_vvp_high = -1;
static int hf_afs4int_makedir_rqst_vvp_low = -1;
static int hf_afs4int_makedir_rqst_flags = -1;
static int hf_afs4int_createfile_rqst_typeuuid = -1;
static int hf_afs4int_createfile_rqst_key_t = -1;
static int hf_afs4int_createfile_rqst_key_size = -1;
static int hf_afs4int_createfile_rqst_tn_tag = -1;
static int hf_afs4int_createfile_rqst_tn_length = -1;
static int hf_afs4int_createfile_rqst_cell_high = -1;
static int hf_afs4int_createfile_rqst_cell_low = -1;
static int hf_afs4int_createfile_rqst_volume_high = -1;
static int hf_afs4int_createfile_rqst_volume_low = -1;
static int hf_afs4int_createfile_rqst_vnode = -1;
static int hf_afs4int_createfile_rqst_unique = -1;
static int hf_afs4int_createfile_rqst_cell_high2 = -1;
static int hf_afs4int_createfile_rqst_cell_low2 = -1;
static int hf_afs4int_createfile_rqst_volume_high2 = -1;
static int hf_afs4int_createfile_rqst_volume_low2 = -1;
static int hf_afs4int_createfile_rqst_vnode2 = -1;
static int hf_afs4int_createfile_rqst_unique2 = -1;
static int hf_afs4int_createfile_rqst_mask = -1;
static int hf_afs4int_createfile_rqst_modtime_sec = -1;
static int hf_afs4int_createfile_rqst_modtime_usec = -1;
static int hf_afs4int_createfile_rqst_accesstime_sec = -1;
static int hf_afs4int_createfile_rqst_accesstime_usec = -1;
static int hf_afs4int_createfile_rqst_changetime_sec = -1;
static int hf_afs4int_createfile_rqst_changetime_usec = -1;
static int hf_afs4int_createfile_rqst_owner = -1;
static int hf_afs4int_createfile_rqst_group = -1;
static int hf_afs4int_createfile_rqst_mode = -1;
static int hf_afs4int_createfile_rqst_trunc_high = -1;
static int hf_afs4int_createfile_rqst_trunc_low = -1;
static int hf_afs4int_createfile_rqst_length_high = -1;
static int hf_afs4int_createfile_rqst_length_low = -1;
static int hf_afs4int_createfile_rqst_devicetype = -1;
static int hf_afs4int_createfile_rqst_devicenumber = -1;
static int hf_afs4int_createfile_rqst_cmask = -1;
static int hf_afs4int_createfile_rqst_clientspare1 = -1;
static int hf_afs4int_createfile_rqst_devicenumberhighbits = -1;
static int hf_afs4int_createfile_rqst_spare1 = -1;
static int hf_afs4int_createfile_rqst_spare2 = -1;
static int hf_afs4int_createfile_rqst_spare3 = -1;
static int hf_afs4int_createfile_rqst_spare4 = -1;
static int hf_afs4int_createfile_rqst_spare5 = -1;
static int hf_afs4int_createfile_rqst_spare6 = -1;
static int hf_afs4int_createfile_rqst_vvp_high = -1;
static int hf_afs4int_createfile_rqst_vvp_low = -1;
static int hf_afs4int_createfile_rqst_flags = -1;
static int hf_afs4int_removefile_rqst_cell_high = -1;
static int hf_afs4int_removefile_rqst_cell_low = -1;
static int hf_afs4int_removefile_rqst_volume_high = -1;
static int hf_afs4int_removefile_rqst_volume_low = -1;
static int hf_afs4int_removefile_rqst_tn = -1;
static int hf_afs4int_removefile_rqst_tn2 = -1;
static int hf_afs4int_removefile_rqst_key_size = -1;
static int hf_afs4int_removefile_rqst_key_t = -1;
static int hf_afs4int_removefile_rqst_vnode = -1;
static int hf_afs4int_removefile_rqst_unique = -1;
static int hf_afs4int_removefile_rqst_cell_high2 = -1;
static int hf_afs4int_removefile_rqst_volume_high2 = -1;
static int hf_afs4int_removefile_rqst_volume_low2 = -1;
static int hf_afs4int_removefile_rqst_vnode2 = -1;
static int hf_afs4int_removefile_rqst_cell_low2 = -1;
static int hf_afs4int_removefile_rqst_unique2 = -1;
static int hf_afs4int_storedata_rqst_cell_high = -1;
static int hf_afs4int_storedata_rqst_cell_low = -1;
static int hf_afs4int_storedata_rqst_volume_high = -1;
static int hf_afs4int_storedata_rqst_volume_low = -1;
static int hf_afs4int_storedata_rqst_vnode = -1;
static int hf_afs4int_storedata_rqst_unique = -1;
static int hf_afs4int_gettoken_rqst_cell_high = -1;
static int hf_afs4int_gettoken_rqst_cell_low = -1;
static int hf_afs4int_gettoken_rqst_volume_high = -1;
static int hf_afs4int_gettoken_rqst_volume_low = -1;
static int hf_afs4int_gettoken_rqst_vnode = -1;
static int hf_afs4int_gettoken_rqst_unique = -1;
static int hf_afs4int_gettoken_rqst_tokenid_hi = -1;
static int hf_afs4int_gettoken_rqst_tokenid_low = -1;
static int hf_afs4int_gettoken_rqst_expirationtime = -1;
static int hf_afs4int_gettoken_rqst_type_hi = -1;
static int hf_afs4int_gettoken_rqst_type_low = -1;
static int hf_afs4int_gettoken_rqst_beginrange = -1;
static int hf_afs4int_gettoken_rqst_endrange = -1;
static int hf_afs4int_gettoken_rqst_beginrangeext = -1;
static int hf_afs4int_gettoken_rqst_endrangeext = -1;
static int hf_afs4int_gettoken_rqst_minvvp_hi = -1;
static int hf_afs4int_gettoken_rqst_minvvp_low = -1;
static int hf_afs4int_gettoken_rqst_flags = -1;
static int hf_afs4int_lookuproot_rqst_cell_high = -1;
static int hf_afs4int_lookuproot_rqst_cell_low = -1;
static int hf_afs4int_lookuproot_rqst_volume_high = -1;
static int hf_afs4int_lookuproot_rqst_volume_low = -1;
static int hf_afs4int_lookuproot_rqst_vnode = -1;
static int hf_afs4int_lookuproot_rqst_unique = -1;
static int hf_afs4int_fetchdata_rqst_cell_high = -1;
static int hf_afs4int_fetchdata_rqst_cell_low = -1;
static int hf_afs4int_fetchdata_rqst_volume_high = -1;
static int hf_afs4int_fetchdata_rqst_volume_low = -1;
static int hf_afs4int_fetchdata_rqst_vnode = -1;
static int hf_afs4int_fetchdata_rqst_unique = -1;
static int hf_afs4int_fetchacl_rqst_cell_high = -1;
static int hf_afs4int_fetchacl_rqst_cell_low = -1;
static int hf_afs4int_fetchacl_rqst_volume_high = -1;
static int hf_afs4int_fetchacl_rqst_volume_low = -1;
static int hf_afs4int_fetchacl_rqst_vnode = -1;
static int hf_afs4int_fetchacl_rqst_unique = -1;
static int hf_afs4int_fetchstatus_rqst_cell_high = -1;
static int hf_afs4int_fetchstatus_rqst_cell_low = -1;
static int hf_afs4int_fetchstatus_rqst_volume_high = -1;
static int hf_afs4int_fetchstatus_rqst_volume_low = -1;
static int hf_afs4int_fetchstatus_rqst_vnode = -1;
static int hf_afs4int_fetchstatus_rqst_unique = -1;
static int hf_afs4int_storeacl_rqst_cell_high = -1;
static int hf_afs4int_storeacl_rqst_cell_low = -1;
static int hf_afs4int_storeacl_rqst_volume_high = -1;
static int hf_afs4int_storeacl_rqst_volume_low = -1;
static int hf_afs4int_storeacl_rqst_vnode = -1;
static int hf_afs4int_storeacl_rqst_unique = -1;
static int hf_afs4int_storeacl_rqst_key_size = -1;
static int hf_afs4int_storeacl_rqst_key_t = -1;
static int hf_afs4int_storeacl_rqst_var1 = -1;
static int hf_afs4int_storeacl_rqst_var2 = -1;
static int hf_afs4int_storestatus_rqst_cell_high = -1;
static int hf_afs4int_storestatus_rqst_cell_low = -1;
static int hf_afs4int_storestatus_rqst_volume_high = -1;
static int hf_afs4int_storestatus_rqst_volume_low = -1;
static int hf_afs4int_storestatus_rqst_vnode = -1;
static int hf_afs4int_storestatus_rqst_unique = -1;
static int hf_afs4int_storestatus_rqst_mask = -1;
static int hf_afs4int_storestatus_rqst_modtime_sec = -1;
static int hf_afs4int_storestatus_rqst_modtime_usec = -1;
static int hf_afs4int_storestatus_rqst_accesstime_sec = -1;
static int hf_afs4int_storestatus_rqst_accesstime_usec = -1;
static int hf_afs4int_storestatus_rqst_changetime_sec = -1;
static int hf_afs4int_storestatus_rqst_changetime_usec = -1;
static int hf_afs4int_storestatus_rqst_owner = -1;
static int hf_afs4int_storestatus_rqst_group = -1;
static int hf_afs4int_storestatus_rqst_mode = -1;
static int hf_afs4int_storestatus_rqst_trunc_high = -1;
static int hf_afs4int_storestatus_rqst_trunc_low = -1;
static int hf_afs4int_storestatus_rqst_length_high = -1;
static int hf_afs4int_storestatus_rqst_length_low = -1;
static int hf_afs4int_storestatus_rqst_typeuuid = -1;
static int hf_afs4int_storestatus_rqst_devicetype = -1;
static int hf_afs4int_storestatus_rqst_devicenumber = -1;
static int hf_afs4int_storestatus_rqst_cmask = -1;
static int hf_afs4int_storestatus_rqst_clientspare1 = -1;
static int hf_afs4int_storestatus_rqst_devicenumberhighbits = -1;
static int hf_afs4int_storestatus_rqst_spare1 = -1;
static int hf_afs4int_storestatus_rqst_spare2 = -1;
static int hf_afs4int_storestatus_rqst_spare3 = -1;
static int hf_afs4int_storestatus_rqst_spare4 = -1;
static int hf_afs4int_storestatus_rqst_spare5 = -1;
static int hf_afs4int_storestatus_rqst_spare6 = -1;
static int hf_afs4int_storestatus_rqst_vvp_high = -1;
static int hf_afs4int_storestatus_rqst_vvp_low = -1;
static int hf_afs4int_storestatus_rqst_flags = -1;
static int hf_afs4int_rename_rqst_cell_high_old = -1;
static int hf_afs4int_rename_rqst_cell_low_old = -1;
static int hf_afs4int_rename_rqst_volume_high_old = -1;
static int hf_afs4int_rename_rqst_volume_low_old = -1;
static int hf_afs4int_rename_rqst_vnode_old = -1;
static int hf_afs4int_rename_rqst_unique_old = -1;
static int hf_afs4int_rename_rqst_tn_old = -1;
static int hf_afs4int_rename_rqst_tn2_old = -1;
static int hf_afs4int_rename_rqst_key_size_old = -1;
static int hf_afs4int_rename_rqst_key_t_old = -1;
static int hf_afs4int_rename_rqst_cell_high_new = -1;
static int hf_afs4int_rename_rqst_cell_low_new = -1;
static int hf_afs4int_rename_rqst_volume_high_new = -1;
static int hf_afs4int_rename_rqst_volume_low_new = -1;
static int hf_afs4int_rename_rqst_vnode_new = -1;
static int hf_afs4int_rename_rqst_unique_new = -1;
static int hf_afs4int_rename_rqst_tn_new = -1;
static int hf_afs4int_rename_rqst_tn2_new = -1;
static int hf_afs4int_rename_rqst_key_size_new = -1;
static int hf_afs4int_rename_rqst_key_t_new = -1;
static int hf_afs4int_rename_rqst_tokenidp_high = -1;
static int hf_afs4int_rename_rqst_tokenidp_low = -1;
static int hf_afs4int_rename_rqst_vvp_high = -1;
static int hf_afs4int_rename_rqst_vvp_low = -1;
static int hf_afs4int_rename_rqst_flags = -1;
static int hf_afs4int_symlink_rqst_cell_high = -1;
static int hf_afs4int_symlink_rqst_cell_low = -1;
static int hf_afs4int_symlink_rqst_volume_high = -1;
static int hf_afs4int_symlink_rqst_volume_low = -1;
static int hf_afs4int_symlink_rqst_vnode = -1;
static int hf_afs4int_symlink_rqst_unique = -1;
static int hf_afs4int_readdir_rqst_cell_high = -1;
static int hf_afs4int_readdir_rqst_cell_low = -1;
static int hf_afs4int_readdir_rqst_volume_high = -1;
static int hf_afs4int_readdir_rqst_volume_low = -1;
static int hf_afs4int_readdir_rqst_vnode = -1;
static int hf_afs4int_readdir_rqst_unique = -1;
static int hf_afs4int_readdir_rqst_offset_high = -1;
static int hf_afs4int_readdir_rqst_offset_low = -1;
static int hf_afs4int_readdir_rqst_size = -1;
static int hf_afs4int_readdir_rqst_vvp_high = -1;
static int hf_afs4int_readdir_rqst_vvp_low = -1;
static int hf_afs4int_readdir_rqst_flags = -1;
static int hf_afs4int_removedir_rqst_cell_high = -1;
static int hf_afs4int_removedir_rqst_cell_low = -1;
static int hf_afs4int_removedir_rqst_volume_high = -1;
static int hf_afs4int_removedir_rqst_volume_low = -1;
static int hf_afs4int_removedir_rqst_vnode = -1;
static int hf_afs4int_removedir_rqst_unique = -1;
static int hf_afs4int_removedir_rqst_cell_high2 = -1;
static int hf_afs4int_removedir_rqst_volume_high2 = -1;
static int hf_afs4int_removedir_rqst_volume_low2 = -1;
static int hf_afs4int_removedir_rqst_vnode2 = -1;
static int hf_afs4int_removedir_rqst_cell_low2 = -1;
static int hf_afs4int_removedir_rqst_unique2 = -1;
static int hf_afs4int_removedir_rqst_key_size = -1;
static int hf_afs4int_removedir_rqst_key_t = -1;
static int hf_afs4int_removedir_rqst_tn = -1;
static int hf_afs4int_removedir_rqst_tn2 = -1;
static int hf_afs4int_lookup_rqst_cell_high = -1;
static int hf_afs4int_lookup_rqst_cell_low = -1;
static int hf_afs4int_lookup_rqst_volume_high = -1;
static int hf_afs4int_lookup_rqst_volume_low = -1;
static int hf_afs4int_lookup_rqst_vnode = -1;
static int hf_afs4int_lookup_rqst_unique = -1;
static int hf_afs4int_lookup_rqst_tn_tag = -1;
static int hf_afs4int_lookup_rqst_tn_length = -1;
static int hf_afs4int_lookup_rqst_vvp_high = -1;
static int hf_afs4int_lookup_rqst_vvp_low = -1;
static int hf_afs4int_lookup_rqst_flags = -1;
static int hf_afs4int_lookup_rqst_key_size = -1;
static int hf_afs4int_lookup_rqst_key_t = -1;
static int hf_afs4int_lookup_resp_cell_high = -1;
static int hf_afs4int_lookup_resp_cell_low = -1;
static int hf_afs4int_lookup_resp_volume_high = -1;
static int hf_afs4int_lookup_resp_volume_low = -1;
static int hf_afs4int_lookup_resp_vnode = -1;
static int hf_afs4int_lookup_resp_unique = -1;
static int hf_afs4int_makemountpoint_rqst_cell_high = -1;
static int hf_afs4int_makemountpoint_rqst_cell_low = -1;
static int hf_afs4int_makemountpoint_rqst_volume_high = -1;
static int hf_afs4int_makemountpoint_rqst_volume_low = -1;
static int hf_afs4int_makemountpoint_rqst_vnode = -1;
static int hf_afs4int_makemountpoint_rqst_unique = -1;
static int hf_afs4int_setcontext_rqst_epochtime = -1;
static int hf_afs4int_setcontext_rqst_afsnetaddr_type = -1;
static int hf_afs4int_setcontext_rqst_key_t = -1;
static int hf_afs4int_setcontext_rqst_key_size = -1;
static int hf_afs4int_setcontext_rqst_flags = -1;
static int hf_afs4int_setcontext_rqst_secobjectid = -1;
static int hf_afs4int_setcontext_rqst_clientsizesattrs = -1;
static int hf_afs4int_setcontext_rqst_parm7 = -1;
static int hf_afs4int_setcontext_rqst_data0 = -1;
static int hf_afs4int_setcontext_rqst_data1 = -1;
static int hf_afs4int_setcontext_rqst_data2 = -1;
static int hf_afs4int_setcontext_rqst_data3 = -1;
static int hf_afs4int_setcontext_rqst_data4 = -1;
static int hf_afs4int_setcontext_rqst_data5 = -1;
static int hf_afs4int_setcontext_rqst_data6 = -1;
static int hf_afs4int_setcontext_rqst_data7 = -1;
static int hf_afs4int_setcontext_rqst_data8 = -1;
static int hf_afs4int_setcontext_rqst_data9 = -1;
static int hf_afs4int_setcontext_rqst_data10 = -1;
static int hf_afs4int_setcontext_rqst_data11 = -1;
static int hf_afs4int_setcontext_rqst_data12 = -1;
static int hf_afs4int_setcontext_rqst_data13 = -1;
static int hf_afs4int_setcontext_rqst_data14 = -1;
static int hf_afs4int_gettoken_resp_objectuuid = -1;
static int hf_afs4int_gettoken_resp_typeuuid = -1;
static int hf_afs4int_gettoken_resp_spare6 = -1;
static int hf_afs4int_gettoken_resp_spare5 = -1;
static int hf_afs4int_gettoken_resp_spare4 = -1;
static int hf_afs4int_gettoken_resp_pathconfspare = -1;
static int hf_afs4int_gettoken_resp_lowmaxspare = -1;
static int hf_afs4int_gettoken_resp_himaxspare = -1;
static int hf_afs4int_gettoken_resp_agtypeunique = -1;
static int hf_afs4int_gettoken_resp_devicenumberhighbits = -1;
static int hf_afs4int_gettoken_resp_clientspare1 = -1;
static int hf_afs4int_gettoken_resp_blocksused = -1;
static int hf_afs4int_gettoken_resp_devicenumber = -1;
static int hf_afs4int_gettoken_resp_servermodtime_msec = -1;
static int hf_afs4int_gettoken_resp_servermodtime_sec = -1;
static int hf_afs4int_gettoken_resp_accesstime_msec = -1;
static int hf_afs4int_gettoken_resp_accesstime_sec = -1;
static int hf_afs4int_gettoken_resp_changetime_msec = -1;
static int hf_afs4int_gettoken_resp_changetime_sec = -1;
static int hf_afs4int_gettoken_resp_modtime_msec = -1;
static int hf_afs4int_gettoken_resp_modtime_sec = -1;
static int hf_afs4int_gettoken_resp_parentunique = -1;
static int hf_afs4int_gettoken_resp_parentvnode = -1;
static int hf_afs4int_gettoken_resp_mode = -1;
static int hf_afs4int_gettoken_resp_aclexpirationtime = -1;
static int hf_afs4int_gettoken_resp_anonymouseaccess = -1;
static int hf_afs4int_gettoken_resp_calleraccess = -1;
static int hf_afs4int_gettoken_resp_group = -1;
static int hf_afs4int_gettoken_resp_owner = -1;
static int hf_afs4int_gettoken_resp_author = -1;
static int hf_afs4int_gettoken_resp_dataversion_low = -1;
static int hf_afs4int_gettoken_resp_dataversion_high = -1;
static int hf_afs4int_gettoken_resp_length_low = -1;
static int hf_afs4int_gettoken_resp_length_high = -1;
static int hf_afs4int_gettoken_resp_linkcount = -1;
static int hf_afs4int_gettoken_resp_filetype = -1;
static int hf_afs4int_gettoken_resp_interfaceversion = -1;
static int hf_afs4int_gettoken_resp_l_end_pos_ext = -1;
static int hf_afs4int_gettoken_resp_l_start_pos_ext = -1;
static int hf_afs4int_gettoken_resp_l_fstype = -1;
static int hf_afs4int_gettoken_resp_l_sysid = -1;
static int hf_afs4int_gettoken_resp_l_pid = -1;
static int hf_afs4int_gettoken_resp_l_end_pos = -1;
static int hf_afs4int_gettoken_resp_l_start_pos = -1;
static int hf_afs4int_gettoken_resp_l_whence = -1;
static int hf_afs4int_gettoken_resp_l_type = -1;
static int hf_afs4int_gettoken_resp_vvspare2 = -1;
static int hf_afs4int_gettoken_resp_vvspare1 = -1;
static int hf_afs4int_gettoken_resp_vvpingage = -1;
static int hf_afs4int_gettoken_resp_vvage = -1;
static int hf_afs4int_gettoken_resp_vv_low = -1;
static int hf_afs4int_gettoken_resp_vv_hi = -1;
static int hf_afs4int_gettoken_resp_volid_low = -1;
static int hf_afs4int_gettoken_resp_volid_hi = -1;
static int hf_afs4int_gettoken_resp_endrangeext = -1;
static int hf_afs4int_gettoken_resp_beginrangeext = -1;
static int hf_afs4int_gettoken_resp_endrange = -1;
static int hf_afs4int_gettoken_resp_beginrange = -1;
static int hf_afs4int_gettoken_resp_type_low = -1;
static int hf_afs4int_gettoken_resp_type_hi = -1;
static int hf_afs4int_gettoken_resp_expirationtime = -1;
static int hf_afs4int_gettoken_resp_tokenid_low = -1;
static int hf_afs4int_gettoken_resp_tokenid_hi = -1;
static int hf_afs4int_gettoken_resp_anonymousaccess = -1;
static int hf_afs4int_gettoken_resp_lomaxspare = -1;
static int hf_afs4int_gettoken_resp_key_t = -1;
static int hf_afs4int_gettoken_resp_key_size = -1;
static int hf_afs4int_fetchstatus_rqst_vvp_hi = -1;
static int hf_afs4int_fetchstatus_rqst_vvp_low = -1;
static int hf_afs4int_fetchstatus_rqst_flags = -1;
static int hf_afs4int_fetchstatus_rqst_error = -1;
static int hf_afs4int_fetchstatus_resp_interfaceversion = -1;
static int hf_afs4int_fetchstatus_resp_filetype = -1;
static int hf_afs4int_fetchstatus_resp_linkcount = -1;
static int hf_afs4int_fetchstatus_resp_length_high = -1;
static int hf_afs4int_fetchstatus_resp_length_low = -1;
static int hf_afs4int_fetchstatus_resp_dataversion_high = -1;
static int hf_afs4int_fetchstatus_resp_dataversion_low = -1;
static int hf_afs4int_fetchstatus_resp_author = -1;
static int hf_afs4int_fetchstatus_resp_owner = -1;
static int hf_afs4int_fetchstatus_resp_group = -1;
static int hf_afs4int_fetchstatus_resp_calleraccess = -1;
static int hf_afs4int_fetchstatus_resp_anonymousaccess = -1;
static int hf_afs4int_fetchstatus_resp_aclexpirationtime = -1;
static int hf_afs4int_fetchstatus_resp_mode = -1;
static int hf_afs4int_fetchstatus_resp_parentvnode = -1;
static int hf_afs4int_fetchstatus_resp_parentunique = -1;
static int hf_afs4int_fetchstatus_resp_modtime_sec = -1;
static int hf_afs4int_fetchstatus_resp_modtime_msec = -1;
static int hf_afs4int_fetchstatus_resp_changetime_sec = -1;
static int hf_afs4int_fetchstatus_resp_changetime_msec = -1;
static int hf_afs4int_fetchstatus_resp_accesstime_sec = -1;
static int hf_afs4int_fetchstatus_resp_accesstime_msec = -1;
static int hf_afs4int_fetchstatus_resp_servermodtime_sec = -1;
static int hf_afs4int_fetchstatus_resp_servermodtime_msec = -1;
static int hf_afs4int_fetchstatus_resp_typeuuid = -1;
static int hf_afs4int_fetchstatus_resp_objectuuid = -1;
static int hf_afs4int_fetchstatus_resp_devicenumber = -1;
static int hf_afs4int_fetchstatus_resp_blocksused = -1;
static int hf_afs4int_fetchstatus_resp_clientspare1 = -1;
static int hf_afs4int_fetchstatus_resp_devicenumberhighbits = -1;
static int hf_afs4int_fetchstatus_resp_agtypeunique = -1;
static int hf_afs4int_fetchstatus_resp_himaxspare = -1;
static int hf_afs4int_fetchstatus_resp_lomaxspare = -1;
static int hf_afs4int_fetchstatus_resp_pathconfspare = -1;
static int hf_afs4int_fetchstatus_resp_spare4 = -1;
static int hf_afs4int_fetchstatus_resp_spare5 = -1;
static int hf_afs4int_fetchstatus_resp_spare6 = -1;
static int hf_afs4int_fetchstatus_resp_tokenid_hi = -1;
static int hf_afs4int_fetchstatus_resp_tokenid_low = -1;
static int hf_afs4int_fetchstatus_resp_expirationtime = -1;
static int hf_afs4int_fetchstatus_resp_type_hi = -1;
static int hf_afs4int_fetchstatus_resp_type_low = -1;
static int hf_afs4int_fetchstatus_resp_beginrange = -1;
static int hf_afs4int_fetchstatus_resp_endrange = -1;
static int hf_afs4int_fetchstatus_resp_beginrangeext = -1;
static int hf_afs4int_fetchstatus_resp_endrangeext = -1;
static int hf_afs4int_fetchstatus_resp_volid_hi = -1;
static int hf_afs4int_fetchstatus_resp_volid_low = -1;
static int hf_afs4int_fetchstatus_resp_vv_hi = -1;
static int hf_afs4int_fetchstatus_resp_vv_low = -1;
static int hf_afs4int_fetchstatus_resp_vvage = -1;
static int hf_afs4int_fetchstatus_resp_vvpingage = -1;
static int hf_afs4int_fetchstatus_resp_vvspare1 = -1;
static int hf_afs4int_fetchstatus_resp_vvspare2 = -1;
static int hf_afs4int_fetchstatus_resp_err1 = -1;
static int hf_afs4int_fetchstatus_resp_err2 = -1;
extern guint32 master_seqnum;

static int proto_AFS4Int = -1;

static gint ett_AFS4Int = -1;


static e_uuid_t uuid_AFS4Int = { 0x4d37f2dd, 0xed93, 0x0000, { 0x02, 0xc0, 0x37, 0xcf, 0x1e, 0x00, 0x00, 0x00 } };
static guint16  ver_AFS4Int = 4;



static int
afs4int_dissect_removefile_rqst (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
	guint64 volume, volume2;
	guint32 volume_high, volume_high2;
	guint32 volume_low, volume_low2;
	guint32 inode, vnode, vnode2, tn;
	guint32 key_size = 257;
	guint16 tn2;
	char *key_t;

         /*  [in]    afsFid          *DirFidp,
	 *         afsHyper Cell;
	 *         afsHyper Volume;
	 *         unsigned32 Vnode;
	 *         unsigned32 Unique;
	 */                                 
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_volume_low, &volume_low);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_vnode, &vnode);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_unique, NULL);

        /*
         *  [in]    afsFidTaggedName        *Namep,
	 *
	 *         afsFid fid;
	 *                afsHyper Cell;
	 *                afsHyper Volume;
	 *                unsigned32 Vnode;
	 *                unsigned32 Unique;
	 *         afsTaggedName name;
	 *          	  codesetTag      tn_tag;
	 *          	     unsigned32  tn_tag
	 *          	  unsigned16      tn_length;
	 * stuck here need max size
	 * 256 + 1
	 * so we just read in a string of size 256 to be safe
	 *          	  byte            tn_chars[AFS_NAMEMAX+1];
	 *
         *  [in]    afsHyper        *returnTokenIDp,
	 *           unsigned32 hi          4
	 *           unsigned32 low         4
         *  [in]    afsHyper        *minVVp,
	 *           unsigned32 hi          4
	 *           unsigned32 low         4
         *   [in]    unsigned32      Flags, 4
	 */
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_cell_high2, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_cell_low2, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_volume_high2, &volume_high2);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_volume_low2, &volume_low2);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_vnode2, &vnode2);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_unique2, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_tn, &tn);
        offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_removefile_rqst_tn2, &tn2);

        proto_tree_add_string (tree, hf_afs4int_removefile_rqst_key_t, tvb, offset, hf_afs4int_removefile_rqst_key_size, tvb_get_ptr (tvb, offset, key_size));
	key_t = tvb_get_ptr(tvb,offset,key_size);
	offset += key_size;

        volume = volume_high | volume_low;
        volume2 = volume_high2 | volume_low2;
	
	if (vnode == 1) {
	        if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_RemoveFile FSID:%llu InFS Name:%s", volume, key_t);

	} else {
		inode = ((volume << 16) + vnode) & 0x7fffffff;
       	        if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_RemoveFile FSID:%llu PDI:%u Name:%s", volume, inode, key_t);
	}
	          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


    return offset;
}

static int
afs4int_dissect_storedata_rqst (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
	guint64 volume=-1;
        guint32 volume_high=-1;
        guint32 volume_low=-1;
	guint32 inode, vnode=-1;

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storedata_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storedata_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storedata_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storedata_rqst_volume_low, &volume_low);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storedata_rqst_vnode, &vnode);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storedata_rqst_unique, NULL);


        volume = volume_high | volume_low;

		if (vnode == 1){
                	if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_StoreData FSID:%llu InFS ", volume);
		} else {
	     		inode = ((volume << 16) + vnode) & 0x7fffffff;
                	if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_StoreData FSID:%llu inode: %u", volume, inode);
		}

        if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
	return offset;
}

static int 
afs4int_dissect_gettoken_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 
	 guint64 volume;
         guint32 volume_high;
         guint32 volume_low;
         guint32 vnode, inode,tokenid_hi, tokenid_low, expirationtime, type_hi, type_low, beginrange, endrange, beginrangeext, endrangeext, type, minvvp_hi, minvvp_low, flags ;


#define	 AFS_FLAG_RETURNTOKEN  		1
#define  AFS_FLAG_TOKENJUMPQUEUE  	2
#define  AFS_FLAG_SKIPTOKEN  		4
#define  AFS_FLAG_NOOPTIMISM 	       0x8
#define  AFS_FLAG_TOKENID  	       0x10
#define  AFS_FLAG_RETURNBLOCKER        0x20
#define  AFS_FLAG_ASYNCGRANT           0x40
#define  AFS_FLAG_NOREVOKE             0x80
#define  AFS_FLAG_MOVE_REESTABLISH     0x100
#define  AFS_FLAG_SERVER_REESTABLISH   0x200
#define  AFS_FLAG_NO_NEW_EPOCH         0x400
#define  AFS_FLAG_MOVE_SOURCE_OK       0x800
#define  AFS_FLAG_SYNC 		       0x1000
#define  AFS_FLAG_ZERO                 0x2000
#define  AFS_FLAG_SKIPSTATUS           0x4000
#define  AFS_FLAG_FORCEREVOCATIONS     0x8000
#define  AFS_FLAG_FORCEVOLQUIESCE      0x10000
#define  AFS_FLAG_FORCEREVOCATIONDOWN  0x20000

#define TKN_LOCK_READ 			0x001
#define TKN_LOCK_WRITE			0x002
#define TKN_DATA_READ 			0x004
#define TKN_DATA_WRITE 			0x008
#define TKN_OPEN_READ 			0x010
#define TKN_OPEN_WRITE 			0x020
#define TKN_OPEN_SHARED 		0x040
#define TKN_OPEN_EXCLUSIVE 		0x080
#define TKN_OPEN_DELETE 		0x100
#define TKN_OPEN_PRESERVE 		0x200
#define TKN_STATUS_READ 		0x400
#define TKN_STATUS_WRITE 		0x800
#define TKN_OPEN_UNLINK 		0x1000
#define TKN_SPOT_HERE 			0x2000
#define TKN_SPOT_THERE 			0x4000
#define TKN_OPEN_NO_READ 		0x8000
#define TKN_OPEN_NO_WRITE 		0x10000
#define TKN_OPEN_NO_UNLINK 		0x20000


	/* 
        * [in]    afsFid          *Fidp,
	*/
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_unique, NULL);

	/*
        * [in]    afsToken        *MinTokenp,
	* unsigned32 tokenID_hi;
	* unsigned32 tokenID_lo;
        * 	     # afsHyper tokenID;
	* unsigned32 expirationTime;
	* unsigned32 type_hi;
	* unsigned32 type_lo;
        *     	     # afsHyper type;
	* unsigned32 beginRange;
	* unsigned32 endRange;
	* unsigned32 beginRangeExt;
	* unsigned32 endRangeExt;
        */

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_tokenid_hi, &tokenid_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_tokenid_low, &tokenid_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_expirationtime, &expirationtime);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_type_hi, &type_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_type_low, &type_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_beginrange, &beginrange);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_endrange, &endrange);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_beginrangeext, &beginrangeext);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_endrangeext, &endrangeext);

	/*
        * [in]    afsHyper        *minVVp,
        * [in]    unsigned32      Flags,
	*/

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_minvvp_hi, &minvvp_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_minvvp_low, &minvvp_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_rqst_flags, &flags);
	type = type_hi | type_low;
	volume = volume_high | volume_low;


	if (vnode == 1) {

            if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_GetToken FSID:%llu FileSetToken", volume);

	} else {

     	    inode = ((volume << 16) + vnode) & 0x7fffffff;

            if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_GetToken FSID:%llu inode: %u", volume, inode);

	}
	

   
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, " Type=");

 if ((type & TKN_LOCK_READ) == TKN_LOCK_READ) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":LOCK_READ");
}
 if ((type & TKN_LOCK_WRITE) == TKN_LOCK_WRITE) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":LOCK_WRITE");
}
 if ((type & TKN_DATA_READ) == TKN_DATA_READ) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":DATA_READ");
}
 if ((type & TKN_DATA_WRITE) == TKN_DATA_WRITE) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":DATA_WRITE");
}
 if ((type & TKN_OPEN_READ) == TKN_OPEN_READ) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":OPEN_READ");
}
 if ((type & TKN_OPEN_WRITE) == TKN_OPEN_WRITE) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":OPEN_WRITE");
}
 if ((type & TKN_OPEN_SHARED) == TKN_OPEN_SHARED) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":OPEN_SHARED");
}
 if ((type & TKN_OPEN_EXCLUSIVE) == TKN_OPEN_EXCLUSIVE) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":OPEN_EXCLUSIVE");
}
 if ((type & TKN_OPEN_DELETE) == TKN_OPEN_DELETE) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":OPEN_DELETE");
}
 if ((type & TKN_OPEN_PRESERVE) == TKN_OPEN_PRESERVE) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":OPEN_PRESERVE");
}
 if ((type & TKN_STATUS_READ) == TKN_STATUS_READ) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":STATUS_READ");
}
 if ((type & TKN_STATUS_WRITE) == TKN_STATUS_WRITE) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":STATUS_WRITE");
}
 if ((type & TKN_OPEN_UNLINK) == TKN_OPEN_UNLINK) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":OPEN_UNLINK");
}
 if ((type & TKN_SPOT_HERE) == TKN_SPOT_HERE) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":SPOT_HERE");
}
 if ((type & TKN_SPOT_THERE) == TKN_SPOT_THERE) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":SPOT_THERE");
}
 if ((type & TKN_OPEN_NO_READ) == TKN_OPEN_NO_READ) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":OPEN_NO_READ");
}
 if ((type & TKN_OPEN_NO_WRITE) == TKN_OPEN_NO_WRITE) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":OPEN_NO_WRITE");
}
 if ((type & TKN_OPEN_NO_UNLINK) == TKN_OPEN_NO_UNLINK) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":OPEN_NO_UNLINK");
}


/* Code to check all the flags on flags, and add the proper line  */
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, " Flags=");

if ((flags & AFS_FLAG_RETURNTOKEN) == AFS_FLAG_RETURNTOKEN) {
     if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":ReturnToken");
}
if ((flags & AFS_FLAG_TOKENJUMPQUEUE) == AFS_FLAG_TOKENJUMPQUEUE) {
     if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":TokenJumpQueue");
}
 if ((flags & AFS_FLAG_SKIPTOKEN) == AFS_FLAG_SKIPTOKEN) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":SkipToken");
}
 if ((flags & AFS_FLAG_NOOPTIMISM) == AFS_FLAG_NOOPTIMISM) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":NoOptimism");
}
 if ((flags & AFS_FLAG_TOKENID ) == AFS_FLAG_TOKENID ) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":TokenID");
}
 if ((flags & AFS_FLAG_RETURNBLOCKER ) == AFS_FLAG_RETURNBLOCKER ) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":ReturnBlocker");
}
 if ((flags & AFS_FLAG_ASYNCGRANT ) == AFS_FLAG_ASYNCGRANT ) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":AsynGrant");
}
 if ((flags & AFS_FLAG_NOREVOKE ) == AFS_FLAG_NOREVOKE ) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":NoRevoke");
}
 if ((flags & AFS_FLAG_MOVE_REESTABLISH ) == AFS_FLAG_MOVE_REESTABLISH ) {
     if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":MoveReestablish");
}
 if ((flags & AFS_FLAG_SERVER_REESTABLISH ) == AFS_FLAG_SERVER_REESTABLISH ) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":ServerReestablish");
}
 if ((flags & AFS_FLAG_NO_NEW_EPOCH ) == AFS_FLAG_NO_NEW_EPOCH ) {
     if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":NewEpoch");
}
 if ((flags & AFS_FLAG_MOVE_SOURCE_OK ) == AFS_FLAG_MOVE_SOURCE_OK ) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":MoveSourceOk");
}
 if ((flags & AFS_FLAG_SYNC ) == AFS_FLAG_SYNC ) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":Sync");
}
 if ((flags & AFS_FLAG_ZERO ) == AFS_FLAG_ZERO ) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":Zero");
}
 if ((flags & AFS_FLAG_SKIPSTATUS ) == AFS_FLAG_SKIPSTATUS ) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":SkipStatus");
}
 if ((flags & AFS_FLAG_FORCEREVOCATIONS ) == AFS_FLAG_FORCEREVOCATIONS ) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":ForceRevocation");
}
 if ((flags & AFS_FLAG_FORCEVOLQUIESCE ) == AFS_FLAG_FORCEVOLQUIESCE ) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":ForceVolQuiesce");
}
 if ((flags & AFS_FLAG_FORCEREVOCATIONDOWN ) == AFS_FLAG_FORCEREVOCATIONDOWN ) {
	      if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":ForceRevocationDown");
}
          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }



    return offset;
}
static int 
afs4int_dissect_gettoken_resp (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
         guint32 tokenid_hi, tokenid_low, expirationtime, type_hi, type_low, beginrange, endrange, beginrangeext, endrangeext;
	 char *key_t;
	 guint32 key_size = 150;
	 guint32 volid_hi, volid_low, vv_hi, vv_low, vvage, vvpingage, vvspare1, vvspare2;
	 guint16 l_type, l_whence;
	 guint32 l_start_pos, l_end_pos, l_pid, l_sysid, l_fstype, l_start_pos_ext, l_end_pos_ext;
	 guint32 interfaceversion, filetype, linkcount, length_high, length_low, dataversion_high, dataversion_low, author, owner, group, calleraccess, anonymousaccess, aclexpirationtime, mode, parentvnode, parentunique, modtime_sec, modtime_msec, changetime_sec, changetime_msec, accesstime_sec, accesstime_msec, servermodtime_sec, servermodtime_msec, devicenumber, blocksused, clientspare1, devicenumberhighbits, agtypeunique, himaxspare, lomaxspare, pathconfspare, spare4, spare5, spare6;
	 e_uuid_t typeuuid, objectuuid;

	/*
	 *         [out]   afsToken        *OutTokenp,
	 *          unsigned32 tokenID_hi;
	 *          unsigned32 tokenID_lo;
	 *          unsigned32 expirationTime;
	 *          unsigned32 type_hi;
         *	    unsigned32 type_lo;
   	 *          unsigned32 beginRange;
	 *          unsigned32 endRange;
         *	    unsigned32 beginRangeExt;
         *	    unsigned32 endRangeExt;
	 */ 

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_tokenid_hi, &tokenid_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_tokenid_low, &tokenid_low);
	if (tokenid_low){
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_expirationtime, &expirationtime);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_type_hi, &type_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_type_low, &type_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_beginrange, &beginrange);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_endrange, &endrange);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_beginrangeext, &beginrangeext);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_endrangeext, &endrangeext);
/* XXX */

	/*
	 *         [out]   afsRecordLock   *OutBlockerp,
	 *             signed16     l_type;
	 *             signed16     l_whence;
	 *             unsigned32   l_start_pos;
	 *             unsigned32   l_end_pos;
	 *             unsigned32   l_pid;
	 *             unsigned32   l_sysid;
	 *             unsigned32   l_fstype;
	 *             unsigned32   l_start_pos_ext; was l_spare0: high 32b of l_start_pos
	 *             unsigned32   l_end_pos_ext; was l_spare1: high 32b of l_end_pos
	 */
	offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_l_type, &l_type);
	offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_l_whence, &l_whence);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_l_start_pos, &l_start_pos);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_l_end_pos, &l_end_pos);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_l_pid, &l_pid);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_l_sysid, &l_sysid);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_l_fstype, &l_fstype);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_l_start_pos_ext, &l_start_pos_ext);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_l_end_pos_ext, &l_end_pos_ext);

	/*
	 *         [out]   afsFetchStatus  *OutStatusp, 
        unsigned32              interfaceVersion;
        unsigned32              fileType;
        unsigned32              linkCount;
        afsHyper                length;
        afsHyper                dataVersion;
        unsigned32              author;
        unsigned32              owner;
        unsigned32              group;
        unsigned32              callerAccess;
        unsigned32              anonymousAccess;
        unsigned32              aclExpirationTime;
        unsigned32              mode;
        unsigned32              parentVnode;
        unsigned32              parentUnique;
        afsTimeval              modTime;
        afsTimeval              changeTime;
        afsTimeval              accessTime;
        afsTimeval              serverModTime;
        afsUUID                 typeUUID;
        afsUUID                 objectUUID;
        unsigned32              deviceNumber;
        unsigned32              blocksUsed;
        unsigned32              clientSpare1;   
        unsigned32              deviceNumberHighBits;
        unsigned32              agTypeUnique; 
        unsigned32              hiMaxSpare; 
        unsigned32              loMaxSpare; 
        unsigned32              pathConfSpare;
        unsigned32              spare4;
        unsigned32              spare5;
        unsigned32              spare6;
	 */
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_interfaceversion, &interfaceversion);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_filetype, &filetype);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_linkcount, &linkcount);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_length_high, &length_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_length_low, &length_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_dataversion_high, &dataversion_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_dataversion_low, &dataversion_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_author, &author);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_owner, &owner);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_group, &group);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_calleraccess, &calleraccess);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_anonymousaccess, &anonymousaccess);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_aclexpirationtime, &aclexpirationtime);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_mode, &mode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_parentvnode, &parentvnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_parentunique, &parentunique);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_modtime_sec, &modtime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_modtime_msec, &modtime_msec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_changetime_sec, &changetime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_changetime_msec, &changetime_msec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_accesstime_sec, &accesstime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_accesstime_msec, &accesstime_msec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_servermodtime_sec, &servermodtime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_servermodtime_msec, &servermodtime_msec);
	offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_typeuuid, &typeuuid);
	offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_objectuuid, &objectuuid);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_devicenumber, &devicenumber);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_blocksused, &blocksused);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_clientspare1, &clientspare1);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_devicenumberhighbits, &devicenumberhighbits);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_agtypeunique, &agtypeunique);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_himaxspare, &himaxspare);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_lomaxspare, &lomaxspare);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_pathconfspare, &pathconfspare);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_spare4, &spare4);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_spare5, &spare5);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_spare6, &spare6);
	/*
	 *       afsHyper VolID;
         *       afsHyper VV;             volume's version 
	 *       unsigned32 VVAge;       age, in seconds, of the knowledge that the given VolVers is current
  	 *       unsigned32 VVPingAge; age, in seconds, of the last probe from the callee (the secondary) to the primary
	 *       unsigned32 vv_spare1;
	 *       unsigned32 vv_spare2;
	 *
	 */

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_volid_hi, &volid_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_volid_low, &volid_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_vv_hi, &vv_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_vv_low, &vv_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_vvage, &vvage);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_vvpingage, &vvpingage);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_vvspare1, &vvspare1);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_gettoken_resp_vvspare2, &vvspare2);
	 
        if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_GetToken Reply");
	}else{

	offset += 104; /* not sure what I missed here.... 104 seems like something different from the idl def for this call */
	proto_tree_add_string (tree, hf_afs4int_gettoken_resp_key_t, tvb, offset, hf_afs4int_gettoken_resp_key_size, tvb_get_ptr (tvb, offset, key_size));
	key_t = tvb_get_ptr(tvb,offset,key_size);
	offset += key_size;

        if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_GetToken Reply EFAILURE: %s", key_t);
	}
	if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }



	return offset;
}

static int 
afs4int_dissect_lookuproot_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume;
         guint32 volume_high;
         guint32 volume_low;
         guint32 vnode;

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookuproot_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookuproot_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookuproot_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookuproot_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookuproot_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookuproot_rqst_unique, NULL);

	/*        [in]    afsFid          *InFidp,
	 *        [in]    afsHyper        *minVVp,
	 *        [in]    unsigned32   Flags,
	 */                                                        


	volume = volume_high | volume_low;

        if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_LookupRoot FSID:%llu", volume);
	if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


   	 return offset;
}

static int 
afs4int_dissect_fetchdata_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume;
         guint32 volume_high;
         guint32 volume_low;
         guint32 inode, vnode;

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchdata_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchdata_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchdata_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchdata_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchdata_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchdata_rqst_unique, NULL);

	volume = volume_high | volume_low;

	if (vnode == 1){
    		if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_FetchData FSID:%llu InFS", volume);

	} else {
		inode = ((volume << 16) + vnode) & 0x7fffffff;
    		if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_FetchData FSID:%llu inode: %u", volume, inode);
	}
	          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


    return offset;
}

static int 
afs4int_dissect_fetchacl_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume;
         guint32 volume_high;
         guint32 volume_low;
         guint32 inode, vnode;

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchacl_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchacl_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchacl_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchacl_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchacl_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchacl_rqst_unique, NULL);

	volume = volume_high | volume_low;

	if (vnode == 1) {
   		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_FetchAcl FSID:%llu InFS", volume);
	} else {
		  inode = ((volume << 16) + vnode) & 0x7fffffff;
    		  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_FetchAcl FSID:%llu inode: %u", volume, inode);

	}
          if (check_col (pinfo->cinfo, COL_INFO)) {
		                            col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum);
        }


    return offset;
}
static int 
afs4int_dissect_fetchstatus_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume;
         guint32 volume_high, volume_low, inode, vnode, vvp_hi, vvp_low, flags, error;

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_rqst_unique, NULL);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_rqst_vvp_hi, &vvp_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_rqst_vvp_low, &vvp_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_rqst_flags, &flags);


	volume = volume_high | volume_low;

	if (vnode == 1) {
   		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_FetchStatus Request FSID:%llu InFS flags:%u ", volume, flags);
	} else {
	         inode = ((volume << 16) + vnode) & 0x7fffffff;
   		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_FetchStatus Request FSID:%llu inode:%u flags:%u ", volume, inode, flags);
	}

	/* We cheat and use master_auth to determine if there is an error. Master auth is the auth var, which is 1, if it's kerberos, 0 if none. 
	 * When this packet type uses kerberos, then there is an error at the end.
	 */


if (master_auth){
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_rqst_error, &error);
	          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": Error=%u", error); }
}
	          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


    return offset;
}
static int 
afs4int_dissect_storeacl_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume;
         guint32 volume_high;
         guint32 volume_low;
         guint32 inode, vnode;
	 guint32 key_size;
	 char *key_t = NULL;

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storeacl_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storeacl_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storeacl_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storeacl_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storeacl_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storeacl_rqst_unique, NULL);
	/*
	 *         [in]    handle_t        h,
	 *         [in]    afsFid          *Fidp,
	 *         # all done above
	 *
	 *         [in]    afsACL          *AccessListp,
	 *                 unsigned32 afsACL_len;
	 *                 [length_is(afsACL_len)] byte afsACL_val[AFS_ACLMAX];
	 */


        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storeacl_rqst_var1, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storeacl_rqst_var2, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storeacl_rqst_key_size, &key_size);
	proto_tree_add_string (tree, hf_afs4int_storeacl_rqst_key_t, tvb, offset, hf_afs4int_storeacl_rqst_key_size, tvb_get_ptr (tvb, offset, key_size));
	key_t = tvb_get_ptr(tvb,offset,key_size);
	offset += key_size;


	/*         [in]    unsigned32      aclType,
	 *         [in]    afsFid          *aclFidp,
	 *         	   unsigned32  cell_high2
	 *         	   unsigned32  cell_low2
	 *         	   unsigned32  volume_high2
	 *         	   unsigned32  volume_low2
	 *         	   unsigned32  vnode2
	 *         	   unsigned32  unique2
	 *
	 *         [in]    afsHyper        *minVVp,
	 *                 unsigned32 vvp_high2
	 *                 unsigned32 vvp_low2
	 *         [in]    unsigned32   Flags,
	 *         	   unsigned32   flags2
	 */

	volume = volume_high | volume_low;

	if (vnode == 1) {
   		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_StoreACL FSID:%llu InFS Key: %s", volume, key_t);
	} else { 
		 inode = ((volume << 16) + vnode) & 0x7fffffff;
   		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_StoreACL FSID:%llu inode: %u Key: %s", volume, inode, key_t);
	}
	if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


    return offset;
}
static int 
afs4int_dissect_storestatus_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume;
         guint32 volume_high;
         guint32 volume_low;
         guint32 inode, vnode;
	 guint32 mask, modtime_sec, modtime_usec, accesstime_sec, accesstime_usec, changetime_sec, changetime_usec, 
		owner, group, mode, trunc_high, trunc_low, length_high, length_low, typeuuid, devicetype, devicenumber, cmask, 
                clientspare1, devicenumberhighbits, spare1, spare2, spare3, spare4, spare5, spare6, vvp_high, vvp_low, flags;



#define AFS_SETMODTIME 1
#define AFS_SETOWNER  2
#define AFS_SETGROUP  4
#define AFS_SETMODE  8
#define AFS_SETACCESSTIME  0x10
#define	AFS_SETCHANGETIME  0x20  
#define	AFS_SETLENGTH  0x40
#define	AFS_SETTYPEUUID  0x80
#define	AFS_SETDEVNUM  0x100
#define	AFS_SETMODEXACT  0x200
#define	AFS_SETTRUNCLENGTH  0x400
#define	AFS_SETCLIENTSPARE  0x800
		


        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_unique, NULL);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_mask, &mask);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_modtime_sec, &modtime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_modtime_usec, &modtime_usec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_accesstime_sec, &accesstime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_accesstime_usec, &accesstime_usec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_changetime_sec, &changetime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_changetime_usec, &changetime_usec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_owner, &owner);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_group, &group);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_mode, &mode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_trunc_high, &trunc_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_trunc_low, &trunc_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_length_high, &length_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_length_low, &length_low);
	offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_typeuuid, &typeuuid);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_devicetype, &devicetype);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_devicenumber, &devicenumber);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_cmask, &cmask);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_clientspare1, &clientspare1);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_devicenumberhighbits, &devicenumberhighbits );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_spare1, &spare1 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_spare2, &spare2 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_spare3, &spare3 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_spare4, &spare4 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_spare5, &spare5 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_spare6, &spare6 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_vvp_high, &vvp_high );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_vvp_low, &vvp_low );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_rqst_flags, &flags);

	volume = volume_high | volume_low;

	/*        unsigned32              mask;
	 *        afsTimeval              modTime;
	 *        afsTimeval              accessTime;
	 *        afsTimeval              changeTime;
	 *        unsigned32              owner;
	 *        unsigned32              group;
	 *        unsigned32              mode;
	 *        afsHyper                truncLength;  
	          afsHyper                length;
	          afsUUID                 typeUUID;
	          unsigned32              deviceType;   
	          unsigned32              deviceNumber;
	          unsigned32              cmask;
	          unsigned32              clientSpare1;   
	          unsigned32              deviceNumberHighBits;
	          unsigned32              spare1;
	          unsigned32              spare2;
	          unsigned32              spare3;
	          unsigned32              spare4;
	          unsigned32              spare5;
	          unsigned32              spare6;
							
	 */


		if (vnode == 1) {
        		if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_StoreStatus FSID:%llu InFS", volume);

		} else { 
			inode = ((volume << 16) + vnode) & 0x7fffffff;
        		if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_StoreStatus FSID:%llu inode: %u", volume, inode);
		}

if (check_col(pinfo->cinfo, COL_INFO)) col_append_fstr(pinfo->cinfo, COL_INFO, " Flags=");

if ((mask & AFS_SETMODTIME) == AFS_SETMODTIME) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_fstr(pinfo->cinfo, COL_INFO, ":SETMODTIME-%u.%u", modtime_sec, modtime_usec);
}
if ((mask & AFS_SETOWNER) == AFS_SETOWNER) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_fstr(pinfo->cinfo, COL_INFO, ":SETOWNER-%u", owner);
}
if ((mask & AFS_SETGROUP) == AFS_SETGROUP) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_fstr(pinfo->cinfo, COL_INFO, ":SETGROUP-%u", group);
}
if ((mask & AFS_SETMODE) == AFS_SETMODE) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_fstr(pinfo->cinfo, COL_INFO, ":SETMODE-%o", mode);
}
if ((mask & AFS_SETACCESSTIME) == AFS_SETACCESSTIME) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_fstr(pinfo->cinfo, COL_INFO, ":SETACCESSTIME-%u.%u", accesstime_sec, accesstime_usec);
}
if ((mask & AFS_SETCHANGETIME) == AFS_SETCHANGETIME) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_fstr(pinfo->cinfo, COL_INFO, ":SETCHANGETIME-%u.%u", changetime_sec, changetime_usec);
}
if ((mask & AFS_SETLENGTH) == AFS_SETLENGTH) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":SETLENGTH" );
}
if ((mask & AFS_SETTYPEUUID) == AFS_SETTYPEUUID) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":SETTYPEUUID");
}
if ((mask & AFS_SETDEVNUM) == AFS_SETDEVNUM) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":SETDEVNUM");
}
if ((mask & AFS_SETMODEXACT) == AFS_SETMODEXACT) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":SETMODEXACT");
}
if ((mask & AFS_SETTRUNCLENGTH) == AFS_SETTRUNCLENGTH) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":SETTRUNCLENGTH");
}
if ((mask & AFS_SETCLIENTSPARE) == AFS_SETCLIENTSPARE) {
if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":SETCLIENTSPARE");
}
          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }

		  

    return offset;
}
static int 
afs4int_dissect_createfile_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume, trunc, length, volume2, vvp;
	 guint32 mask, modtime_sec, modtime_usec, accesstime_sec, accesstime_usec, changetime_sec, changetime_usec, 
		owner, group, mode, trunc_high, trunc_low, length_high, length_low, devicetype, devicenumber, cmask, 
		clientspare1, devicenumberhighbits, spare1, spare2, spare3, spare4, spare5, spare6, vvp_high, vvp_low, flags,
	        volume_high, volume_low, volume_high2, volume_low2, vnode, vnode2, inode, inode2, tn_tag;
	guint16 tn_length;
	guint32 key_size = 257;
        e_uuid_t  typeuuid;
	char *key_t;

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_unique, NULL);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_tn_tag, &tn_tag);
	offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_tn_length, &tn_length);
	proto_tree_add_string (tree, hf_afs4int_createfile_rqst_key_t, tvb, offset, hf_afs4int_createfile_rqst_key_size, tvb_get_ptr (tvb, offset, key_size));
	key_t = tvb_get_ptr(tvb,offset,key_size);
	offset += key_size;
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_mask, &mask);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_modtime_sec, &modtime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_modtime_usec, &modtime_usec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_accesstime_sec, &accesstime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_accesstime_usec, &accesstime_usec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_changetime_sec, &changetime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_changetime_usec, &changetime_usec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_owner, &owner);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_group, &group);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_mode, &mode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_trunc_high, &trunc_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_trunc_low, &trunc_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_length_high, &length_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_length_low, &length_low);
	offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_typeuuid, &typeuuid);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_devicetype, &devicetype);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_devicenumber, &devicenumber);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_cmask, &cmask);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_clientspare1, &clientspare1);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_devicenumberhighbits, &devicenumberhighbits );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_spare1, &spare1 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_spare2, &spare2 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_spare3, &spare3 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_spare4, &spare4 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_spare5, &spare5 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_spare6, &spare6 );

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_vvp_high, &vvp_high );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_vvp_low, &vvp_low );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_createfile_rqst_flags, &flags);

	volume = volume_high | volume_low;
	volume2 = volume_high2 | volume_low2;
	if (vnode == 1) {
    		if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_CreateFile FSID:%llu InFS Name: %s", volume, key_t);
	} else {
        	inode = ((volume << 16) + vnode) & 0x7fffffff;
    		if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_CreateFile FSID:%llu PDI: %u Name: %s", volume, inode, key_t);
	}
	          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


    return offset;
}
static int 
afs4int_dissect_rename_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume;
         guint32 volume_low_old, volume_high_old, volume_low_new, volume_high_new, vnode_old, vnode_new, tn_old, tn_new, inode_new;
	 guint16 tn2_old, tn2_new;
	 guint32 key_size = 257;
	 char *key_t_old = NULL;
	 char *key_t_new = NULL;

	/*
	 *     [in]    afsFid          *OldDirFidp,
	 */

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_cell_high_old, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_cell_low_old, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_volume_high_old, &volume_high_old);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_volume_low_old, &volume_low_old);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_vnode_old, &vnode_old);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_unique_old, NULL);

	/*
	 *     [in]    afsFidTaggedName        *OldNamep,
	 */

	offset += 24;
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_tn_old, &tn_old);
	offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_tn2_old, &tn2_old);
	proto_tree_add_string (tree, hf_afs4int_rename_rqst_key_t_old, tvb, offset, hf_afs4int_rename_rqst_key_size_old, tvb_get_ptr (tvb, offset, key_size));
	key_t_old = tvb_get_ptr(tvb,offset,key_size);
	offset += key_size;

        /*
	 *     [in]    afsFid          *NewDirFidp,
	 */

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_cell_high_new, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_cell_low_new, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_volume_high_new, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_volume_low_new, NULL);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_vnode_new, &vnode_new);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_unique_new, NULL);

	/*     [in]    afsFidTaggedName        *NewNamep,
	 */

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_tn_new, NULL);
	offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_tn2_new, NULL);
	offset += 24;
	proto_tree_add_string (tree, hf_afs4int_rename_rqst_key_t_new, tvb, offset, hf_afs4int_rename_rqst_key_size_new, tvb_get_ptr (tvb, offset, key_size));
	key_t_new = tvb_get_ptr(tvb,offset,key_size);
	offset += key_size;

	/*     [in]    afsHyper        *returnTokenIDp,
	 *     [in]    afsHyper        *minVVp,
	 *     [in]    unsigned32      Flags,
	 */

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_tokenidp_high, NULL);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_tokenidp_low, NULL);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_vvp_high, NULL);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_vvp_low, NULL);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_rename_rqst_flags, NULL);

	volume = volume_high_old | volume_low_old;

	if (vnode_new == 1) {
    		if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_Rename FSID:%llu InFS From %s to %s", volume, key_t_old, key_t_new);
	} else { 
		inode_new = ((volume << 16) + vnode_new) & 0x7fffffff;
    		if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_Rename FSID:%llu From: %s to %s inode: %u ", volume, key_t_old, key_t_new, inode_new );
	}
	          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


    return offset;
}

static int 
afs4int_dissect_symlink_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume;
         guint32 volume_high;
         guint32 volume_low;
         guint32 inode, vnode;


        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_symlink_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_symlink_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_symlink_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_symlink_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_symlink_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_symlink_rqst_unique, NULL);

	volume = volume_high | volume_low;

	if (vnode == 1) {
    	      if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_Symlink FSID:%llu InFS", volume);
	} else {
	      inode = ((volume << 16) + vnode) & 0x7fffffff;
    	      if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_Symlink FSID:%llu inode: %u", volume, inode);
	}

    return offset;
}
static int 
afs4int_dissect_readdir_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume, vvp;
         guint32 volume_high, volume_low, inode, vnode, unique, offset_high, offset_low, size, vvp_high, vvp_low, flags ;

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_readdir_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_readdir_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_readdir_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_readdir_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_readdir_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_readdir_rqst_unique, NULL);

	/* [in]    afsHyper        *Offsetp,
	   [in]    unsigned32      Size,
	   [in]    afsHyper        *minVVp,
	   [in]    unsigned32      Flags,*/
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_readdir_rqst_offset_high, &offset_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_readdir_rqst_offset_low, &offset_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_readdir_rqst_size, &size);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_readdir_rqst_vvp_high, &vvp_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_readdir_rqst_vvp_low, &vvp_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_readdir_rqst_flags, &flags);

	volume = volume_high | volume_low;

	if (vnode == 1) {
		    if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_ReadDir FSID:%llu InFS", volume);
	} else {
 		    inode = ((volume << 16) + vnode) & 0x7fffffff;
		    if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_ReadDir FSID:%llu inode: %u", volume, inode);
	}
	          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


    return offset;
}

static int 
afs4int_dissect_makedir_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume, trunc, length, volume2, vvp;
	 guint32 mask, modtime_sec, modtime_usec, accesstime_sec, accesstime_usec, changetime_sec, changetime_usec, 
		owner, group, mode, trunc_high, trunc_low, length_high, length_low, devicetype, devicenumber, cmask, 
		clientspare1, devicenumberhighbits, spare1, spare2, spare3, spare4, spare5, spare6, vvp_high, vvp_low, flags,
	        volume_high, volume_low, volume_high2, volume_low2, vnode, vnode2, inode, inode2, tn_tag;
	guint16 tn_length;
	guint32 key_size = 257;
        e_uuid_t  typeuuid;
	char *key_t;


        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_unique, NULL);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_tn_tag, &tn_tag);
	offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_tn_length, &tn_length);
	proto_tree_add_string (tree, hf_afs4int_makedir_rqst_key_t, tvb, offset, hf_afs4int_makedir_rqst_key_size, tvb_get_ptr (tvb, offset, key_size));
	key_t = tvb_get_ptr(tvb,offset,key_size);
	offset += key_size;

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_mask, &mask);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_modtime_sec, &modtime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_modtime_usec, &modtime_usec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_accesstime_sec, &accesstime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_accesstime_usec, &accesstime_usec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_changetime_sec, &changetime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_changetime_usec, &changetime_usec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_owner, &owner);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_group, &group);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_mode, &mode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_trunc_high, &trunc_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_trunc_low, &trunc_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_length_high, &length_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_length_low, &length_low);
	offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_typeuuid, &typeuuid);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_devicetype, &devicetype);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_devicenumber, &devicenumber);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_cmask, &cmask);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_clientspare1, &clientspare1);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_devicenumberhighbits, &devicenumberhighbits );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_spare1, &spare1 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_spare2, &spare2 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_spare3, &spare3 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_spare4, &spare4 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_spare5, &spare5 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_spare6, &spare6 );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_vvp_high, &vvp_high );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_vvp_low, &vvp_low );
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makedir_rqst_flags, &flags);

	volume = volume_high | volume_low;
	if (vnode == 1) {
   		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_MakeDir FSID:%llu InFS Name: %s", volume, key_t);
	} else {
	         inode = ((volume << 16) + vnode) & 0x7fffffff;
   		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_MakeDir FSID:%llu PDI: %u Name: %s", volume, inode, key_t);
	}
	          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


    return offset;
}

static int 
afs4int_dissect_removedir_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume, volume2;
	 guint32 volume_high, volume_high2;
	 guint32 volume_low, volume_low2;
	 guint32 inode, vnode, vnode2, tn;
	 guint32 key_size = 257;
	 guint16 tn2;
	 char *key_t;



        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_unique, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_cell_high2, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_cell_low2, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_volume_high2, &volume_high2);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_volume_low2, &volume_low2);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_vnode2, &vnode2);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_unique2, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_tn, &tn);
        offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_removedir_rqst_tn2, &tn2);

        proto_tree_add_string (tree, hf_afs4int_removedir_rqst_key_t, tvb, offset, hf_afs4int_removedir_rqst_key_size, tvb_get_ptr (tvb, offset, key_size));
	key_t = tvb_get_ptr(tvb,offset,key_size);
	offset += key_size;

	volume = volume_high | volume_low;
	volume2 = volume_high2 | volume_low2;
	if (vnode == 1) {
    		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_RemoveDir FSID:%llu FSdir Name: %s", volume, key_t);
	} else {
		 inode = ((volume << 16) + vnode) & 0x7fffffff;
    		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_RemoveDir FSID:%llu PDI %u Name: %s", volume, inode, key_t);
	}
	          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


    return offset;
}

static int 
afs4int_dissect_lookup_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume;
         guint32 volume_high;
         guint32 volume_low;
         guint32 inode, vnode, tn_tag, vvp_high, vvp_low, flags;
	 guint32 key_size =  257;
	 guint16 tn_length;
	 char *key_t;


        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_rqst_unique, NULL);

	/*
	 *         [in]    afsFid          *DirFidp,
	 *                         afsHyper Cell;
	 *                         afsHyper Volume;
	 *                         unsigned32 Vnode;
	 *                         unsigned32 Unique;
	 *                       
	 *         [in]    afsTaggedName   *Namep,
	 *                         codesetTag      tn_tag;
	 *                            unsigned32 tn_tag
	 *                         unsigned16      tn_length;
	 *                            unsigned16 tn_length
	 *                         byte            tn_chars[AFS_NAMEMAX+1];
	 *                            pointer?
	 *
	 *         [in]    afsHyper        *minVVp,
	 *         		   unsigned32 vvp_high
	 *         		   unsigned32 vvp_low
	 *         [in]    unsigned32      Flags,
	 *                         unsigned32 flags
	 *                 
	 *                                  
         */

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_rqst_tn_tag, &tn_tag);
	offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_rqst_tn_length, &tn_length);
	proto_tree_add_string (tree, hf_afs4int_lookup_rqst_key_t, tvb, offset, hf_afs4int_lookup_rqst_key_size, tvb_get_ptr (tvb, offset, key_size));
	key_t = tvb_get_ptr(tvb,offset,key_size);
	offset += key_size;
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_rqst_vvp_high, &vvp_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_rqst_vvp_low, &vvp_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_rqst_flags, &flags);

	volume = volume_high | volume_low;

	if (vnode == 1 ) {
   		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_Lookup Request FSID:%llu InFS Name: %s", volume, key_t);

	} else {
	      	inode = ((volume << 16) + vnode) & 0x7fffffff;
   		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_Lookup Request FSID:%llu PDI: %u Name: %s", volume, inode, key_t);
	}
	          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


    return offset;
}
static int 
afs4int_dissect_lookup_resp (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	guint64 volume = -1;
	guint32 volume_high, volume_low, vnode, inode, cell_low;

	/*        [out]   afsFid          *OutFidp,
	 *        [out]   afsFetchStatus  *OutFidStatusp,
	 *        [out]   afsFetchStatus  *OutDirStatusp,
	 *        [out]   afsToken        *OutTokenp,
	 *        [out]   afsVolSync      *Syncp
	 */


        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_resp_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_resp_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_resp_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_resp_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_resp_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lookup_resp_unique, NULL);
	volume = volume_high | volume_low;

	if (!volume_low){

   		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_str(pinfo->cinfo, COL_INFO, "DFS_Lookup Reply ENOEXIST");
	}else{

	      	inode = ((volume << 16) + vnode) & 0x7fffffff;
   		 if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_Lookup Reply FSID:%llu Inode: %u ", volume, inode);
	}

	          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


	 return offset;
	
}

static int 
afs4int_dissect_makemountpoint_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{
	 guint64 volume;
         guint32 volume_high;
         guint32 volume_low;
         guint32 inode, vnode;

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makemountpoint_rqst_cell_high, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makemountpoint_rqst_cell_low, NULL);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makemountpoint_rqst_volume_high, &volume_high);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makemountpoint_rqst_volume_low, &volume_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makemountpoint_rqst_vnode, &vnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_makemountpoint_rqst_unique, NULL);

	volume = volume_high | volume_low;
	if (vnode == 1) {
    		if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_MakeMountPoint FSID:%llu InFS", volume);
	} else {
	      	inode = ((volume << 16) + vnode) & 0x7fffffff;
    		if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_MakeMountPoint FSID:%llu inode: %u", volume, inode);
	}
	          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }


    return offset;
}
static int 
afs4int_dissect_setcontext_rqst (tvbuff_t *tvb, int offset,
	       	               packet_info *pinfo, proto_tree *tree,
			       char *drep)
{

        e_uuid_t secobjectid;
        guint32 epochtime, flags, clientsizesattrs, parm7, key_size;
	guint16 type;
	guint8 data0, data1, data2, data3, data4, data5, data6, data7, data8, data9, data10, data11, data12, data13, data14;
	char *key_t = NULL;

#define	AFS_FLAG_SEC_SERVICE    0x1
#define AFS_FLAG_CONTEXT_NEW_IF  0x2
#define AFS_FLAG_CONTEXT_DO_RESET  0x4
#define AFS_FLAG_CONTEXT_NEW_ACL_IF  0x8
#define AFS_FLAG_CONTEXT_NEW_TKN_TYPES  0x10

        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_epochtime, &epochtime);
	if ( epochtime != 16) {
        offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_afsnetaddr_type, &type);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data0, &data0);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data1, &data1);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data2, &data2);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data3, &data3);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data4, &data4);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data5, &data5);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data6, &data6);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data7, &data7);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data8, &data8);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data9, &data9);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data10, &data10);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data11, &data11);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data12, &data12);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data13, &data13);
        offset = dissect_ndr_uint8 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data14, &data14);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_key_size, &key_size);
        proto_tree_add_string (tree, hf_afs4int_setcontext_rqst_key_t, tvb, offset, hf_afs4int_setcontext_rqst_key_size, tvb_get_ptr (tvb, offset, key_size));
        key_t = tvb_get_ptr(tvb,offset,key_size);
        offset += key_size;
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_flags, &flags);
	offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_secobjectid, &secobjectid);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_clientsizesattrs, &clientsizesattrs);
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_parm7, &parm7);

    	if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_SetContext Request Data:%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",data0, data1, data2, data3, data4, data5, data6, data7, data8, data9, data10, data11, data12, data13, data14);
 }else{
	 offset += 44;
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_key_size, &key_size);
        proto_tree_add_string (tree, hf_afs4int_setcontext_rqst_key_t, tvb, offset, hf_afs4int_setcontext_rqst_key_size, tvb_get_ptr (tvb, offset, key_size));
        key_t = tvb_get_ptr(tvb,offset,key_size);
        offset += key_size;


    	if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_SetContext Request for: %s", key_t);

}

if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, " Flags="); 

if ((flags & AFS_FLAG_SEC_SERVICE) == AFS_FLAG_SEC_SERVICE) {
	if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":SEC_SERVICE");
}
if ((flags & AFS_FLAG_CONTEXT_NEW_IF) == AFS_FLAG_CONTEXT_NEW_IF) {
	if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":NEW_IF");
}
if ((flags & AFS_FLAG_CONTEXT_DO_RESET) == AFS_FLAG_CONTEXT_DO_RESET) {
	if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":DO_RESET");
}
if ((flags & AFS_FLAG_CONTEXT_NEW_ACL_IF) == AFS_FLAG_CONTEXT_NEW_ACL_IF) {
	if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":NEW_ACL_IF");
}
if ((flags & AFS_FLAG_CONTEXT_NEW_TKN_TYPES) == AFS_FLAG_CONTEXT_NEW_TKN_TYPES) {
	if (check_col(pinfo->cinfo, COL_INFO)) col_append_str(pinfo->cinfo, COL_INFO, ":NEW_TKN_TYPES");
}
          if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
	return offset;
}

static int
afs4int_dissect_setcontext_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
	guint32 var1, var2;
	if (master_auth){
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_resp_var1, &var1);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_setcontext_resp_var1, &var2);
        if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_SetContext Reply DCE_ERR: %u VAL: %u", var1, var2);
	}else{
        if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_SetContext Reply");
	} 

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
   return offset;
 
}
static int
afs4int_dissect_lookuproot_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{	
	/*
	 *        [out]   afsFid          *OutFidp,
	 *        [out]   afsFetchStatus  *OutFidStatusp,
	 *        [out]   afsToken        *OutTokenp,
	 *        [out]   afsVolSync      *Syncp
	 */
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_LookupRoot Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_fetchdata_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_FetchData Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_fetchacl_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_FetchACL Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_fetchstatus_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{

         guint32 tokenid_hi, tokenid_low, expirationtime, type_hi, type_low, beginrange, endrange, beginrangeext, endrangeext;
	 guint32 volid_hi, volid_low, vv_hi, vv_low, vvage, vvpingage, vvspare1, vvspare2;
	 guint32 interfaceversion, filetype, linkcount, length_high, length_low, dataversion_high, dataversion_low, author, owner, group, calleraccess, anonymousaccess, aclexpirationtime, mode, parentvnode, parentunique, modtime_sec, modtime_msec, changetime_sec, changetime_msec, accesstime_sec, accesstime_msec, servermodtime_sec, servermodtime_msec, devicenumber, blocksused, clientspare1, devicenumberhighbits, agtypeunique, himaxspare, lomaxspare, pathconfspare, spare4, spare5, spare6;
	 e_uuid_t typeuuid, objectuuid;
	 guint32 err1, err2;
   /*       [out]   afsFetchStatus  *OutStatusp, */
  /* XXX */
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_interfaceversion, &interfaceversion);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_filetype, &filetype);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_linkcount, &linkcount);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_length_high, &length_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_length_low, &length_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_dataversion_high, &dataversion_high);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_dataversion_low, &dataversion_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_author, &author);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_owner, &owner);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_group, &group);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_calleraccess, &calleraccess);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_anonymousaccess, &anonymousaccess);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_aclexpirationtime, &aclexpirationtime);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_mode, &mode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_parentvnode, &parentvnode);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_parentunique, &parentunique);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_modtime_sec, &modtime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_modtime_msec, &modtime_msec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_changetime_sec, &changetime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_changetime_msec, &changetime_msec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_accesstime_sec, &accesstime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_accesstime_msec, &accesstime_msec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_servermodtime_sec, &servermodtime_sec);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_servermodtime_msec, &servermodtime_msec);
	offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_typeuuid, &typeuuid);
	offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_objectuuid, &objectuuid);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_devicenumber, &devicenumber);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_blocksused, &blocksused);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_clientspare1, &clientspare1);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_devicenumberhighbits, &devicenumberhighbits);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_agtypeunique, &agtypeunique);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_himaxspare, &himaxspare);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_lomaxspare, &lomaxspare);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_pathconfspare, &pathconfspare);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_spare4, &spare4);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_spare5, &spare5);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_spare6, &spare6);
   /*       [out]   afsToken        *OutTokenp, */
	
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_tokenid_hi, &tokenid_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_tokenid_low, &tokenid_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_expirationtime, &expirationtime);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_type_hi, &type_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_type_low, &type_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_beginrange, &beginrange);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_endrange, &endrange);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_beginrangeext, &beginrangeext);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_endrangeext, &endrangeext);

   /*       [out]   afsVolSync      *Syncp */

	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_volid_hi, &volid_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_volid_low, &volid_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_vv_hi, &vv_hi);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_vv_low, &vv_low);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_vvage, &vvage);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_vvpingage, &vvpingage);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_vvspare1, &vvspare1);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_vvspare2, &vvspare2);

	if (master_auth){
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_err1, &err1);
	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_fetchstatus_resp_err2, &err2);
	}

        if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_FetchStatus Reply E1: %u E2: %u", err1, err2);
	if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }

   return offset;
}
static int
afs4int_dissect_storedata_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_StoreData Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_storeacl_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_StoreAcl Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_storestatus_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_StoreStatus Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_removefile_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_RemoveFile Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_createfile_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_CreateFile Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_rename_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_Rename Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_symlink_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_Symlink Reply  ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_hardlink_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_HardLink Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_hardlink_rqst (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_HardLink Request ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_makedir_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_MakeDir Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_removedir_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_RemoveDir Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_readdir_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_ReadDir Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_releasetokens_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_ReleaseTokens Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_releasetokens_rqst (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_ReleaseTokens Request ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_gettime_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_GetTime Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_gettime_rqst (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_GetTime Request ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_processquota_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_ProcessQuota Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_processquota_rqst (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_ProcessQuota Request ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_getserverinterfaces_rqst (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_GetServerInterfaces Request ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_getserverinterfaces_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_GetServerInterfaces Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_setparams_rqst (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_SetParams Request ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_setparams_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_SetParams Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_makemountpoint_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_MakeMountPoint Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_getstatistics_rqst (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_GetStatistics Request ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_getstatistics_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_GetStatistics Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_bulkfetchvv_rqst (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_BulkFetchVV Request ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_bulkfetchvv_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_BulkFetchVV Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_bulkkeepalive_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_BulkKeepAlive Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_bulkkeepalive_rqst (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_BulkKeepAlive Request ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}
static int
afs4int_dissect_bulkfetchstatus_rqst (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_BulkFetchStatus Request ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}

static int
afs4int_dissect_bulkfetchstatus_resp (tvbuff_t *tvb, int offset,
                               packet_info *pinfo, proto_tree *tree,
                               char *drep)
{
  if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, "DFS_BulkFetchStatus Reply ");
        

   if (check_col (pinfo->cinfo, COL_INFO)) { col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum); }
 
   return offset;
}

static dcerpc_sub_dissector AFS4Int_dissectors[] = {
    { 0, "DFS_SetContext", afs4int_dissect_setcontext_rqst , afs4int_dissect_setcontext_resp },
    { 1, "DFS_LookupRoot", afs4int_dissect_lookuproot_rqst, afs4int_dissect_lookuproot_resp},
    { 2, "DFS_FetchData", afs4int_dissect_fetchdata_rqst, afs4int_dissect_fetchdata_resp},
    { 3, "DFS_FetchACL", afs4int_dissect_fetchacl_rqst, afs4int_dissect_fetchacl_resp},
    { 4, "DFS_FetchStatus", afs4int_dissect_fetchstatus_rqst, afs4int_dissect_fetchstatus_resp},
    { 5, "DFS_StoreData", afs4int_dissect_storedata_rqst, afs4int_dissect_storedata_resp},
    { 6, "DFS_StoreACL", afs4int_dissect_storeacl_rqst, afs4int_dissect_storeacl_resp},
    { 7, "DFS_StoreStatus", afs4int_dissect_storestatus_rqst, afs4int_dissect_storestatus_resp},
    { 8, "DFS_RemoveFile", afs4int_dissect_removefile_rqst, afs4int_dissect_removefile_resp},
    { 9, "DFS_CreateFile", afs4int_dissect_createfile_rqst, afs4int_dissect_createfile_resp},
    { 10, "DFS_Rename", afs4int_dissect_rename_rqst, afs4int_dissect_rename_resp},
    { 11, "DFS_Symlink", afs4int_dissect_symlink_rqst, afs4int_dissect_symlink_resp},
    { 12, "DFS_HardLink", afs4int_dissect_hardlink_rqst, afs4int_dissect_hardlink_resp},
    { 13, "DFS_MakeDir", afs4int_dissect_makedir_rqst, afs4int_dissect_makedir_resp},
    { 14, "DFS_RemoveDir", afs4int_dissect_removedir_rqst, afs4int_dissect_removedir_resp},
    { 15, "DFS_Readdir", afs4int_dissect_readdir_rqst, afs4int_dissect_readdir_resp},
    { 16, "DFS_Lookup", afs4int_dissect_lookup_rqst, afs4int_dissect_lookup_resp},
    { 17, "DFS_GetToken",afs4int_dissect_gettoken_rqst, afs4int_dissect_gettoken_resp},
    { 18, "DFS_ReleaseTokens", afs4int_dissect_releasetokens_rqst, afs4int_dissect_releasetokens_resp},
    { 19, "DFS_GetTime", afs4int_dissect_gettime_rqst, afs4int_dissect_gettime_resp},
    { 20, "DFS_MakeMountPoint", afs4int_dissect_makemountpoint_rqst, afs4int_dissect_makemountpoint_resp},
    { 21, "DFS_GetStatistics", afs4int_dissect_getstatistics_rqst, afs4int_dissect_getstatistics_resp},
    { 22, "DFS_BulkFetchVV", afs4int_dissect_bulkfetchvv_rqst, afs4int_dissect_bulkfetchvv_resp},
    { 23, "DFS_BulkKeepAlive", afs4int_dissect_bulkkeepalive_rqst, afs4int_dissect_bulkkeepalive_resp},
    { 24, "DFS_ProcessQuota", afs4int_dissect_processquota_rqst, afs4int_dissect_processquota_resp},
    { 25, "DFS_GetServerInterfaces", afs4int_dissect_getserverinterfaces_rqst, afs4int_dissect_getserverinterfaces_resp },
    { 26, "DFS_SetParams", afs4int_dissect_setparams_rqst, afs4int_dissect_setparams_resp},
    { 27, "DFS_BulkFetchStatus", afs4int_dissect_bulkfetchstatus_rqst, afs4int_dissect_bulkfetchstatus_resp},
    { 0, NULL, NULL, NULL },

};

void
proto_register_AFS4Int (void)
{

static hf_register_info hf[] = {

{ &hf_afs4int_removefile_rqst_cell_high,
        {"hf_afs4int_removefile_rqst_cell_high", "afs4int.hf_afs4int_removefile_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_cell_low,
        {"hf_afs4int_removefile_rqst_cell_low", "afs4int.hf_afs4int_removefile_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_volume_high,
        {"hf_afs4int_removefile_rqst_volume_high", "afs4int.hf_afs4int_removefile_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_volume_low,
        {"hf_afs4int_removefile_rqst_volume_low", "afs4int.hf_afs4int_removefile_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_vnode,
        {"hf_afs4int_removefile_rqst_vnode", "afs4int.hf_afs4int_removefile_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_unique,
        {"hf_afs4int_removefile_rqst_unique", "afs4int.hf_afs4int_removefile_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_cell_high2,
        {"hf_afs4int_removefile_rqst_cell_high2", "afs4int.hf_afs4int_removefile_rqst_cell_high2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_cell_low2,
        {"hf_afs4int_removefile_rqst_cell_low2", "afs4int.hf_afs4int_removefile_rqst_cell_low2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_volume_high2,
        {"hf_afs4int_removefile_rqst_volume_high2", "afs4int.hf_afs4int_removefile_rqst_volume_high2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_volume_low2,
        {"hf_afs4int_removefile_rqst_volume_low2", "afs4int.hf_afs4int_removefile_rqst_volume_low2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_vnode2,
        {"hf_afs4int_removefile_rqst_vnode2", "afs4int.hf_afs4int_removefile_rqst_vnode2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_unique2,
        {"hf_afs4int_removefile_rqst_unique2", "afs4int.hf_afs4int_removefile_rqst_unique2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_tn,
        {"hf_afs4int_removefile_rqst_tn", "afs4int.hf_afs4int_removefile_rqst_tn", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_tn2,
        {"hf_afs4int_removefile_rqst_tn2", "afs4int.hf_afs4int_removefile_rqst_tn2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_key_size,
        {"hf_afs4int_removefile_rqst_key_size", "afs4int.hf_afs4int_removefile_rqst_key_size", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removefile_rqst_key_t,
        {"hf_afs4int_removefile_rqst_key_t", "afs4int.hf_afs4int_removefile_rqst_key_t", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storedata_rqst_cell_high,
        {"hf_afs4int_storedata_rqst_cell_high", "afs4int.hf_afs4int_storedata_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storedata_rqst_cell_low,
        {"hf_afs4int_storedata_rqst_cell_low", "afs4int.hf_afs4int_storedata_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storedata_rqst_volume_high,
        {"hf_afs4int_storedata_rqst_volume_high", "afs4int.hf_afs4int_storedata_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storedata_rqst_volume_low,
        {"hf_afs4int_storedata_rqst_volume_low", "afs4int.hf_afs4int_storedata_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storedata_rqst_vnode,
        {"hf_afs4int_storedata_rqst_vnode", "afs4int.hf_afs4int_storedata_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storedata_rqst_unique,
        {"hf_afs4int_storedata_rqst_unique", "afs4int.hf_afs4int_storedata_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_cell_high,
        {"hf_afs4int_gettoken_rqst_cell_high", "afs4int.hf_afs4int_gettoken_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_cell_low,
        {"hf_afs4int_gettoken_rqst_cell_low", "afs4int.hf_afs4int_gettoken_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_volume_high,
        {"hf_afs4int_gettoken_rqst_volume_high", "afs4int.hf_afs4int_gettoken_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_volume_low,
        {"hf_afs4int_gettoken_rqst_volume_low", "afs4int.hf_afs4int_gettoken_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_vnode,
        {"hf_afs4int_gettoken_rqst_vnode", "afs4int.hf_afs4int_gettoken_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_unique,
        {"hf_afs4int_gettoken_rqst_unique", "afs4int.hf_afs4int_gettoken_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_tokenid_hi,
        {"hf_afs4int_gettoken_rqst_tokenid_hi", "afs4int.hf_afs4int_gettoken_rqst_tokenid_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_tokenid_low,
        {"hf_afs4int_gettoken_rqst_tokenid_low", "afs4int.hf_afs4int_gettoken_rqst_tokenid_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_expirationtime,
        {"hf_afs4int_gettoken_rqst_expirationtime", "afs4int.hf_afs4int_gettoken_rqst_expirationtime", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_type_hi,
        {"hf_afs4int_gettoken_rqst_type_hi", "afs4int.hf_afs4int_gettoken_rqst_type_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_type_low,
        {"hf_afs4int_gettoken_rqst_type_low", "afs4int.hf_afs4int_gettoken_rqst_type_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_beginrange,
        {"hf_afs4int_gettoken_rqst_beginrange", "afs4int.hf_afs4int_gettoken_rqst_beginrange", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_endrange,
        {"hf_afs4int_gettoken_rqst_endrange", "afs4int.hf_afs4int_gettoken_rqst_endrange", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_beginrangeext,
        {"hf_afs4int_gettoken_rqst_beginrangeext", "afs4int.hf_afs4int_gettoken_rqst_beginrangeext", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_endrangeext,
        {"hf_afs4int_gettoken_rqst_endrangeext", "afs4int.hf_afs4int_gettoken_rqst_endrangeext", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_minvvp_hi,
        {"hf_afs4int_gettoken_rqst_minvvp_hi", "afs4int.hf_afs4int_gettoken_rqst_minvvp_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_minvvp_low,
        {"hf_afs4int_gettoken_rqst_minvvp_low", "afs4int.hf_afs4int_gettoken_rqst_minvvp_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_rqst_flags,
        {"hf_afs4int_gettoken_rqst_flags", "afs4int.hf_afs4int_gettoken_rqst_flags", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookuproot_rqst_cell_high,
        {"hf_afs4int_lookuproot_rqst_cell_high", "afs4int.hf_afs4int_lookuproot_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookuproot_rqst_cell_low,
        {"hf_afs4int_lookuproot_rqst_cell_low", "afs4int.hf_afs4int_lookuproot_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookuproot_rqst_volume_high,
        {"hf_afs4int_lookuproot_rqst_volume_high", "afs4int.hf_afs4int_lookuproot_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookuproot_rqst_volume_low,
        {"hf_afs4int_lookuproot_rqst_volume_low", "afs4int.hf_afs4int_lookuproot_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookuproot_rqst_vnode,
        {"hf_afs4int_lookuproot_rqst_vnode", "afs4int.hf_afs4int_lookuproot_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookuproot_rqst_unique,
        {"hf_afs4int_lookuproot_rqst_unique", "afs4int.hf_afs4int_lookuproot_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchdata_rqst_cell_high,
        {"hf_afs4int_fetchdata_rqst_cell_high", "afs4int.hf_afs4int_fetchdata_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchdata_rqst_cell_low,
        {"hf_afs4int_fetchdata_rqst_cell_low", "afs4int.hf_afs4int_fetchdata_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchdata_rqst_volume_high,
        {"hf_afs4int_fetchdata_rqst_volume_high", "afs4int.hf_afs4int_fetchdata_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchdata_rqst_volume_low,
        {"hf_afs4int_fetchdata_rqst_volume_low", "afs4int.hf_afs4int_fetchdata_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchdata_rqst_vnode,
        {"hf_afs4int_fetchdata_rqst_vnode", "afs4int.hf_afs4int_fetchdata_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchdata_rqst_unique,
        {"hf_afs4int_fetchdata_rqst_unique", "afs4int.hf_afs4int_fetchdata_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchacl_rqst_cell_high,
        {"hf_afs4int_fetchacl_rqst_cell_high", "afs4int.hf_afs4int_fetchacl_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchacl_rqst_cell_low,
        {"hf_afs4int_fetchacl_rqst_cell_low", "afs4int.hf_afs4int_fetchacl_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchacl_rqst_volume_high,
        {"hf_afs4int_fetchacl_rqst_volume_high", "afs4int.hf_afs4int_fetchacl_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchacl_rqst_volume_low,
        {"hf_afs4int_fetchacl_rqst_volume_low", "afs4int.hf_afs4int_fetchacl_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchacl_rqst_vnode,
        {"hf_afs4int_fetchacl_rqst_vnode", "afs4int.hf_afs4int_fetchacl_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchacl_rqst_unique,
        {"hf_afs4int_fetchacl_rqst_unique", "afs4int.hf_afs4int_fetchacl_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_rqst_cell_high,
        {"hf_afs4int_fetchstatus_rqst_cell_high", "afs4int.hf_afs4int_fetchstatus_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_rqst_cell_low,
        {"hf_afs4int_fetchstatus_rqst_cell_low", "afs4int.hf_afs4int_fetchstatus_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_rqst_volume_high,
        {"hf_afs4int_fetchstatus_rqst_volume_high", "afs4int.hf_afs4int_fetchstatus_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_rqst_volume_low,
        {"hf_afs4int_fetchstatus_rqst_volume_low", "afs4int.hf_afs4int_fetchstatus_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_rqst_vnode,
        {"hf_afs4int_fetchstatus_rqst_vnode", "afs4int.hf_afs4int_fetchstatus_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_rqst_unique,
        {"hf_afs4int_fetchstatus_rqst_unique", "afs4int.hf_afs4int_fetchstatus_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_rqst_vvp_hi,
        {"hf_afs4int_fetchstatus_rqst_vvp_hi", "afs4int.hf_afs4int_fetchstatus_rqst_vvp_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_rqst_vvp_low,
        {"hf_afs4int_fetchstatus_rqst_vvp_low", "afs4int.hf_afs4int_fetchstatus_rqst_vvp_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_rqst_flags,
        {"hf_afs4int_fetchstatus_rqst_flags", "afs4int.hf_afs4int_fetchstatus_rqst_flags", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_rqst_error,
        {"hf_afs4int_fetchstatus_rqst_error", "afs4int.hf_afs4int_fetchstatus_rqst_error", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storeacl_rqst_cell_high,
        {"hf_afs4int_storeacl_rqst_cell_high", "afs4int.hf_afs4int_storeacl_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storeacl_rqst_cell_low,
        {"hf_afs4int_storeacl_rqst_cell_low", "afs4int.hf_afs4int_storeacl_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storeacl_rqst_volume_high,
        {"hf_afs4int_storeacl_rqst_volume_high", "afs4int.hf_afs4int_storeacl_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storeacl_rqst_volume_low,
        {"hf_afs4int_storeacl_rqst_volume_low", "afs4int.hf_afs4int_storeacl_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storeacl_rqst_vnode,
        {"hf_afs4int_storeacl_rqst_vnode", "afs4int.hf_afs4int_storeacl_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storeacl_rqst_unique,
        {"hf_afs4int_storeacl_rqst_unique", "afs4int.hf_afs4int_storeacl_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_symlink_rqst_cell_high,
        {"hf_afs4int_symlink_rqst_cell_high", "afs4int.hf_afs4int_symlink_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_symlink_rqst_cell_low,
        {"hf_afs4int_symlink_rqst_cell_low", "afs4int.hf_afs4int_symlink_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_symlink_rqst_volume_high,
        {"hf_afs4int_symlink_rqst_volume_high", "afs4int.hf_afs4int_symlink_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_symlink_rqst_volume_low,
        {"hf_afs4int_symlink_rqst_volume_low", "afs4int.hf_afs4int_symlink_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_symlink_rqst_vnode,
        {"hf_afs4int_symlink_rqst_vnode", "afs4int.hf_afs4int_symlink_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_symlink_rqst_unique,
        {"hf_afs4int_symlink_rqst_unique", "afs4int.hf_afs4int_symlink_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_readdir_rqst_cell_high,
        {"hf_afs4int_readdir_rqst_cell_high", "afs4int.hf_afs4int_readdir_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_readdir_rqst_cell_low,
        {"hf_afs4int_readdir_rqst_cell_low", "afs4int.hf_afs4int_readdir_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_readdir_rqst_volume_high,
        {"hf_afs4int_readdir_rqst_volume_high", "afs4int.hf_afs4int_readdir_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_readdir_rqst_volume_low,
        {"hf_afs4int_readdir_rqst_volume_low", "afs4int.hf_afs4int_readdir_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_readdir_rqst_vnode,
        {"hf_afs4int_readdir_rqst_vnode", "afs4int.hf_afs4int_readdir_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_readdir_rqst_unique,
        {"hf_afs4int_readdir_rqst_unique", "afs4int.hf_afs4int_readdir_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_readdir_rqst_offset_high,
        {"hf_afs4int_readdir_rqst_offset_high", "afs4int.hf_afs4int_readdir_rqst_offset_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_readdir_rqst_offset_low,
        {"hf_afs4int_readdir_rqst_offset_low", "afs4int.hf_afs4int_readdir_rqst_offset_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_readdir_rqst_size,
        {"hf_afs4int_readdir_rqst_size", "afs4int.hf_afs4int_readdir_rqst_size", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_readdir_rqst_vvp_high,
        {"hf_afs4int_readdir_rqst_vvp_high", "afs4int.hf_afs4int_readdir_rqst_vvp_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_readdir_rqst_vvp_low,
        {"hf_afs4int_readdir_rqst_vvp_low", "afs4int.hf_afs4int_readdir_rqst_vvp_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_readdir_rqst_flags,
        {"hf_afs4int_readdir_rqst_flags", "afs4int.hf_afs4int_readdir_rqst_flags", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_cell_high,
        {"hf_afs4int_removedir_rqst_cell_high", "afs4int.hf_afs4int_removedir_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_cell_low,
        {"hf_afs4int_removedir_rqst_cell_low", "afs4int.hf_afs4int_removedir_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_volume_high,
        {"hf_afs4int_removedir_rqst_volume_high", "afs4int.hf_afs4int_removedir_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_volume_low,
        {"hf_afs4int_removedir_rqst_volume_low", "afs4int.hf_afs4int_removedir_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_vnode,
        {"hf_afs4int_removedir_rqst_vnode", "afs4int.hf_afs4int_removedir_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_unique,
        {"hf_afs4int_removedir_rqst_unique", "afs4int.hf_afs4int_removedir_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_cell_high2,
        {"hf_afs4int_removedir_rqst_cell_high2", "afs4int.hf_afs4int_removedir_rqst_cell_high2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_cell_low2,
        {"hf_afs4int_removedir_rqst_cell_low2", "afs4int.hf_afs4int_removedir_rqst_cell_low2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_volume_high2,
        {"hf_afs4int_removedir_rqst_volume_high2", "afs4int.hf_afs4int_removedir_rqst_volume_high2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_volume_low2,
        {"hf_afs4int_removedir_rqst_volume_low2", "afs4int.hf_afs4int_removedir_rqst_volume_low2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_vnode2,
        {"hf_afs4int_removedir_rqst_vnode2", "afs4int.hf_afs4int_removedir_rqst_vnode2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_unique2,
        {"hf_afs4int_removedir_rqst_unique2", "afs4int.hf_afs4int_removedir_rqst_unique2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_tn,
        {"hf_afs4int_removedir_rqst_tn", "afs4int.hf_afs4int_removedir_rqst_tn", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_tn2,
        {"hf_afs4int_removedir_rqst_tn2", "afs4int.hf_afs4int_removedir_rqst_tn2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_key_size,
        {"hf_afs4int_removedir_rqst_key_size", "afs4int.hf_afs4int_removedir_rqst_key_size", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_removedir_rqst_key_t,
        {"hf_afs4int_removedir_rqst_key_t", "afs4int.hf_afs4int_removedir_rqst_key_t", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_cell_high,
        {"hf_afs4int_lookup_rqst_cell_high", "afs4int.hf_afs4int_lookup_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_resp_cell_high,
        {"hf_afs4int_lookup_resp_cell_high", "afs4int.hf_afs4int_lookup_resp_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_cell_low,
        {"hf_afs4int_lookup_rqst_cell_low", "afs4int.hf_afs4int_lookup_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_resp_cell_low,
        {"hf_afs4int_lookup_resp_cell_low", "afs4int.hf_afs4int_lookup_resp_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_volume_high,
        {"hf_afs4int_lookup_rqst_volume_high", "afs4int.hf_afs4int_lookup_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_resp_volume_high,
        {"hf_afs4int_lookup_resp_volume_high", "afs4int.hf_afs4int_lookup_resp_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_volume_low,
        {"hf_afs4int_lookup_rqst_volume_low", "afs4int.hf_afs4int_lookup_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_resp_volume_low,
        {"hf_afs4int_lookup_resp_volume_low", "afs4int.hf_afs4int_lookup_resp_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_vnode,
        {"hf_afs4int_lookup_rqst_vnode", "afs4int.hf_afs4int_lookup_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_resp_vnode,
        {"hf_afs4int_lookup_resp_vnode", "afs4int.hf_afs4int_lookup_resp_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_unique,
        {"hf_afs4int_lookup_rqst_unique", "afs4int.hf_afs4int_lookup_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_resp_unique,
        {"hf_afs4int_lookup_resp_unique", "afs4int.hf_afs4int_lookup_resp_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_tn_tag,
        {"hf_afs4int_lookup_rqst_tn_tag", "afs4int.hf_afs4int_lookup_rqst_tn_tag", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_tn_length,
        {"hf_afs4int_lookup_rqst_tn_length", "afs4int.hf_afs4int_lookup_rqst_tn_length", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_vvp_high,
        {"hf_afs4int_lookup_rqst_vvp_high", "afs4int.hf_afs4int_lookup_rqst_vvp_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_vvp_low,
        {"hf_afs4int_lookup_rqst_vvp_low", "afs4int.hf_afs4int_lookup_rqst_vvp_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_flags,
        {"hf_afs4int_lookup_rqst_flags", "afs4int.hf_afs4int_lookup_rqst_flags", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_key_size,
        {"hf_afs4int_lookup_rqst_key_size", "afs4int.hf_afs4int_lookup_rqst_key_size", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_lookup_rqst_key_t,
        {"hf_afs4int_lookup_rqst_key_t", "afs4int.hf_afs4int_lookup_rqst_key_t", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makemountpoint_rqst_cell_high,
        {"hf_afs4int_makemountpoint_rqst_cell_high", "afs4int.hf_afs4int_makemountpoint_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makemountpoint_rqst_cell_low,
        {"hf_afs4int_makemountpoint_rqst_cell_low", "afs4int.hf_afs4int_makemountpoint_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makemountpoint_rqst_volume_high,
        {"hf_afs4int_makemountpoint_rqst_volume_high", "afs4int.hf_afs4int_makemountpoint_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makemountpoint_rqst_volume_low,
        {"hf_afs4int_makemountpoint_rqst_volume_low", "afs4int.hf_afs4int_makemountpoint_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makemountpoint_rqst_vnode,
        {"hf_afs4int_makemountpoint_rqst_vnode", "afs4int.hf_afs4int_makemountpoint_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makemountpoint_rqst_unique,
        {"hf_afs4int_makemountpoint_rqst_unique", "afs4int.hf_afs4int_makemountpoint_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_cell_high,
        {"hf_afs4int_createfile_rqst_cell_high", "hf_afs4int_createfile_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_cell_low,
        {"hf_afs4int_createfile_rqst_cell_low", "hf_afs4int_createfile_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_volume_high,
        {"hf_afs4int_createfile_rqst_volume_high", "hf_afs4int_createfile_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_volume_low,
        {"hf_afs4int_createfile_rqst_volume_low", "hf_afs4int_createfile_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_vnode,
        {"hf_afs4int_createfile_rqst_vnode", "hf_afs4int_createfile_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_unique,
        {"hf_afs4int_createfile_rqst_unique", "hf_afs4int_createfile_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_cell_high2,
        {"hf_afs4int_createfile_rqst_cell_high2", "hf_afs4int_createfile_rqst_cell_high2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_cell_low2,
        {"hf_afs4int_createfile_rqst_cell_low2", "hf_afs4int_createfile_rqst_cell_low2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_volume_high2,
        {"hf_afs4int_createfile_rqst_volume_high2", "hf_afs4int_createfile_rqst_volume_high2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_volume_low2,
        {"hf_afs4int_createfile_rqst_volume_low2", "hf_afs4int_createfile_rqst_volume_low2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_vnode2,
        {"hf_afs4int_createfile_rqst_vnode2", "hf_afs4int_createfile_rqst_vnode2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_unique2,
        {"hf_afs4int_createfile_rqst_unique2", "hf_afs4int_createfile_rqst_unique2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_mask,
        {"hf_afs4int_createfile_rqst_mask", "hf_afs4int_createfile_rqst_mask", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_modtime_sec,
        {"hf_afs4int_createfile_rqst_modtime_sec", "hf_afs4int_createfile_rqst_modtime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_modtime_usec,
        {"hf_afs4int_createfile_rqst_modtime_usec", "hf_afs4int_createfile_rqst_modtime_usec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_accesstime_sec,
        {"hf_afs4int_createfile_rqst_accesstime_sec", "hf_afs4int_createfile_rqst_accesstime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_accesstime_usec,
        {"hf_afs4int_createfile_rqst_accesstime_usec", "hf_afs4int_createfile_rqst_accesstime_usec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_changetime_sec,
        {"hf_afs4int_createfile_rqst_changetime_sec", "hf_afs4int_createfile_rqst_changetime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_changetime_usec,
        {"hf_afs4int_createfile_rqst_changetime_usec", "hf_afs4int_createfile_rqst_changetime_usec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_owner,
        {"hf_afs4int_createfile_rqst_owner", "hf_afs4int_createfile_rqst_owner", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_group,
        {"hf_afs4int_createfile_rqst_group", "hf_afs4int_createfile_rqst_group", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_mode,
        {"hf_afs4int_createfile_rqst_mode", "hf_afs4int_createfile_rqst_mode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_trunc_high,
        {"hf_afs4int_createfile_rqst_trunc_high", "hf_afs4int_createfile_rqst_trunc_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_trunc_low,
        {"hf_afs4int_createfile_rqst_trunc_low", "hf_afs4int_createfile_rqst_trunc_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_length_high,
        {"hf_afs4int_createfile_rqst_length_high", "hf_afs4int_createfile_rqst_length_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_length_low,
        {"hf_afs4int_createfile_rqst_length_low", "hf_afs4int_createfile_rqst_length_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_devicetype,
        {"hf_afs4int_createfile_rqst_devicetype", "hf_afs4int_createfile_rqst_devicetype", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_devicenumber,
        {"hf_afs4int_createfile_rqst_devicenumber", "hf_afs4int_createfile_rqst_devicenumber", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_cmask,
        {"hf_afs4int_createfile_rqst_cmask", "hf_afs4int_createfile_rqst_cmask", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_clientspare1,
        {"hf_afs4int_createfile_rqst_clientspare1", "hf_afs4int_createfile_rqst_clientspare1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_devicenumberhighbits,
        {"hf_afs4int_createfile_rqst_devicenumberhighbits", "hf_afs4int_createfile_rqst_devicenumberhighbits", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_spare1,
        {"hf_afs4int_createfile_rqst_spare1", "hf_afs4int_createfile_rqst_spare1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_spare2,
        {"hf_afs4int_createfile_rqst_spare2", "hf_afs4int_createfile_rqst_spare2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_spare3,
        {"hf_afs4int_createfile_rqst_spare3", "hf_afs4int_createfile_rqst_spare3", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_spare4,
        {"hf_afs4int_createfile_rqst_spare4", "hf_afs4int_createfile_rqst_spare4", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_spare5,
        {"hf_afs4int_createfile_rqst_spare5", "hf_afs4int_createfile_rqst_spare5", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_spare6,
        {"hf_afs4int_createfile_rqst_spare6", "hf_afs4int_createfile_rqst_spare6", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_vvp_high,
        {"hf_afs4int_createfile_rqst_vvp_high", "hf_afs4int_createfile_rqst_vvp_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_vvp_low,
        {"vvp_low", "hf_afs4int_createfile_rqst_vvp_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_flags,
        {"Flags:", "hf_afs4int_createfile_rqst_flags", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_tn_tag,
        {"TN Tag:", "hf_afs4int_createfile_rqst_tn_tag", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_tn_length,
        {"Tn Length:", "hf_afs4int_createfile_rqst_tn_length", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_key_t,
        {"File name:", "hf_afs4int_createfile_rqst_key_t", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_key_size,
        {"Key Size:", "hf_afs4int_createfile_rqst_key_size", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_createfile_rqst_typeuuid,
        {"Type UUID:", "hf_afs4int_createfile_rqst_typeuuid", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_afs4int_makedir_rqst_cell_high,
        {"hf_afs4int_makedir_rqst_cell_high", "hf_afs4int_makedir_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_cell_low,
        {"hf_afs4int_makedir_rqst_cell_low", "hf_afs4int_makedir_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_volume_high,
        {"hf_afs4int_makedir_rqst_volume_high", "hf_afs4int_makedir_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_volume_low,
        {"hf_afs4int_makedir_rqst_volume_low", "hf_afs4int_makedir_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_vnode,
        {"hf_afs4int_makedir_rqst_vnode", "hf_afs4int_makedir_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_unique,
        {"hf_afs4int_makedir_rqst_unique", "hf_afs4int_makedir_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_cell_high2,
        {"hf_afs4int_makedir_rqst_cell_high2", "hf_afs4int_makedir_rqst_cell_high2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_cell_low2,
        {"hf_afs4int_makedir_rqst_cell_low2", "hf_afs4int_makedir_rqst_cell_low2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_volume_high2,
        {"hf_afs4int_makedir_rqst_volume_high2", "hf_afs4int_makedir_rqst_volume_high2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_volume_low2,
        {"hf_afs4int_makedir_rqst_volume_low2", "hf_afs4int_makedir_rqst_volume_low2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_vnode2,
        {"hf_afs4int_makedir_rqst_vnode2", "hf_afs4int_makedir_rqst_vnode2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_unique2,
        {"hf_afs4int_makedir_rqst_unique2", "hf_afs4int_makedir_rqst_unique2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_mask,
        {"hf_afs4int_makedir_rqst_mask", "hf_afs4int_makedir_rqst_mask", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_modtime_sec,
        {"hf_afs4int_makedir_rqst_modtime_sec", "hf_afs4int_makedir_rqst_modtime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_modtime_usec,
        {"hf_afs4int_makedir_rqst_modtime_usec", "hf_afs4int_makedir_rqst_modtime_usec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_accesstime_sec,
        {"hf_afs4int_makedir_rqst_accesstime_sec", "hf_afs4int_makedir_rqst_accesstime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_accesstime_usec,
        {"hf_afs4int_makedir_rqst_accesstime_usec", "hf_afs4int_makedir_rqst_accesstime_usec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_changetime_sec,
        {"hf_afs4int_makedir_rqst_changetime_sec", "hf_afs4int_makedir_rqst_changetime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_changetime_usec,
        {"hf_afs4int_makedir_rqst_changetime_usec", "hf_afs4int_makedir_rqst_changetime_usec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_owner,
        {"hf_afs4int_makedir_rqst_owner", "hf_afs4int_makedir_rqst_owner", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_group,
        {"hf_afs4int_makedir_rqst_group", "hf_afs4int_makedir_rqst_group", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_mode,
        {"hf_afs4int_makedir_rqst_mode", "hf_afs4int_makedir_rqst_mode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_trunc_high,
        {"hf_afs4int_makedir_rqst_trunc_high", "hf_afs4int_makedir_rqst_trunc_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_trunc_low,
        {"hf_afs4int_makedir_rqst_trunc_low", "hf_afs4int_makedir_rqst_trunc_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_length_high,
        {"hf_afs4int_makedir_rqst_length_high", "hf_afs4int_makedir_rqst_length_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_length_low,
        {"hf_afs4int_makedir_rqst_length_low", "hf_afs4int_makedir_rqst_length_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_devicetype,
        {"hf_afs4int_makedir_rqst_devicetype", "hf_afs4int_makedir_rqst_devicetype", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_devicenumber,
        {"hf_afs4int_makedir_rqst_devicenumber", "hf_afs4int_makedir_rqst_devicenumber", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_cmask,
        {"hf_afs4int_makedir_rqst_cmask", "hf_afs4int_makedir_rqst_cmask", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_clientspare1,
        {"hf_afs4int_makedir_rqst_clientspare1", "hf_afs4int_makedir_rqst_clientspare1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_devicenumberhighbits,
        {"hf_afs4int_makedir_rqst_devicenumberhighbits", "hf_afs4int_makedir_rqst_devicenumberhighbits", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_spare1,
        {"hf_afs4int_makedir_rqst_spare1", "hf_afs4int_makedir_rqst_spare1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_spare2,
        {"hf_afs4int_makedir_rqst_spare2", "hf_afs4int_makedir_rqst_spare2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_spare3,
        {"hf_afs4int_makedir_rqst_spare3", "hf_afs4int_makedir_rqst_spare3", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_spare4,
        {"hf_afs4int_makedir_rqst_spare4", "hf_afs4int_makedir_rqst_spare4", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_spare5,
        {"hf_afs4int_makedir_rqst_spare5", "hf_afs4int_makedir_rqst_spare5", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_spare6,
        {"hf_afs4int_makedir_rqst_spare6", "hf_afs4int_makedir_rqst_spare6", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_vvp_high,
        {"hf_afs4int_makedir_rqst_vvp_high", "hf_afs4int_makedir_rqst_vvp_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_vvp_low,
        {"vvp_low", "hf_afs4int_makedir_rqst_vvp_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_flags,
        {"Flags:", "hf_afs4int_makedir_rqst_flags", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_tn_tag,
        {"TN Tag:", "hf_afs4int_makedir_rqst_tn_tag", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_tn_length,
        {"Tn Length:", "hf_afs4int_makedir_rqst_tn_length", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_key_t,
        {"File name:", "hf_afs4int_makedir_rqst_key_t", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_key_size,
        {"Key Size:", "hf_afs4int_makedir_rqst_key_size", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_makedir_rqst_typeuuid,
        {"Type UUID:", "hf_afs4int_makedir_rqst_typeuuid", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_afs4int_storestatus_rqst_cell_high,
        {"hf_afs4int_storestatus_rqst_cell_high", "afs4int.hf_afs4int_storestatus_rqst_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_cell_low,
        {"hf_afs4int_storestatus_rqst_cell_low", "afs4int.hf_afs4int_storestatus_rqst_cell_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_volume_high,
        {"hf_afs4int_storestatus_rqst_volume_high", "afs4int.hf_afs4int_storestatus_rqst_volume_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_volume_low,
        {"hf_afs4int_storestatus_rqst_volume_low", "afs4int.hf_afs4int_storestatus_rqst_volume_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_vnode,
        {"hf_afs4int_storestatus_rqst_vnode", "afs4int.hf_afs4int_storestatus_rqst_vnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_unique,
        {"hf_afs4int_storestatus_rqst_unique", "afs4int.hf_afs4int_storestatus_rqst_unique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_mask,
        {"hf_afs4int_storestatus_rqst_mask", "hf_afs4int_storestatus_rqst_mask", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_modtime_sec,
        {"hf_afs4int_storestatus_rqst_modtime_sec", "hf_afs4int_storestatus_rqst_modtime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_modtime_usec,
        {"hf_afs4int_storestatus_rqst_modtime_usec", "hf_afs4int_storestatus_rqst_modtime_usec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_accesstime_sec,
        {"hf_afs4int_storestatus_rqst_accesstime_sec", "hf_afs4int_storestatus_rqst_accesstime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_accesstime_usec,
        {"hf_afs4int_storestatus_rqst_accesstime_usec", "hf_afs4int_storestatus_rqst_accesstime_usec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_changetime_sec,
        {"hf_afs4int_storestatus_rqst_changetime_sec", "hf_afs4int_storestatus_rqst_changetime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_changetime_usec,
        {"hf_afs4int_storestatus_rqst_changetime_usec", "hf_afs4int_storestatus_rqst_changetime_usec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_owner,
        {"hf_afs4int_storestatus_rqst_owner", "hf_afs4int_storestatus_rqst_owner", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_group,
        {"hf_afs4int_storestatus_rqst_group", "hf_afs4int_storestatus_rqst_group", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_mode,
        {"hf_afs4int_storestatus_rqst_mode", "hf_afs4int_storestatus_rqst_mode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_trunc_high,
        {"hf_afs4int_storestatus_rqst_trunc_high", "hf_afs4int_storestatus_rqst_trunc_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_trunc_low,
        {"hf_afs4int_storestatus_rqst_trunc_low", "hf_afs4int_storestatus_rqst_trunc_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_length_high,
        {"hf_afs4int_storestatus_rqst_length_high", "hf_afs4int_storestatus_rqst_length_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_length_low,
        {"hf_afs4int_storestatus_rqst_length_low", "hf_afs4int_storestatus_rqst_length_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_typeuuid,
        {"Type UUID:", "hf_afs4int_storestatus_rqst_typeuuid", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_afs4int_storestatus_rqst_devicetype,
        {"hf_afs4int_storestatus_rqst_devicetype", "hf_afs4int_storestatus_rqst_devicetype", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_devicenumber,
        {"hf_afs4int_storestatus_rqst_devicenumber", "hf_afs4int_storestatus_rqst_devicenumber", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_cmask,
        {"hf_afs4int_storestatus_rqst_cmask", "hf_afs4int_storestatus_rqst_cmask", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_clientspare1,
        {"hf_afs4int_storestatus_rqst_clientspare1", "hf_afs4int_storestatus_rqst_clientspare1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_devicenumberhighbits,
        {"hf_afs4int_storestatus_rqst_devicenumberhighbits", "hf_afs4int_storestatus_rqst_devicenumberhighbits", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_spare1,
        {"hf_afs4int_storestatus_rqst_spare1", "hf_afs4int_storestatus_rqst_spare1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_spare2,
        {"hf_afs4int_storestatus_rqst_spare2", "hf_afs4int_storestatus_rqst_spare2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_spare3,
        {"hf_afs4int_storestatus_rqst_spare3", "hf_afs4int_storestatus_rqst_spare3", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_spare4,
        {"hf_afs4int_storestatus_rqst_spare4", "hf_afs4int_storestatus_rqst_spare4", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_spare5,
        {"hf_afs4int_storestatus_rqst_spare5", "hf_afs4int_storestatus_rqst_spare5", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_spare6,
        {"hf_afs4int_storestatus_rqst_spare6", "hf_afs4int_storestatus_rqst_spare6", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_vvp_high,
        {"hf_afs4int_storestatus_rqst_vvp_high", "hf_afs4int_storestatus_rqst_vvp_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_vvp_low,
        {"vvp_low", "hf_afs4int_storestatus_rqst_vvp_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storestatus_rqst_flags,
        {"Flags:", "hf_afs4int_storestatus_rqst_flags", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storeacl_rqst_key_t,
        {"Key_t:", "hf_afs4int_storeacl_rqst_key_t", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storeacl_rqst_key_size,
        {"Key_Size:", "hf_afs4int_storeacl_rqst_key_size", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storeacl_rqst_var1,
        {"Var 1:", "hf_afs4int_storeacl_rqst_var1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_storeacl_rqst_var2,
        {"Var 2:", "hf_afs4int_storeacl_rqst_var2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_epochtime,
        {"EpochTime:", "hf_afs4int_setcontext_rqst_epochtime", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_afsnetaddr_type,
        {"AfsNetAddr_Type:", "hf_afs4int_setcontext_rqst_afsnetaddr_type", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_key_t,
        {"Key_t:", "hf_afs4int_setcontext_rqst_key_t", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_key_size,
        {"Key_Size:", "hf_afs4int_setcontext_rqst_key_size", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_flags,
        {"Flags:", "hf_afs4int_setcontext_rqst_flags", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_secobjectid,
        {"SetObjectid:", "hf_afs4int_setcontext_rqst_secobjectid", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_afs4int_setcontext_rqst_clientsizesattrs,
        {"ClientSizeAttrs:", "hf_afs4int_setcontext_rqst_clientsizesattrs", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_parm7,
        {"Parm7:", "hf_afs4int_setcontext_rqst_parm7", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data0,
        {"Data 0:", "hf_afs4int_setcontext_rqst_data0", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data1,
        {"Data 1:", "hf_afs4int_setcontext_rqst_data1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data2,
        {"Data 2:", "hf_afs4int_setcontext_rqst_data2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data3,
        {"Data 3:", "hf_afs4int_setcontext_rqst_data3", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data4,
        {"Data 4:", "hf_afs4int_setcontext_rqst_data4", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data5,
        {"Data 5:", "hf_afs4int_setcontext_rqst_data5", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data6,
        {"Data 6:", "hf_afs4int_setcontext_rqst_data6", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data7,
        {"Data 7:", "hf_afs4int_setcontext_rqst_data7", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data8,
        {"Data 8:", "hf_afs4int_setcontext_rqst_data8", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data9,
        {"Data 9:", "hf_afs4int_setcontext_rqst_data9", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data10,
        {"Data 10:", "hf_afs4int_setcontext_rqst_data10", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data11,
        {"Data 11:", "hf_afs4int_setcontext_rqst_data11", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data12,
        {"Data 12:", "hf_afs4int_setcontext_rqst_data12", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data13,
        {"Data 13:", "hf_afs4int_setcontext_rqst_data13", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_rqst_data14,
        {"Data 14:", "hf_afs4int_setcontext_rqst_data14", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_objectuuid,
        {"hf_afs4int_gettoken_resp_objectuuid:", "hf_afs4int_gettoken_resp_objectuuid", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_afs4int_gettoken_resp_typeuuid,
        {"hf_afs4int_gettoken_resp_typeuuid:", "hf_afs4int_gettoken_resp_typeuuid", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_afs4int_gettoken_resp_spare6,
        {"hf_afs4int_gettoken_resp_spare6:", "hf_afs4int_gettoken_resp_spare6", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_spare5,
        {"hf_afs4int_gettoken_resp_spare5:", "hf_afs4int_gettoken_resp_spare5", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_spare4,
        {"hf_afs4int_gettoken_resp_spare4:", "hf_afs4int_gettoken_resp_spare4", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_pathconfspare,
        {"hf_afs4int_gettoken_resp_pathconfspare:", "hf_afs4int_gettoken_resp_pathconfspare", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_lowmaxspare,
        {"hf_afs4int_gettoken_resp_lowmaxspare:", "hf_afs4int_gettoken_resp_lowmaxspare", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_himaxspare,
        {"hf_afs4int_gettoken_resp_himaxspare:", "hf_afs4int_gettoken_resp_himaxspare", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_agtypeunique,
        {"hf_afs4int_gettoken_resp_agtypeunique:", "hf_afs4int_gettoken_resp_agtypeunique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_devicenumberhighbits,
        {"hf_afs4int_gettoken_resp_devicenumberhighbits:", "hf_afs4int_gettoken_resp_devicenumberhighbits", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_clientspare1,
        {"hf_afs4int_gettoken_resp_clientspare1:", "hf_afs4int_gettoken_resp_clientspare1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_blocksused,
        {"hf_afs4int_gettoken_resp_blocksused:", "hf_afs4int_gettoken_resp_blocksused", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_devicenumber,
        {"hf_afs4int_gettoken_resp_devicenumber:", "hf_afs4int_gettoken_resp_devicenumber", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_servermodtime_msec,
        {"hf_afs4int_gettoken_resp_servermodtime_msec:", "hf_afs4int_gettoken_resp_servermodtime_msec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_servermodtime_sec,
        {"hf_afs4int_gettoken_resp_servermodtime_sec:", "hf_afs4int_gettoken_resp_servermodtime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_accesstime_msec,
        {"hf_afs4int_gettoken_resp_accesstime_msec:", "hf_afs4int_gettoken_resp_accesstime_msec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_accesstime_sec,
        {"hf_afs4int_gettoken_resp_accesstime_sec:", "hf_afs4int_gettoken_resp_accesstime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_changetime_msec,
        {"hf_afs4int_gettoken_resp_changetime_msec:", "hf_afs4int_gettoken_resp_changetime_msec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_changetime_sec,
        {"hf_afs4int_gettoken_resp_changetime_sec:", "hf_afs4int_gettoken_resp_changetime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_modtime_msec,
        {"hf_afs4int_gettoken_resp_modtime_msec:", "hf_afs4int_gettoken_resp_modtime_msec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_modtime_sec,
        {"hf_afs4int_gettoken_resp_modtime_sec:", "hf_afs4int_gettoken_resp_modtime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_parentunique,
        {"hf_afs4int_gettoken_resp_parentunique:", "hf_afs4int_gettoken_resp_parentunique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_parentvnode,
        {"hf_afs4int_gettoken_resp_parentvnode:", "hf_afs4int_gettoken_resp_parentvnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_mode,
        {"hf_afs4int_gettoken_resp_mode:", "hf_afs4int_gettoken_resp_mode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_aclexpirationtime,
        {"hf_afs4int_gettoken_resp_aclexpirationtime:", "hf_afs4int_gettoken_resp_aclexpirationtime", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_anonymouseaccess,
        {"hf_afs4int_gettoken_resp_anonymouseaccess:", "hf_afs4int_gettoken_resp_anonymouseaccess", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_calleraccess,
        {"hf_afs4int_gettoken_resp_calleraccess:", "hf_afs4int_gettoken_resp_calleraccess", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_group,
        {"hf_afs4int_gettoken_resp_group:", "hf_afs4int_gettoken_resp_group", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_owner,
        {"hf_afs4int_gettoken_resp_owner:", "hf_afs4int_gettoken_resp_owner", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_author,
        {"hf_afs4int_gettoken_resp_author:", "hf_afs4int_gettoken_resp_author", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_dataversion_low,
        {"hf_afs4int_gettoken_resp_dataversion_low:", "hf_afs4int_gettoken_resp_dataversion_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_dataversion_high,
        {"hf_afs4int_gettoken_resp_dataversion_high:", "hf_afs4int_gettoken_resp_dataversion_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_length_low,
        {"hf_afs4int_gettoken_resp_length_low:", "hf_afs4int_gettoken_resp_length_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_length_high,
        {"hf_afs4int_gettoken_resp_length_high:", "hf_afs4int_gettoken_resp_length_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_linkcount,
        {"hf_afs4int_gettoken_resp_linkcount:", "hf_afs4int_gettoken_resp_linkcount", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_filetype,
        {"hf_afs4int_gettoken_resp_filetype:", "hf_afs4int_gettoken_resp_filetype", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_interfaceversion,
        {"hf_afs4int_gettoken_resp_interfaceversion:", "hf_afs4int_gettoken_resp_interfaceversion", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_l_end_pos_ext,
        {"hf_afs4int_gettoken_resp_l_end_pos_ext:", "hf_afs4int_gettoken_resp_l_end_pos_ext", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_l_start_pos_ext,
        {"hf_afs4int_gettoken_resp_l_start_pos_ext:", "hf_afs4int_gettoken_resp_l_start_pos_ext", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_l_fstype,
        {"hf_afs4int_gettoken_resp_l_fstype:", "hf_afs4int_gettoken_resp_l_fstype", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_l_sysid,
        {"hf_afs4int_gettoken_resp_l_sysid:", "hf_afs4int_gettoken_resp_l_sysid", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_l_pid,
        {"hf_afs4int_gettoken_resp_l_pid:", "hf_afs4int_gettoken_resp_l_pid", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_l_end_pos,
        {"hf_afs4int_gettoken_resp_l_end_pos:", "hf_afs4int_gettoken_resp_l_end_pos", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_l_start_pos,
        {"hf_afs4int_gettoken_resp_l_start_pos:", "hf_afs4int_gettoken_resp_l_start_pos", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_l_whence,
        {"hf_afs4int_gettoken_resp_l_whence:", "hf_afs4int_gettoken_resp_l_whence", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_l_type,
        {"hf_afs4int_gettoken_resp_l_type:", "hf_afs4int_gettoken_resp_l_type", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_vvspare2,
        {"hf_afs4int_gettoken_resp_vvspare2:", "hf_afs4int_gettoken_resp_vvspare2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_vvspare1,
        {"hf_afs4int_gettoken_resp_vvspare1:", "hf_afs4int_gettoken_resp_vvspare1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_vvpingage,
        {"hf_afs4int_gettoken_resp_vvpingage:", "hf_afs4int_gettoken_resp_vvpingage", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_vvage,
        {"hf_afs4int_gettoken_resp_vvage:", "hf_afs4int_gettoken_resp_vvage", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_vv_low,
        {"hf_afs4int_gettoken_resp_vv_low:", "hf_afs4int_gettoken_resp_vv_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_vv_hi,
        {"hf_afs4int_gettoken_resp_vv_hi:", "hf_afs4int_gettoken_resp_vv_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_volid_low,
        {"hf_afs4int_gettoken_resp_volid_low:", "hf_afs4int_gettoken_resp_volid_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_volid_hi,
        {"hf_afs4int_gettoken_resp_volid_hi:", "hf_afs4int_gettoken_resp_volid_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_endrangeext,
        {"hf_afs4int_gettoken_resp_endrangeext:", "hf_afs4int_gettoken_resp_endrangeext", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_beginrangeext,
        {"hf_afs4int_gettoken_resp_beginrangeext:", "hf_afs4int_gettoken_resp_beginrangeext", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_endrange,
        {"hf_afs4int_gettoken_resp_endrange:", "hf_afs4int_gettoken_resp_endrange", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_beginrange,
        {"hf_afs4int_gettoken_resp_beginrange:", "hf_afs4int_gettoken_resp_beginrange", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_type_low,
        {"hf_afs4int_gettoken_resp_type_low:", "hf_afs4int_gettoken_resp_type_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_type_hi,
        {"hf_afs4int_gettoken_resp_type_hi:", "hf_afs4int_gettoken_resp_type_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_expirationtime,
        {"hf_afs4int_gettoken_resp_expirationtime:", "hf_afs4int_gettoken_resp_expirationtime", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_tokenid_low,
        {"hf_afs4int_gettoken_resp_tokenid_low:", "hf_afs4int_gettoken_tokenid_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_tokenid_hi,
        {"hf_afs4int_gettoken_resp_tokenid_hi:", "hf_afs4int_gettoken_resp_tokenid_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_anonymousaccess,
        {"hf_afs4int_gettoken_resp_anonymousaccess:", "hf_afs4int_gettoken_resp_anonymousaccess", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_lomaxspare,
        {"hf_afs4int_gettoken_resp_lomaxspare:", "hf_afs4int_gettoken_resp_lomaxspare", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_key_size,
        {"hf_afs4int_gettoken_resp_key_size:", "hf_afs4int_gettoken_resp_key_size", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_gettoken_resp_key_t,
        {"hf_afs4int_gettoken_resp_key_t:", "hf_afs4int_gettoken_resp_key_t", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_cell_high_old,
        {"hf_afs4int_rename_rqst_cell_high_old:", "hf_afs4int_rename_rqst_cell_high_old", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_cell_low_old,
        {"hf_afs4int_rename_rqst_cell_low_old:", "hf_afs4int_rename_rqst_cell_low_old", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_volume_high_old,
        {"hf_afs4int_rename_rqst_volume_high_old:", "hf_afs4int_rename_rqst_volume_high_old", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_volume_low_old,
        {"hf_afs4int_rename_rqst_volume_low_old:", "hf_afs4int_rename_rqst_volume_low_old", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_vnode_old,
        {"hf_afs4int_rename_rqst_vnode_old:", "hf_afs4int_rename_rqst_vnode_old", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_unique_old,
        {"hf_afs4int_rename_rqst_unique_old:", "hf_afs4int_rename_rqst_unique_old", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_tn_old,
        {"hf_afs4int_rename_rqst_tn_old:", "hf_afs4int_rename_rqst_tn_old", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_tn2_old,
        {"hf_afs4int_rename_rqst_tn2_old:", "hf_afs4int_rename_rqst_tn2_old", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_key_size_old,
        {"hf_afs4int_rename_rqst_key_size_old:", "hf_afs4int_rename_rqst_key_size_old", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_key_t_old,
        {"hf_afs4int_rename_rqst_key_t_old:", "hf_afs4int_rename_rqst_key_t_old", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_cell_high_new,
        {"hf_afs4int_rename_rqst_cell_high_new:", "hf_afs4int_rename_rqst_cell_high_new", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_cell_low_new,
        {"hf_afs4int_rename_rqst_cell_low_new:", "hf_afs4int_rename_rqst_cell_low_new", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_volume_high_new,
        {"hf_afs4int_rename_rqst_volume_high_new:", "hf_afs4int_rename_rqst_volume_high_new", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_volume_low_new,
        {"hf_afs4int_rename_rqst_volume_low_new:", "hf_afs4int_rename_rqst_volume_low_new", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_vnode_new,
        {"hf_afs4int_rename_rqst_vnode_new:", "hf_afs4int_rename_rqst_vnode_new", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_unique_new,
        {"hf_afs4int_rename_rqst_unique_new:", "hf_afs4int_rename_rqst_unique_new", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_tn_new,
        {"hf_afs4int_rename_rqst_tn_new:", "hf_afs4int_rename_rqst_tn_new", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_tn2_new,
        {"hf_afs4int_rename_rqst_tn2_new:", "hf_afs4int_rename_rqst_tn2_new", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_key_size_new,
        {"hf_afs4int_rename_rqst_key_size_new:", "hf_afs4int_rename_rqst_key_size_new", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_key_t_new,
        {"hf_afs4int_rename_rqst_key_t_new:", "hf_afs4int_rename_rqst_key_t_new", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_tokenidp_high,
        {"hf_afs4int_rename_rqst_tokenidp_high:", "hf_afs4int_rename_rqst_tokenidp_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_tokenidp_low,
        {"hf_afs4int_rename_rqst_tokenidp_low:", "hf_afs4int_rename_rqst_tokenidp_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_vvp_high,
        {"hf_afs4int_rename_rqst_vvp_high:", "hf_afs4int_rename_rqst_vvp_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_vvp_low,
        {"hf_afs4int_rename_rqst_vvp_low:", "hf_afs4int_rename_rqst_vvp_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_rename_rqst_flags,
        {"hf_afs4int_rename_rqst_flags:", "hf_afs4int_rename_rqst_flags", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_resp_var1,
        {"hf_afs4int_setcontext_resp_var1:", "hf_afs4int_setcontext_resp_var1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_setcontext_resp_var2,
        {"hf_afs4int_setcontext_resp_var2:", "hf_afs4int_setcontext_resp_var2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_interfaceversion,
{"hf_afs4int_fetchstatus_resp_interfaceversion", "hf_afs4int_fetchstatus_resp_interfaceversion", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_filetype,
{"hf_afs4int_fetchstatus_resp_filetype", "hf_afs4int_fetchstatus_resp_filetype", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_linkcount,
{"hf_afs4int_fetchstatus_resp_linkcount", "hf_afs4int_fetchstatus_resp_linkcount", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_length_high,
{"hf_afs4int_fetchstatus_resp_length_high", "hf_afs4int_fetchstatus_resp_length_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_length_low,
{"hf_afs4int_fetchstatus_resp_length_low", "hf_afs4int_fetchstatus_resp_length_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_dataversion_high,
{"hf_afs4int_fetchstatus_resp_dataversion_high", "hf_afs4int_fetchstatus_resp_dataversion_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_dataversion_low,
{"hf_afs4int_fetchstatus_resp_dataversion_low", "hf_afs4int_fetchstatus_resp_dataversion_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_author,
{"hf_afs4int_fetchstatus_resp_author", "hf_afs4int_fetchstatus_resp_author", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_owner,
{"hf_afs4int_fetchstatus_resp_owner", "hf_afs4int_fetchstatus_resp_owner", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_group,
{"hf_afs4int_fetchstatus_resp_group", "hf_afs4int_fetchstatus_resp_group", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_calleraccess,
{"hf_afs4int_fetchstatus_resp_calleraccess", "hf_afs4int_fetchstatus_resp_calleraccess", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_anonymousaccess,
{"hf_afs4int_fetchstatus_resp_anonymousaccess", "hf_afs4int_fetchstatus_resp_anonymousaccess", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_aclexpirationtime,
{"hf_afs4int_fetchstatus_resp_aclexpirationtime", "hf_afs4int_fetchstatus_resp_aclexpirationtime", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_mode,
{"hf_afs4int_fetchstatus_resp_mode", "hf_afs4int_fetchstatus_resp_mode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_parentvnode,
{"hf_afs4int_fetchstatus_resp_parentvnode", "hf_afs4int_fetchstatus_resp_parentvnode", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_parentunique,
{"hf_afs4int_fetchstatus_resp_parentunique", "hf_afs4int_fetchstatus_resp_parentunique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_modtime_sec,
{"hf_afs4int_fetchstatus_resp_modtime_sec", "hf_afs4int_fetchstatus_resp_modtime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_modtime_msec,
{"hf_afs4int_fetchstatus_resp_modtime_msec", "hf_afs4int_fetchstatus_resp_modtime_msec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_changetime_sec,
{"hf_afs4int_fetchstatus_resp_changetime_sec", "hf_afs4int_fetchstatus_resp_changetime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_changetime_msec,
{"hf_afs4int_fetchstatus_resp_changetime_msec", "hf_afs4int_fetchstatus_resp_changetime_msec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_accesstime_sec,
{"hf_afs4int_fetchstatus_resp_accesstime_sec", "hf_afs4int_fetchstatus_resp_accesstime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_accesstime_msec,
{"hf_afs4int_fetchstatus_resp_accesstime_msec", "hf_afs4int_fetchstatus_resp_accesstime_msec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_servermodtime_sec,
{"hf_afs4int_fetchstatus_resp_servermodtime_sec", "hf_afs4int_fetchstatus_resp_servermodtime_sec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_servermodtime_msec,
{"hf_afs4int_fetchstatus_resp_servermodtime_msec", "hf_afs4int_fetchstatus_resp_servermodtime_msec", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_typeuuid,
{"hf_afs4int_fetchstatus_resp_typeuuid", "hf_afs4int_fetchstatus_resp_typeuuid", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_afs4int_fetchstatus_resp_objectuuid,
{"hf_afs4int_fetchstatus_resp_objectuuid", "hf_afs4int_fetchstatus_resp_objectuuid", FT_STRING, BASE_NONE, NULL, 0x0, "UUID", HFILL }},
{ &hf_afs4int_fetchstatus_resp_devicenumber,
{"hf_afs4int_fetchstatus_resp_devicenumber", "hf_afs4int_fetchstatus_resp_devicenumber", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_blocksused,
{"hf_afs4int_fetchstatus_resp_blocksused", "hf_afs4int_fetchstatus_resp_blocksused", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_clientspare1,
{"hf_afs4int_fetchstatus_resp_clientspare1", "hf_afs4int_fetchstatus_resp_clientspare1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_devicenumberhighbits,
{"hf_afs4int_fetchstatus_resp_devicenumberhighbits", "hf_afs4int_fetchstatus_resp_devicenumberhighbits", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_agtypeunique,
{"hf_afs4int_fetchstatus_resp_agtypeunique", "hf_afs4int_fetchstatus_resp_agtypeunique", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_himaxspare,
{"hf_afs4int_fetchstatus_resp_himaxspare", "hf_afs4int_fetchstatus_resp_himaxspare", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_lomaxspare,
{"hf_afs4int_fetchstatus_resp_lomaxspare", "hf_afs4int_fetchstatus_resp_lomaxspare", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_pathconfspare,
{"hf_afs4int_fetchstatus_resp_pathconfspare", "hf_afs4int_fetchstatus_resp_pathconfspare", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_spare4,
{"hf_afs4int_fetchstatus_resp_spare4", "hf_afs4int_fetchstatus_resp_spare4", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_spare5,
{"hf_afs4int_fetchstatus_resp_spare5", "hf_afs4int_fetchstatus_resp_spare5", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_spare6,
{"hf_afs4int_fetchstatus_resp_spare6", "hf_afs4int_fetchstatus_resp_spare6", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_tokenid_hi,
{"hf_afs4int_fetchstatus_resp_tokenid_hi", "hf_afs4int_fetchstatus_resp_tokenid_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_tokenid_low,
{"hf_afs4int_fetchstatus_resp_tokenid_low", "hf_afs4int_fetchstatus_resp_tokenid_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_expirationtime,
{"hf_afs4int_fetchstatus_resp_expirationtime", "hf_afs4int_fetchstatus_resp_expirationtime", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_type_hi,
{"hf_afs4int_fetchstatus_resp_type_hi", "hf_afs4int_fetchstatus_resp_type_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_type_low,
{"hf_afs4int_fetchstatus_resp_type_low", "hf_afs4int_fetchstatus_resp_type_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_beginrange,
{"hf_afs4int_fetchstatus_resp_beginrange", "hf_afs4int_fetchstatus_resp_beginrange", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_endrange,
{"hf_afs4int_fetchstatus_resp_endrange", "hf_afs4int_fetchstatus_resp_endrange", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_beginrangeext,
{"hf_afs4int_fetchstatus_resp_beginrangeext", "hf_afs4int_fetchstatus_resp_beginrangeext", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_endrangeext,
{"hf_afs4int_fetchstatus_resp_endrangeext", "hf_afs4int_fetchstatus_resp_endrangeext", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_volid_hi,
{"hf_afs4int_fetchstatus_resp_volid_hi", "hf_afs4int_fetchstatus_resp_volid_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_volid_low,
{"hf_afs4int_fetchstatus_resp_volid_low", "hf_afs4int_fetchstatus_resp_volid_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_vv_hi,
{"hf_afs4int_fetchstatus_resp_vv_hi", "hf_afs4int_fetchstatus_resp_vv_hi", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_vv_low,
{"hf_afs4int_fetchstatus_resp_vv_low", "hf_afs4int_fetchstatus_resp_vv_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_vvage,
{"hf_afs4int_fetchstatus_resp_vvage", "hf_afs4int_fetchstatus_resp_vvage", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_vvpingage,
{"hf_afs4int_fetchstatus_resp_vvpingage", "hf_afs4int_fetchstatus_resp_vvpingage", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_vvspare1,
{"hf_afs4int_fetchstatus_resp_vvspare1", "hf_afs4int_fetchstatus_resp_vvspare1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_vvspare2,
{"hf_afs4int_fetchstatus_resp_vvspare2", "hf_afs4int_fetchstatus_resp_vvspare2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_err1,
{"hf_afs4int_fetchstatus_resp_err1", "hf_afs4int_fetchstatus_resp_err1", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
{ &hf_afs4int_fetchstatus_resp_err2,
{"hf_afs4int_fetchstatus_resp_err2", "hf_afs4int_fetchstatus_resp_err2", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
     };  


	static gint *ett[] = {
		&ett_AFS4Int,
	};
	proto_AFS4Int = proto_register_protocol ("DFS Calls", "DFS", "DFS");
	proto_register_field_array (proto_AFS4Int, hf, array_length (hf));
	proto_register_subtree_array (ett, array_length (ett));

}
void
proto_reg_handoff_AFS4Int (void)
{
	/* Register the protocol as dcerpc */
	dcerpc_init_uuid (proto_AFS4Int, ett_AFS4Int, &uuid_AFS4Int, ver_AFS4Int, AFS4Int_dissectors, -1);
}

